<?php

class Auth_GroupModel extends Unl_Model
{
	public static function find($id)
	{
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        } else if ($id === NULL) {
            return null;
        }
        
        $db = Zend_Registry::get('db');
        
        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqGroups'));
        if (Unl_Util::isArray($id)) {
            $select->where('g.groupId IN(?)', $id);
        } else {
            $select->where('g.groupId = ?', $id);
        }
        
        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }
        
        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects->pop();
        }
	}

    static public function findAll($includePrimary = true)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('g' => 'creqGroups'));
        $select->where('type = 1');
        if (!$includePrimary) {
        	$select->joinLeft(array('u' => 'creqUsers'), 'u.primaryGroup = g.groupId', array('userId'));
        	$select->where('ISNULL(u.userId)');
        }
        $records = $select->query()->fetchAll();
        $groupIds = array();
        foreach ($records as $record) {
            $groupIds[] = $record['groupId'];
        }
        
        return self::find($groupIds);
    }
	
    /**
     * Returns a list of groups that the user(s) is/are a member of
     *
     * @param Auth_UserModel|Unl_Model_Collection $user
     * @param bool[optional] $implicitMemberships
     * @return Unl_Model_Collection
     */
	static public function findByUser($user, $implicitMemberships = true)
	{
        if ((Unl_Util::isArray($user) && count($user) == 0) || $user === NULL) {
            return new Unl_Model_Collection(__CLASS__);
        }
        
		$db = Zend_Registry::get('db');
		$select = new Zend_Db_Select($db);

		// get userId as a string or array from the user object(s)
		if (Unl_Util::isArray($user)) {
			$userId = array();
			foreach ($user as $aUser) {
				$userId[] = $aUser->getId();
			}
		} else {
			$userId = $user->getId();
		}

		// find the group(s) that the user(s) is a "primary" member of
		$select = new Zend_Db_Select($db);
		$select->from(array('u' => 'creqUsers'), array('userId'));
		$select->join(array('g' => 'creqGroups'), 'u.primaryGroup = g.groupId');
		if (Unl_Util::isArray($userId)) {
			$select->where('u.userId IN (?)', $userId);
		} else {
			$select->where('u.userId = ?', $userId);
		}

		$records = $select->query()->fetchAll();
		$objects = array();
		if (Unl_Util::isArray($user)) {
		    foreach ($user as $aUser) {
		        $objects[$aUser->getId()] = new Unl_Model_Collection(__CLASS__);
		    }
		} else {
		    $objects[$user->getId()] = new Unl_Model_Collection(__CLASS__);
		}
		foreach ($records as $record) {
			$recordUserId = $record['userId'];
			unset($record['userId']);
			$objects[$recordUserId][] = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
		}

		// find any groups that the user(s) is a "non-primary" member of
		$select = new Zend_Db_Select($db);
		$select->from(array('u' => 'creqUsers'), array('userId'));
		$select->join(array('m' => 'creqGroupImpliedMemberships'), 'u.primaryGroup = m.childGroup', array());
		$select->join(array('g' => 'creqGroups'), 'm.parentGroup  = g.groupId');
        if (!$implicitMemberships) {
            $select->where("m.explicit = 'yes'");
        }
		if (Unl_Util::isArray($userId)) {
			$select->where('u.userId IN (?)', $userId);
		} else {
			$select->where('u.userId = ?', $userId);
		}

		$records = $select->query()->fetchAll();
		foreach ($records as $record) {
			$recordUserId = $record['userId'];
			unset($record['userId']);
			$objects[$recordUserId][] = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
		}

		// if we were passed an array of users, return an array for each user
		if (Unl_Util::isArray($userId)) {
			return $objects;
		// otherwise, return an array of groups for the single user we were passed
		} else {
			return $objects[$userId];
		}
	}

    /**
     * Returns a collection of the groups that the provided group(s) is/are a member of.
     *
     * @param Auth_GroupModel|Unl_Model_Collection $group
     * @param bool[optional] $impliedMemberships
     * @return Unl_Model_Collection
     */
    static public function findByChildGroup($group, $impliedMemberships = true)
    {
        if ((Unl_Util::isArray($group) && count($group) == 0) || $group === NULL) {
            return new Unl_Model_Collection(__CLASS__);
        }
        
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('m' => 'creqGroupImpliedMemberships'), array('childGroup', 'parentGroup'));
        $select->where('m.type = 1');
        if (!$impliedMemberships) {
            $select->where("m.explicit = 'yes'");
        }
        if (Unl_Util::isArray($group)) {
            if (count($group) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('childGroup IN(?)', $group->getId());
        } else {
            $select->where($db->quoteInto('childGroup = ?', $group->getId()));
        }
        
        $records = $select->query()->fetchAll();
        $parentGroupIds = array();
        foreach ($records as $record) {
            $parentGroupIds[] = $record['parentGroup'];
        }
        
        self::find($parentGroupIds);
    
        $parentGroups = array();
        if (Unl_Util::isArray($group)) {
            foreach ($group->getId() as $groupId) {
                $parentGroups[$groupId] = new Unl_Model_Collection(__CLASS__);
            }
        } else {
            $parentGroups[$group->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($records as $record) {
            $parentGroups[$record['childGroup']][$record['parentGroup']] = Unl_Model_Registry::getInstance()->get(__CLASS__, $record['parentGroup']);
        }
        
        if (Unl_Util::isArray($group)) {
            return $parentGroups;
        } else {
            return $parentGroups[$group->getId()];
        }
    }

    /**
     * Returns a collection of the groups that are members of the given group(s).
     *
     * @param Auth_GroupModel|Unl_Model_Collection $group
     * @param bool[optional] $impliedMemberships
     * @return Unl_Model_Collection
     */
    static public function findByParentGroup($group, $impliedMemberships = true, $primaryGroups = false)
    {
        if ((Unl_Util::isArray($id) && count($id) == 0) || $group === NULL) {
            return new Unl_Model_Collection(__CLASS__);
        }
        
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('m' => 'creqGroupImpliedMemberships'), array('childGroup', 'parentGroup'));
        $select->joinLeft(array('u' => 'creqUsers'), 'm.childGroup = u.primaryGroup', array());
        $select->where('m.type = 1');
        if (!$primaryGroups) {
            $select->where('u.userId IS NULL');
        }
        if (!$impliedMemberships) {
            $select->where("m.explicit = 'yes'");
        }
        if (Unl_Util::isArray($group)) {
            if (count($group) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('parentGroup IN(?)', $group->getId());
        } else {
            $select->where($db->quoteInto('parentGroup = ?', $group->getId()));
        }
        
        $records = $select->query()->fetchAll();
        $childGroupIds = array();
        foreach ($records as $record) {
            $childGroupIds[] = $record['childGroup'];
        }
        
        self::find($childGroupIds);
    
        $childGroups = array();
        if (Unl_Util::isArray($group)) {
            foreach ($group->getId() as $groupId) {
                $childGroups[$groupId] = new Unl_Model_Collection(__CLASS__);
            }
        } else {
            $childGroups[$group->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($records as $record) {
            $childGroups[$record['parentGroup']][$record['childGroup']] = Unl_Model_Registry::getInstance()->get(__CLASS__, $record['childGroup']);
        }
        
        if (Unl_Util::isArray($group)) {
            return $childGroups;
        } else {
            return $childGroups[$group->getId()];
        }
    }

    static public function fetchNew()
    {
        $data = array(
            'groupId'     => null,
            'type'        => 1,
            'name'        => null,
            'description' => null
        );

        $new = new self($data);
        return $new;
    }
    
    static public function save($models)
    {
        if (!Unl_Util::isArray($models)) {
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $models;
            $models = $collection;
        }

        $db = Zend_Registry::get('db');
        $db->beginTransaction();

        $unsavedData = array();
        foreach ($models as $id => $model) {
            $unsavedData[$id] = $model->_data;
        }

        try {

            $modelsToInsert = new Unl_Model_Collection(__CLASS__);
            $modelsToUpdate = new Unl_Model_Collection(__CLASS__);
            foreach ($models as $model) {
                if ($model->getId()) {
                    $modelsToUpdate[] = $model;
                } else {
                    $modelsToInsert[] = $model;
                }
            }
            self::_insert($modelsToInsert);
            self::_update($modelsToUpdate);

            $db->commit();
        } catch (Exception $e) {
            $db->rollback();
            foreach ($models as $id => $model) {
                $model->_data = $unsavedData[$id];
            }
            throw $e;
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        if (count($models) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqGroupsUpdate '
             . 'SELECT * FROM creqGroups LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqGroupsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['groupId'])
                        . $db->quoteInto('?, ' , $model->_data['type'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['description']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqGroups AS a, '
             . '       creqGroupsUpdate AS b '
             . 'SET a.name        = b.name, '
             . '    a.type        = b.type, '
             . '    a.description = b.description '
             . 'WHERE a.groupId = b.groupId ';
        $db->query($sql);
        $db->query('DROP TABLE creqGroupsUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        if (count($models) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');
    
        $sql = 'INSERT INTO creqGroups (type, name, description) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['type'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['description']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['groupId'] = $lastId;
            $lastId++;
        }
    }

    /**
     * Sets the groups that the supplied user is a member of.
     * The supplied array should have one key for each user (the primary key)
     * and a collection of parentGroups for each value
     *
     * @param array $groupMap
     */
    static public function setUserParentGroups($userGroupMap)
    {
    	$userIds = array();
    	foreach ($userGroupMap as $userId => $targetGroups) {
    		$userIds[] = $userId;
    	}
    	
    	$users = Auth_UserModel::find($userIds);
    	
    	$groupMap = array();
    	foreach ($userGroupMap as $userId => $targetGroups) {
    		$user = $users[$userId];
    		$primaryGroup = $user->getPrimaryGroup();
    		$groupMap[$primaryGroup] = $targetGroups;
    	}
    	
    	self::setGroupParentGroups($groupMap);
    }
    
    /**
     * Sets the groups that the supplied group is a member of.
     * The supplied array should have one key for each childGroup (the primary key)
     * and a collection of parentGroups for each value
     *
     * @param array $groupMap
     */
    static public function setGroupParentGroups($groupMap)
    {
        $childGroupIds = array();
        foreach ($groupMap as $childGroupId => $targetParentGroups) {
            $childGroupIds[] = $childGroupId;
        }
        
        $childGroups = self::find($childGroupIds);
        $childGroupParentGroups = self::findByChildGroup($childGroups, false);
        
        $membershipsToAdd = array();
        $membershipsToRemove = array();
        
        foreach ($childGroups as $childGroup) {
            $membershipsToAdd[$childGroup->getId()] = new Unl_Model_Collection(__CLASS__);
            $membershipsToRemove[$childGroup->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($groupMap as $childGroupId => $targetParentGroups) {
        	if (!$targetParentGroups) {
        		$targetParentGroups = new Unl_Model_Collection(__CLASS__);
        	}
            $currentParentGroups = $childGroupParentGroups[$childGroupId];
            foreach ($currentParentGroups as $currentParentGroup) {
                if (!in_array($currentParentGroup->getId(), $targetParentGroups->getId())) {
                    $membershipsToRemove[$childGroupId][] = $currentParentGroup;
                }
            }
            
            foreach ($targetParentGroups as $targetParentGroup) {
                if (!in_array($targetParentGroup->getId(), $currentParentGroups->getId())) {
                    $membershipsToAdd[$childGroupId][] = $targetParentGroup;
                }
            }
        }
        
        self::addGroupParentGroups($membershipsToAdd);
        self::removeGroupParentGroups($membershipsToRemove);
    }
    
    /**
     * Adds the supplied child group to additional parent groups.
     * The supplied array should have one key for each childGroup (the primary key)
     * and a collection of parentGroups for each value
     *
     * @param array $groupMap
     */
    static public function addGroupParentGroups($groupMap)
    {
        $membershipMap = array();
        foreach ($groupMap as $childGroupId => $newParentGroups) {
            foreach ($newParentGroups as $newParentGroup) {
                $membershipMap[] = array('childGroupId' => $childGroupId, 'parentGroupId' => $newParentGroup->getId());
            }
        }
    
        if (count($membershipMap) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');
    
        $sql = 'INSERT INTO creqGroupMemberships (type, parentGroup, childGroup) VALUES ';
        $sqlParts = array();
        foreach ($membershipMap as $membership) {
            $sqlParts[] = $db->quoteInto('(?, ', 1)
                        . $db->quoteInto('?, ' , $membership['parentGroupId'])
                        . $db->quoteInto('?)'  , $membership['childGroupId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
    
    /**
     * Removes the supplied child group from the respective parent groups.
     * The supplied array should have one key for each childGroup (the primary key)
     * and a collection of parentGroups for each value
     *
     * @param array $groupMap
     */
    static public function removeGroupParentGroups($groupMap)
    {
        $membershipMap = array();
        foreach ($groupMap as $childGroupId => $oldParentGroups) {
            foreach ($oldParentGroups as $oldParentGroup) {
                $membershipMap[] = array('childGroupId' => $childGroupId, 'parentGroupId' => $oldParentGroup->getId());
            }
        }
    
        if (count($membershipMap) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');
    
        $sql = 'DELETE FROM creqGroupMemberships WHERE ';
        $sqlParts = array();
        foreach ($membershipMap as $membership) {
            $sqlParts[] = $db->quoteInto('(type = 1 AND ')
                        . $db->quoteInto('parentGroup = ? AND ' , $membership['parentGroupId'])
                        . $db->quoteInto('childGroup = ?)'  , $membership['childGroupId']);
        }
        $sql .= implode(' OR ', $sqlParts);
        $db->query($sql);
    }
    
    

    /**
     * Sets the users that are members of the supplied group.
     * The supplied array should have one key for each parentGroup (the primary key)
     * and a collection of users for each value
     *
     * @param array $groupMap
     */
    static public function setGroupUserMembers($groupUserMap)
    {   
        $parentGroupIds = array();
        $primaryGroupIds = array();
        foreach ($groupUserMap as $parentGroupId => $targetUsers) {
            $parentGroupIds[] = $parentGroupId;
            foreach ($targetUsers as $targetUser) {
            	$primaryGroupIds[] = $targetUser->getPrimaryGroup();
            }
        }
        
        $parentGroups = self::find($parentGroupIds);
        $parentGroupUsers = Auth_UserModel::findByParentGroup($parentGroups, false);
        
        foreach ($parentGroupUsers as $aParentGroupUsers) {
        	foreach ($aParentGroupUsers as $parentGroupUser) {
                $primaryGroupIds[] = $parentGroupUser->getPrimaryGroup();
        	}
        }
        
        $primaryGroups = self::find($primaryGroupIds);
        
        $membershipsToAdd = array();
        $membershipsToRemove = array();
        
        foreach ($parentGroups as $parentGroup) {
            $membershipsToAdd[$parentGroup->getId()] = new Unl_Model_Collection(__CLASS__);
            $membershipsToRemove[$parentGroup->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($groupUserMap as $parentGroupId => $targetUsers) {
            $currentUsers = $parentGroupUsers[$parentGroupId];
            foreach ($currentUsers as $currentUser) {
                if (!in_array($currentUser->getPrimaryGroup(), $targetUsers->arrayFromMethod('getPrimaryGroup'))) {
                    $membershipsToRemove[$parentGroupId][] = $primaryGroups[$currentUser->getPrimaryGroup()];
                }
            }
            
            foreach ($targetUsers as $targetUser) {
                if (!in_array($targetUser->getPrimaryGroup(), $currentUsers->arrayFromMethod('getPrimaryGroup'))) {
                    $membershipsToAdd[$parentGroupId][] = $primaryGroups[$targetUser->getPrimaryGroup()];
                }
            }
        }
        
        self::addGroupChildGroups($membershipsToAdd);
        self::removeGroupChildGroups($membershipsToRemove);
    }
    
    /**
     * Sets the groups that are members of the given parentGroup.
     * The supplied array should have one key for each parentGroup (the primary key)
     * and a collection of childGroups for each value
     *
     * @param array $groupMap
     */
    static public function setGroupChildGroups($groupMap)
    {
        $parentGroupIds = array();
        foreach ($groupMap as $parentGroupId => $targetChildGroups) {
            $parentGroupIds[] = $parentGroupId;
        }
        
        $parentGroups = self::find($parentGroupIds);
        $parentGroupChildGroups = self::findByParentGroup($parentGroups, false);
        
        $membershipsToAdd = array();
        $membershipsToRemove = array();
        
        foreach ($parentGroups as $parentGroup) {
            $membershipsToAdd[$parentGroup->getId()] = new Unl_Model_Collection(__CLASS__);
            $membershipsToRemove[$parentGroup->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($groupMap as $parentGroupId => $targetChildGroups) {
            $currentChildGroups = $parentGroupChildGroups[$parentGroupId];
            foreach ($currentChildGroups as $currentChildGroup) {
                if (!in_array($currentChildGroup->getId(), $targetChildGroups->getId())) {
                    $membershipsToRemove[$parentGroupId][] = $currentChildGroup;
                }
            }
            
            foreach ($targetChildGroups as $targetChildGroup) {
                if (!in_array($targetChildGroup->getId(), $currentChildGroups->getId())) {
                    $membershipsToAdd[$parentGroupId][] = $targetChildGroup;
                }
            }
        }
        
        self::addGroupChildGroups($membershipsToAdd);
        self::removeGroupChildGroups($membershipsToRemove);
    }
    
    /**
     * Adds additional child groups to the supplied parentGroup.
     * The supplied array should have one key for each parentGroup (the primary key)
     * and a collection of childGroups for each value
     *
     * @param array $groupMap
     */
    static public function addGroupChildGroups($groupMap)
    {
        $membershipMap = array();
        foreach ($groupMap as $parentGroupId => $newChildGroups) {
            foreach ($newChildGroups as $newChildGroup) {
                $membershipMap[] = array('parentGroupId' => $parentGroupId, 'childGroupId' => $newChildGroup->getId());
            }
        }
    
        if (count($membershipMap) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');
    
        $sql = 'INSERT INTO creqGroupMemberships (type, parentGroup, childGroup) VALUES ';
        $sqlParts = array();
        foreach ($membershipMap as $membership) {
            $sqlParts[] = $db->quoteInto('(?, ', 1)
                        . $db->quoteInto('?, ' , $membership['parentGroupId'])
                        . $db->quoteInto('?)'  , $membership['childGroupId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
    
    /**
     * Removes the respective child groups from the supplied parent group.
     * The supplied array should have one key for each parentGroup (the primary key)
     * and a collection of childGroups for each value
     *
     * @param array $groupMap
     */
    static public function removeGroupChildGroups($groupMap)
    {
        $membershipMap = array();
        foreach ($groupMap as $parentGroupId => $oldChildGroups) {
            foreach ($oldChildGroups as $oldChildGroup) {
                $membershipMap[] = array('parentGroupId' => $parentGroupId, 'childGroupId' => $oldChildGroup->getId());
            }
        }
    
        if (count($membershipMap) == 0) {
            return;
        }
        
        $db = Zend_Registry::get('db');
    
        $sql = 'DELETE FROM creqGroupMemberships WHERE ';
        $sqlParts = array();
        foreach ($membershipMap as $membership) {
            $sqlParts[] = $db->quoteInto('(type = 1 AND ')
                        . $db->quoteInto('parentGroup = ? AND ' , $membership['parentGroupId'])
                        . $db->quoteInto('childGroup = ?)'  , $membership['childGroupId']);
        }
        $sql .= implode(' OR ', $sqlParts);
        $db->query($sql);
    }
    
	public function getId()
	{
	    return $this->_data['groupId'];
	}

    public function getName()
    {
        return $this->_data['name'];
    }
    public function setName($name)
    {
    	$this->_data['name'] = $name;
    }
    
    public function getDescription()
    {
        return $this->_data['description'];
    }
    public function setDescription($description)
    {
        $this->_data['description'] = $description;
    }
}
