<?php

class Bulletin_EditController extends Creq_Controller_Action
{
    public function indexAction()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $section = $session->section;
        $request = $session->request;

        $this->view->section = $section;
        $this->view->request = $request;
        $this->view->formSessionId = $in['id'];

        $this->view->title = Bulletin_UtilityModel::getTextBetweenTokens($session->fileContents, '<title>', '</title>');
        $this->view->body  = Bulletin_SectionModel::getBodyHtml($session->fileContents, '<body>', '</body>');
        foreach (Bulletin_SectionModel::parseHtml($session->fileContents, $section) as $key => $value) {
            $this->view->$key = $value;
        }
    }

    public function loadAction()
    {
        $in = $this->getAllParams();

        $tempId = hash('md5', openssl_random_pseudo_bytes(16));
        $session = new Zend_Session_Namespace('Edit Request ' . $tempId);

        if ($in['sessionId'] !== null) {
            $id = $in['sessionId'];
            $user = Auth_UserModel::findCurrentUser();
            $savedRequests = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests')->getData();
            $request = $savedRequests[$id]['request'];
            $section = $savedRequests[$id]['section'];
            $fileContents = $savedRequests[$id]['fileContents'];

            /*
            if ($this->_isSavedRequestStale($request, $section)) {
                $controllerSession = new Zend_Session_Namespace(__CLASS__);
                $controllerSession->errorMessage = 'Unfortunately another request modifying this course has been submitted since you saved your request.  You will need to create a new request to continue.';
                $this->redirect('/courses/edit/error');
                exit;
            }
            */
            $session->currentSavedRequestId = $id;
        } else {
            $id = $in['id'];
            $request = Requests_RequestModel::find($id);
            $section = Bulletin_SectionModel::findWithRequest($request);
            $section = clone $section;
            $fileContents = Bulletin_RepositoryModel::getInstance()->getFileNotes(
                'data/' . Bulletin_RepositoryModel::getInstance()->getFilePathForSection($section),
                'request-' . $request->getId()
            );

            unset($session->currentSavedRequestId);
        }

        $session->request = $request;
        $session->section = $section;
        $session->fileContents = $fileContents;

        $this->redirect('/bulletin/edit/index/id/' . $tempId);
    }

    public function editPostAction()
    {
        $in = $this->getAllParams();
        $uploads = new Zend_File_Transfer_Adapter_Http();

        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $session->fileContents = $this->_cleanUpContents(Bulletin_SectionModel::reconstructHtml($in, $session->section));

        if (isset($in['justification'])) {
            $session->request->setJustification($in['justification']);
        }
        foreach ($uploads->getFileInfo() as $file) {
            if ($file['error'] !== UPLOAD_ERR_OK) {
                continue;
            }
            $fileExtension = strtolower(array_pop(explode('.', $file['name'])));
            if (!in_array($fileExtension, array('rtf', 'pdf', 'odt', 'doc', 'docx'))) {
                continue;
            }
            $session->request->addFile(Requests_RequestModel::FILE_TYPE_OTHER, $file['name'], $file['type'], file_get_contents($file['tmp_name']));
        }
        foreach ($in['files'] as $fileId => $data) {
            if ($data['delete'] === 'yes') {
                $session->request->removeFileById($fileId);
            }
        }

        $this->redirect('/bulletin/view/index/type/session/id/' . $in['id']);
    }

    public function submitPostAction()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);

        if ($in['submitType'] == 'Submit') {
            set_time_limit(5);
            $this->_saveRequest();
            $this->_startApproval();
            $this->redirect('/requests');
        } else if ($in['submitType'] == 'Return to Edit') {
            $this->redirect('/bulletin/edit/index/id/' . $in['id']);
        } else if ($in['submitType'] == 'Save Changes') {
            $this->_saveRequest();
            $this->redirect('/bulletin/view/index/id/' . $session->request->getId());
        } else if ($in['submitType'] == 'Save for Later Submission') {
            $this->_saveForLater();
            $this->redirect('/requests');
        } else if ($in['submitType'] == 'Cancel Request' || $in['submitType'] == 'Abandon Changes') {
            $this->redirect('/requests');
        }
        exit;
    }

    protected function _saveRequest()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);

        if ($session->currentSavedRequestId !== null) {
            $id = $session->currentSavedRequestId;
            $user = Auth_UserModel::findCurrentUser();

            $savedRequestRegistry = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests');
            $savedRequests = $savedRequestRegistry->getData();
            unset($savedRequests[$id]);
            $savedRequestRegistry->setData($savedRequests);
            $collection = new Unl_Model_Collection('RegistryModel');
            $collection[] = $savedRequestRegistry;
            RegistryModel::save($collection);
            unset($session->currentSavedRequestId);
        }

        $session->fileContents = $this->_cleanUpContents($session->fileContents);

        Requests_RequestModel::save($session->request);
        $session->section->setRequest($session->request);
        Bulletin_SectionModel::save($session->section);

        Bulletin_RepositoryModel::getInstance()->saveFileContents(
            'data/' . Bulletin_RepositoryModel::getInstance()->getFilePathForSection($session->section),
            'request-' . $session->request->getId(),
            Auth_UserModel::findCurrentUser(),
            $this->_trimDiffHtml($session->fileContents),
            $session->request->getJustification()
        );

        Bulletin_RepositoryModel::getInstance()->saveFileNotes(
            'data/' . Bulletin_RepositoryModel::getInstance()->getFilePathForSection($session->section),
            'request-' . $session->request->getId(),
            $session->fileContents
        );
    }

    protected function _cleanUpContents($html)
    {
        // for testing
        //$pos = strpos($html,"\xA0");
        //if ($pos !== false) {
            //throw new Exception("Char A0 found: " . ord(substr($html,$pos-2,1)) . " " . ord(substr($html,$pos-1,1)) . " " . ord(substr($html,$pos,1)) . " " . ord(substr($html,$pos+1,1)) . " " . ord(substr($html,$pos+2,1)));
        //}

        // somehow hex A0 gets into content as space, need to convert to regular space (and a hex C2 creeps in too)
        $html = preg_replace("/\xA0/"," ",$html);
        $html = preg_replace("/\xC2/","",$html);

        return $html;
    }

    protected function _trimDiffHtml($html)
    {
        // remove <ins> tags and keep content (remove beginning tag and ending tag individually to handle nested <ins> tags)
        // old code, regex doesn't work with nested tags: $html = preg_replace('/<ins[^>]*>(.*?)<\/ins>/', '\1', $html);
        $html = preg_replace('/<ins[^>]*>/', '', $html);
        $html = preg_replace('/<\/ins>/', '', $html);

        // remove <del> tags and remove inner content (handles nested tags, little hairy)
        // old code, regex doesn't work with nested tags: $html = preg_replace('/<del[^>]*>(.*?)<\/del>/', '', $html);
        $html = $this->_removeTag($html, "del");

        // remove empty <p> tags
        $html = preg_replace('/<p[^>]*>\s*<\/p>/', '', $html);

        // replace ' & ' with '&amp;'
        $html = str_replace(' & ', ' &amp; ', $html);

        // replace '&AMP;' with '&amp;'
        $html = str_replace('&AMP;', '&amp;', $html);

        // replace CRLF with LF
        $html = str_replace((chr(13).chr(10)), chr(10), $html);

        return $html;
    }

    protected function _removeTag($html, $tag)
    {
        // remove specified tags and remove inner content (handles nested tags, little hairy)
        $is_valid = false;
        $bln_found = false;
        $i = 0;
        $start_begin_tag = -1;
        $end_begin_tag = -1;
        $start_end_tag = -1;
        $end_end_tag = -1;
        $no_inner_content = false;
        $str_len = strlen($html);
        $str_start = 0;
        while ($is_valid == false && $i < 10000)
        {
            $bln_found = false;

            // search for tag
            for ($k = $str_start; $k < $str_len; $k++)
            {
                if (substr($html,$k,(2 + strlen($tag))) == '<'.$tag.'>' || substr($html,$k,(2 + strlen($tag))) == '<'.$tag.' ' || substr($html,$k,(3 + strlen($tag))) == '<'.$tag.'/>')
                {
                    $bln_found = true;
                    $start_begin_tag = $k;

                    // found start of tag, look for end of beginning tag '>'
                    // if we hit another '<', ignore tag
                    for ($n = ($start_begin_tag + 1); $n < $str_len; $n++)
                    {
                        if (substr($html,$n,2) == '/>')
                        {
                            // found end of beginning tag
                            $end_begin_tag = $n + 1;
                            $no_inner_content = true;
                            $n = $str_len;
                        }
                        else if (substr($html,$n,1) == '>')
                        {
                            // found end of beginning tag
                            $end_begin_tag = $n;
                            $n = $str_len;
                        }
                        else if (substr($html,$n,1) == '<')
                        {
                            // new tag found before closing of previous tag, so it must not be a tag
                            // this assumes '<' are not found inside attribute values
                            // need to reset search to this spot
                            $str_start = $n;
                            $n = $str_len;
                        }
                    }

                    if ($start_begin_tag != -1 && $end_begin_tag != -1 && $no_inner_content == false)
                    {
                        // look for ending tag
                        for ($n = ($end_begin_tag + 1); $n < $str_len; $n++)
                        {
                            if (substr($html,$n,(3 + strlen($tag))) == '</'.$tag.'>')
                            {
                                // found ending tag
                                $start_end_tag = $n;
                                $end_end_tag = $n + 5;
                                $n = $str_len;
                            }
                            else if (substr($html,$n,(2 + strlen($tag))) == '<'.$tag.'>' || substr($html,$n,(2 + strlen($tag))) == '<'.$tag.' ' || substr($html,$n,(3 + strlen($tag))) == '<'.$tag.'/>')
                            {
                                // found a new tag, need to reset search to this one
                                $str_start = $n;
                                $n = $str_len;
                            }
                        }
                    }

                    if ($start_begin_tag != -1 && $end_begin_tag != -1 && (($start_end_tag != -1 && $end_end_tag != -1) || $no_inner_content == true))
                    {
                        // we have all the info we need to remove this tag
                        $html_new = "";
                        if ($start_begin_tag > 0)
                        {
                            // keep text before tag
                            $html_new .= substr($html,0,$start_begin_tag);
                        }
                        if ($no_inner_content == false)
                        {
                            if ($end_end_tag < ($str_len - 1))
                            {
                                // keep text after ending tag
                                $html_new .= substr($html,($end_end_tag + 1),($str_len - $end_end_tag - 1));
                            }
                        }
                        else
                        {
                            if ($end_begin_tag < ($str_len - 1))
                            {
                                // keep text after beginning tag ending '/>'
                                $html_new .= substr($html,($end_begin_tag + 1),($str_len - $end_begin_tag - 1));
                            }
                        }
                        $html = $html_new;

                        // reset search to beginning to check for more tags
                        $str_start = 0;
                    }

                    // reset variables
                    $start_begin_tag = -1;
                    $end_begin_tag = -1;
                    $start_end_tag = -1;
                    $end_end_tag = -1;
                    $no_inner_content = false;
                }
            }

            if ($bln_found == false)
            {
                $is_valid = true;
            }

            // use as a safety net in case of endless loop
            $i += 1;
        }

        return $html;
    }

    protected function _startApproval()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        Requests_ApprovalChainModel::consider($session->request);
    }

    protected function _saveForLater()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $user = Auth_UserModel::findCurrentUser();
        $savedRequestRegistry = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests');
        $savedRequests = $savedRequestRegistry->getData();

        if (!Unl_Util::isArray($savedRequests)) {
            $savedRequests = array();
        }
        $newSavedRequest = array(
            'request' => $session->request,
            'section' => $session->section,
            'fileContents' => $session->fileContents,
        );

        if ($session->currentSavedRequestId !== null) {
            $id = $session->currentSavedRequestId;
            $savedRequests[$id] = $newSavedRequest;
        } else {
            $savedRequests[] = $newSavedRequest;
        }

        $savedRequestRegistry->setData($savedRequests);
        $collection = new Unl_Model_Collection('RegistryModel');
        $collection[] = $savedRequestRegistry;
        RegistryModel::save($collection);
    }
}
