<?php

class Bulletin_SectionModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('r' => 'creqBulletinSections'));

        if (Unl_Util::isArray($id)) {
            if (count($id) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('bulletinSectionId IN(?)', $id);
        } else {
            $select->where('bulletinSectionId = ?', $id);
        }
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array, return an array.
        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects[$id];
        }
    }

    static public function findWithRequest($request)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $requestId = array();
            foreach ($request as $aRequest) {
                $requestId[] = $aRequest->getId();
            }
        } else {
            $requestId = $request->getId();
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('b' => 'creqBulletinSections'), array('bulletinSectionId', 'request'));
        if (Unl_Util::isArray($request)) {
            $select->where('b.request IN(?)', $requestId);
        } else {
            $select->where('b.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();

        $bulletinSectionIds = array();
        foreach ($records as $record) {
            $bulletinSectionIds[$record['request']] = $record['bulletinSectionId'];
        }

        self::find($bulletinSectionIds);

        if (Unl_Util::isArray($requestId)) {
            $rv = new Unl_Model_Collection(__CLASS__);
            foreach ($requestId as $rId) {
                $rv[$rId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $bulletinSectionIds[$rId]);
            }
            return $rv;
        } else {
            return Unl_Model_Registry::getInstance()->get(__CLASS__, $bulletinSectionIds[$requestId]);
        }
    }

    static public function findActiveRequestByCollegeAndMajorId($college, $major)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('b' => 'creqBulletinSections'), array());
        $select->join(array('c' => 'creqColleges'), 'b.collegeId = c.collegeId', array('college' => 'c.description'));
        $select->join(array('r' => 'creqRequests'), 'b.request = r.requestId', array('requestId'));
        $select->where('c.description = ?', $college);
        if ($major) {
            $select->where('b.majorId = ?', $major);
        } else {
            $select->where('b.majorId IS NULL');
        }
        $select->where('r.complete = ?', 'no');

        $records = $select->query()->fetchAll();
        $requestIds = array();
        foreach ($records as $record) {
            $requestIds[$record['requestId']] = $record['requestId'];
        }

        return Requests_RequestModel::find($requestIds);
    }

    static public function findAllRequestsByCollegeAndMajorId($college, $major, $completeApproveOnly = TRUE)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('b' => 'creqBulletinSections'), array());
        $select->join(array('c' => 'creqColleges'), 'b.collegeId = c.collegeId', array('college' => 'c.description'));
        $select->join(array('r' => 'creqRequests'), 'b.request = r.requestId', array('requestId'));
        $select->where('c.description = ?', $college);
        if ($major) {
            $select->where('b.majorId = ?', $major);
        } else {
            $select->where('b.majorId IS NULL');
        }
        if ($completeApproveOnly) {
            $select->where('r.complete = ?', 'yes');
            $select->where('r.state = ?', 'Approve');
        }

        $records = $select->query()->fetchAll();
        $requestIds = array();
        foreach ($records as $record) {
            $requestIds[$record['requestId']] = $record['requestId'];
        }

        return Requests_RequestModel::find($requestIds);
    }

    /**
     * Returns a new, empty model.
     *
     * @return Bulletin_SectionModel
     */
    static public function fetchNew()
    {
        $data = array(
            'bulletinSectionId' => NULL,
            'request'           => NULL,
            'collegeId'         => NULL,
            'majorId'           => NULL,
            'year'              => NULL,
        );

        $new = new self($data);
        $new->_setClean();

        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        self::_migrateCollegeAndMajorIds($models);

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _migrateCollegeAndMajorIds($models)
    {
        $colleges = Courses_CourseModel::getCollegeNames();
        $majors = Bulletin_UtilityModel::getAllMajors();

        foreach ($models as $model) {
            if (!$model->_data['collegeId'] && $model->_data['college']) {
                $model->_data['collegeId'] = array_search($model->_data['college'], $colleges);
            }
            if (!$model->_data['majorId'] && $model->_data['major']) {
                $model->_data['majorId'] = array_search($model->_data['major'], $majors);
            }
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqBulletinSectionsUpdate '
                . 'SELECT * FROM creqBulletinSections LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqBulletinSectionsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['bulletinSectionId'])
                        . $db->quoteInto('?, ' , $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['collegeId'])
                        . $db->quoteInto('?, ' , $model->_data['majorId'])
                        . $db->quoteInto('?) ' , $model->_data['year']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqBulletinSections AS a, '
             . '       creqBulletinSectionsUpdate AS b '
             . 'SET a.request   = b.request, '
             . '    a.collegeId = b.collegeId, '
             . '    a.majorId   = b.majorId, '
             . '    a.year      = b.year '
             . 'WHERE a.bulletinSectionId = b.bulletinSectionId ';
        $db->query($sql);
        $db->query('DROP TABLE creqBulletinSectionsUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqBulletinSections (`request`, `collegeId`, `majorId`, `year`) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['collegeId'])
                        . $db->quoteInto('?, ' , $model->_data['majorId'])
                        . $db->quoteInto('?)'  , $model->_data['year']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['bulletinSectionId'] = $lastId;
            $lastId++;
        }
    }

    static public function delete($models)
    {
        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');
        $sql = $db->quoteInto('DELETE FROM creqBulletinSections WHERE bulletinSectionId IN (?)', $models->getId());
        $db->query($sql);
    }

    public function __construct($data)
    {
        parent::__construct($data);
        $this->_data['data'] = unserialize(base64_decode($this->_data['data']));
    }

    public function getId()
    {
        return $this->_data['bulletinSectionId'];
    }

    public function getRequest()
    {
        return $this->_data['request'];
    }

    public function setRequest(Requests_RequestModel $request)
    {
        $this->_data['request'] = $request->getId();
    }

    public function clearRequest()
    {
        $this->_data['request'] = null;
    }

    public function getCollege()
    {
        if (!$this->_data['collegeId'] && $this->_data['college']) {
            return $this->_data['college'];
        }

        $colleges = Courses_CourseModel::getColleges();
        if (isset($colleges[$this->_data['collegeId']])) {
            return $colleges[$this->_data['collegeId']];
        }

        return NULL;
    }

    public function getCollegeLong()
    {
        if (!$this->_data['collegeId'] && $this->_data['college']) {
            return $this->_data['college'];
        }

        $colleges = Courses_CourseModel::getCollegeNames();
        if (isset($colleges[$this->_data['collegeId']])) {
            return $colleges[$this->_data['collegeId']];
        }

        return NULL;
    }

    public function setCollegeId($collegeId)
    {
        $colleges = Courses_CourseModel::getCollegeNames();
        if (!isset($colleges[$collegeId])) {
            throw new Exception('College not found.');
        }
        $this->_data['collegeId'] = $collegeId;
    }

    public function setCollege($college)
    {
        $colleges = Courses_CourseModel::getCollegeNames();
        $collegeId = array_search($college, $colleges);

        if (!$collegeId) {
            throw new Exception('The college "' . $college . '" does not exist.');
        }

        $this->_data['collegeId'] = $collegeId;
    }

    public function getMajor()
    {
        if (!$this->_data['majorId'] && $this->_data['major']) {
            return $this->_data['major'];
        }

        $majors = Bulletin_UtilityModel::getAllMajors();
        if (isset($majors[$this->_data['majorId']])) {
            return $majors[$this->_data['majorId']];
        }

        return NULL;
    }

    public function setMajorId($majorId)
    {
        $majors = Bulletin_UtilityModel::getAllMajors();
        if (!isset($majors[$majorId])) {
            throw new Exception('Major not found.');
        }
        $this->_data['majorId'] = $majorId;
        $this->_data['major'] = $majors[$majorId];
    }

    public function setMajor($major)
    {
        $majors = Bulletin_UtilityModel::getAllMajors();
        $majorId = array_search($major, $majors);

        if (!$majorId) {
            throw new Exception ('The major "' . $major . '" does not exist.');
        }

        $this->_data['majorId'] = $majorId;
        $this->_data['major'] = $majors[$majorId];
    }

    public function getYear()
    {
        return $this->_data['year'];
    }

    public function setYear($year)
    {
        $this->_data['year'] = $year;
    }

    public function hasQuickPoints()
    {
        return $this->getCollege() != 'OUP' && $this->getMajor();
    }

    static public $collegeSectionNames = array(
        'description'                            => 'Description',
        'admission'                              => 'Admission',
        'advising'                               => 'Advising',
        'academicProgramsAndPolicies'            => 'Academic Programs &amp; Policies',
        'aceRequirements'                        => 'ACE Requirements',
        'bulletinToUse'                          => 'Bulletin To Use',
        'collegeDegreeRequirements'              => 'College Degree Requirements',
        'degreesAndMajors'                       => 'Degrees &amp; Majors',
        'minorsAndAreasOfSpecializationsOffered' => 'Minors &amp; Areas of Specializations Offered',
        'graduateStudentInformation'             => 'Graduate Student Information',
    );

    static public $departmentSectionNames = array(
        'default' => array(
            'description'                             => 'Description',
            'majorDepartmentAdmission'                => 'Major Department Admission',
            'majorRequirements'                       => 'Major Requirements',
            'additionalMajorRequirements'             => 'Additional Major Requirements',
            'requirementsForMinorOfferedByDepartment' => 'Requirements For Minor Offered By Department',
        ),
        'Architecture' => array(
            'description'                             => 'Description',
            'admission'                               => 'Admission',
            'majorRequirements'                       => 'Major Requirements',
            'additionalMajorRequirements'             => 'Additional Major Requirements',
            'requirementsForMinorOfferedByDepartment' => 'Requirements For Minor Offered By Department',
        ),
        'Business Administration' => array(
            'description'                             => 'Description',
            'admission'                               => 'Admission',
            'majorRequirements'                       => 'Major Requirements',
            'additionalMajorRequirements'             => 'Additional Major Requirements',
            'requirementsForMinorOfferedByDepartment' => 'Requirements For Minor Offered By Department',
        ),
        'Education & Human Sciences' => array(
            'description'                             => 'Description',
            'admission'                               => 'Admission',
            'majorRequirements'                       => 'Major Requirements',
            'additionalMajorEndorsementRequirements'  => 'Additional Major/Endorsement Requirements',
            'requirementsForMinorOfferedByDepartment' => 'Requirements For Minor Offered By Department',
        ),
        'Public Affairs & Community Service' => array(
            'description'                             => 'Description',
            'admission'                               => 'Admission',
            'majorRequirements'                       => 'Major Requirements',
            'additionalMajorRequirements'             => 'Additional Major Requirements',
            'requirementsForMinorOfferedByDepartment' => 'Requirements For Minor Offered By Department',
        ),
        'Other University Programs' => array(
            'Academic Policies & Procedures' => array(
                'creditHourDefinition'                                 => 'Credit Hour Definition',
                'academicAdviserAssignment'                            => 'Academic Adviser Assignment',
                'studentsResponsibilitiesInAcademicAdvising'           => 'Students Responsibilities in Academic Advising',
                'maximumCreditHourLoad'                                => 'Maximum Credit Hour Load',
                'classAttendance'                                      => 'Class Attendance',
                'glossaryOfAcademicAndAdvisingTerms'                   => 'Glossary of Academic and Advising Terms',
                'subjectAreaAbbreviations'                             => 'Subject Area Abbreviations',
                'coursePrerequisitesAndCorequisites'                   => 'Course Prerequisites and Corequisites',
                'gradingSystem'                                        => 'Grading System',
                'academicStandardsInstitutionalPolicy'                 => 'Academic Standards-Institutional Policy',
                'honorsConvocationRecognitionRequirements'             => 'Honors Convocation Recognition Requirements',
                'recognitionOfOutstandingAcademicAchievement'          => 'Recognition of Outstanding Academic Achievement',
                'gradingPolicies'                                      => 'Grading Policies',
            ),
            'Admissions' => array(
                'assuredAdmission' => 'Assured Admission',
                'deficiencies'     => 'Deficiencies',
            ),
            'Aerospace Studies/Air Force ROTC' => array(
                'description'              => 'Description',
                'eligibilityAndEnrollment' => 'Eligibility and Enrollment',
                'programs'                 => 'Programs',
                'scholarshipsBenefits'     => 'Scholarships/Benefits',
                'curriculum'               => 'Curriculum',
                'creditHours'              => 'Credit Hours',
                'militaryObligation'       => 'Military Obligation',
                'fieldTrainingProgram'     => 'Field Training Program',
            ),
            'General Education Requirements (ACE)' => array(
                'generalEducationRequirementsAce'                      => 'General Education Requirements (ACE)',
                'aceInstitutionalObjectivesAndStudentLearningOutcomes' => 'ACE Institutional Objectives and Student Learning Outcomes',
                'aceStructuralCriteria'                                => 'ACE Structural Criteria',
            ),
            'Graduate Studies' => array(
                'admissionOfUnlSeniors' => 'Admission of UNL Seniors',
                'graduateCoursesTakenByUnlSeniors' => 'Graduate Courses Taken by UNL Seniors',
            ),
            'Jeffrey S. Raikes School of Computer Science & Management' => array(
                'description' => 'Description',
            ),
            'Military Science/Army ROTC' => array(
                'description'               => 'Description',
                'objectives'                => 'Objectives',
                'benefits'                  => 'Benefits',
                'participationInArmyRotc'   => 'Participation in Army ROTC',
                'curriculum'                => 'Curriculum',
                'commissioningRequirements' => 'Commissioning Requirements',
            ),
            'Naval Science/Naval ROTC' => array(
                'description'                                  => 'Description',
                'benefits'                                     => 'Benefits',
                'curriculum'                                   => 'Curriculum',
                'navalScienceOpenCurriculum'                   => 'Naval Science Open Curriculum',
                'individualizedProgramOfStudiesInNavalScience' => 'Individualized Program of Studies in Naval Science',
            ),
            'Pre-Professional Studies' => array(
                'description' => 'Description',
            ),
            'The Libraries' => array(
                'description'          => 'Description',
                'coursesOfInstruction' => 'Courses of Instruction',
            ),
            'Transfer' => array(
                'undergraduateTransferCreditPolicy'                                          => 'Undergraduate Transfer Credit Policy',
                'transferCreditPractices'                                                    => 'Transfer Credit Practices',
                'aCreditPresentedForTransferFromWithinTheUniversityOfNebraskaSystem'         => 'A. Credit presented for transfer from within the University of Nebraska System',
                'bInterInstitutionalAgreements'                                              => 'B. Inter-institutional Agreements',
                'cCreditPresentedForTransferWhichRequiresAdditionalReviewByTheDegreeCollege' => 'C. Credit presented for transfer which requires additional review by the degree college',
                'specialSituationsQualificationsAndLimitationsRelatedToTransferOfCredit'     => 'Special Situations, Qualifications, and Limitations related to Transfer of Credit',
            ),
            'University Honors Program' => array(
                'description' => 'Description',
            ),
        ),
    );

    static protected $_sectionHeaderTag = '<p class="content-box-h-1">';

    static public function parseHtml($html, self $section)
    {
        $html = strtr($html, array(" xml:lang='en-us'" => ''));
        $html = strtr($html, array('content-box-m-p' => 'content-box-h-1'));
        $html = mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8');
        $data = array('other' => '');

        $dom = new DOMDocument();
        $dom->loadHTML($html);
        foreach ($dom->getElementsByTagName('p') as $node) {
            if ($node->getAttribute('class') != 'quick-points') {
                continue;
            }
            $key = $node->textContent;
            $key = substr($key, 0, strpos($key, ':'));
            $key = strtolower($key);
            $key = ucwords($key);
            $key = strtr($key, array(' ' => ''));
            $key[0] = strtolower($key[0]);

            $keyMap = array(
                'adviser'        => 'chiefAdviser',
                'chiefAdvisers'  => 'chiefAdviser',
                'degree'         => 'degreeOffered',
                'degreesOffered' => 'degreeOffered',
            );
            if (isset($keyMap[$key])) {
                $key = $keyMap[$key];
            }

            $value = $node->textContent;
            $value = substr($value, strpos($value, ':') + 1);
            $value = trim($value);

            $data[$key] = $value;
        }

        $data['departmentSectionNames'] = self::getDepartmentSectionNames($section->getCollegeLong(), $section->getMajor());

        $segments = explode(self::$_sectionHeaderTag, $html);
        array_shift($segments);
        $lastSegment = array_pop($segments);
        $lastSegment = substr($lastSegment, 0, strpos($lastSegment, '</body>'));
        $segments[] = $lastSegment;

        foreach ($segments as $segment) {
            $key = $segment;
            $key = substr($key, 0, strpos($key, '</p>'));
            $key = strtr($key, array(
                '<span class="generated-style">' => '',
                '</span>' => '',
            ));
            $key = strtolower($key);
            $key = strtr($key, array('&amp;' => 'and'));
            $key = preg_replace('/\W/', ' ', $key);
            $key = ucwords($key);
            $key = strtr($key, array(' ' => ''));
            $key = preg_replace('/\W/', '', $key);
            $key[0] = strtolower($key[0]);
            $key = strtr($key, array(
                'majorEndorsementRequirements' => 'majorRequirements'
            ));

            if (
                !$section->getMajor() && in_array($key, array_keys(self::$collegeSectionNames)) ||
                $section->getMajor()  && in_array($key, array_keys($data['departmentSectionNames']))
            ) {
                $value = $segment;
                $value = substr($value, strpos($value, '</p>') + 4);
                $value = trim($value);
                $data[$key] = $value;
            } else {
                $data['other'] = trim($data['other'] . PHP_EOL . self::$_sectionHeaderTag . $segment);
            }
        }

        return $data;
    }

    static public function reconstructHtml($data, self $section)
    {
        if ($section->getMajor()) {
            $title = $section->getMajor() . ' ' . substr($section->getYear(), -2) . '-' . substr($section->getYear() + 1, -2);
        } else {
            $title = $section->getCollegeLong() . ' Main Page ' . substr($section->getYear(), -2) . '-' . substr($section->getYear() + 1, -2);
        }

        $hyphenatedTitle = preg_replace('/[^a-zA-Z0-9]+/', '-', strtolower($title));
        $majorName = $section->getMajor() ? $section->getMajor() : $section->getCollegeLong();

        $html = <<<EOF
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
    <head>
        <title>{$title}.xhtml</title>
        <link href="template.css" rel="stylesheet" type="text/css" />
    </head>
    <body>
        <div id="{$hyphenatedTitle}">
            <div class="generated-style">
                <p class="major-name">{$majorName}</p>
            </div>
            <div class="generated-style">
EOF;
        $htmlData = $data;
        $htmlData['college'] = $section->getCollegeLong();
        $htmlData['major'] = $section->getMajor();
        $htmlData = array_map('htmlentities', $htmlData);
        if ($section->hasQuickPoints()) {
            $html .= <<<EOF

                <p class="quick-points"><span class="quick-point-bold">COLLEGE:</span> {$htmlData['college']}</p>
                <p class="quick-points"><span class="quick-point-bold">MAJOR:</span> {$htmlData['major']}</p>
                <p class="quick-points"><span class="quick-point-bold">DEGREE OFFERED:</span> {$htmlData['degreeOffered']}</p>
                <p class="quick-points"><span class="quick-point-bold">HOURS REQUIRED:</span> {$htmlData['hoursRequired']}</p>
                <p class="quick-points"><span class="quick-point-bold">MINIMUM CUMULATIVE GPA:</span> {$htmlData['minimumCumulativeGpa']}</p>
                <p class="quick-points"><span class="quick-point-bold">MINOR AVAILABLE:</span> {$htmlData['minorAvailable']}</p>
                <p class="quick-points"><span class="quick-point-bold">CHIEF ADVISER:</span> {$htmlData['chiefAdviser']}</p>
EOF;
        }

        foreach (($section->getMajor() ? self::getDepartmentSectionNames($section->getCollegeLong(), $section->getMajor()) : self::$collegeSectionNames) as $machineName => $humanName) {
            if (!$data[$machineName]) {
                continue;
            }
            $html .= PHP_EOL . self::$_sectionHeaderTag . strtoupper($humanName) . '</p>';
            $html .= PHP_EOL . $data[$machineName];
        }

        if (isset($data['other'])) {
            $html .= PHP_EOL . $data['other'];
        }

        $html .= <<<EOF
            </div>
        </div>
    </body>
</html>
EOF;
        if (!$section->getMajor()) {
            $html = strtr($html, array('content-box-h-1' => 'content-box-m-p'));
        }
        return $html;
    }

    static public function getBodyHtml($html)
    {
        $html = Bulletin_UtilityModel::getTextBetweenTokens($html, '<body>', '</body>');
        $html = substr($html, strpos($html, '<div class="generated-style">') + strlen('<div class="generated-style">'));
        $html = substr($html, strpos($html, '<div class="generated-style">') + strlen('<div class="generated-style">'));
        $html = substr($html, 0, strrpos($html, '</div>'));
        $html = substr($html, 0, strrpos($html, '</div>'));
        $html = trim($html);
        return $html;
    }

    static public function getDepartmentSectionNames($college, $major)
    {
        $departmentSectionNames = self::$departmentSectionNames['default'];
        if (isset(self::$departmentSectionNames[$college])) {
            $departmentSectionNames = self::$departmentSectionNames[$college];
            if (isset(self::$departmentSectionNames[$college][$major])) {
                $departmentSectionNames = self::$departmentSectionNames[$college][$major];
            }
        }
        return $departmentSectionNames;
    }

    static public function updateCourseCode($oldCode, $newCode)
    {
        if ($oldCode == $newCode) {
            return;
        }

        $colleges = Bulletin_UtilityModel::getColleges();
        $collegeMajors = Bulletin_UtilityModel::getMajors();
        $majors = Bulletin_UtilityModel::getAllMajors();
        $year = 2014;

        $requestsToConsider = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($colleges as $collegeId => $college) {
            set_time_limit(max(5, ini_get('max_execution_time')));
            $requests = Bulletin_SectionModel::findActiveRequestByCollegeAndMajorId($college, NULL);
            if (count($requests)) {
                $request = $requests->pop();
                $section = Bulletin_SectionModel::findWithRequest($request);
            } else {
                $request = Requests_RequestModel::fetchNew();
                $section = Bulletin_SectionModel::fetchNew();
                $section->setYear($year);
                $section->setCollegeId($collegeId);
            }

            $result = self::_updateCourseCode($oldCode, $newCode, $request, $section);
            if ($result instanceof Requests_RequestModel) {
                $requestsToConsider[$request->getId()] = $result;
            }

            foreach ($collegeMajors[$college] as $majorId => $major) {
                $requests = Bulletin_SectionModel::findActiveRequestByCollegeAndMajorId($college, $majorId);
                if (count($requests)) {
                    $request = $requests->pop();
                    $section = Bulletin_SectionModel::findWithRequest($request);
                } else {
                    $request = Requests_RequestModel::fetchNew();
                    $section = Bulletin_SectionModel::fetchNew();
                    $section->setYear($year);
                    $section->setCollegeId($collegeId);
                    $section->setMajorId($majorId);
                }

                $result = self::_updateCourseCode($oldCode, $newCode, $request, $section);
                if ($result instanceof Requests_RequestModel) {
                    $requestsToConsider[$request->getId()] = $result;
                }
            }
        }

        Requests_ApprovalChainModel::consider($requestsToConsider);
    }

    static protected function _updateCourseCode($oldCode, $newCode, Requests_RequestModel $request, Bulletin_SectionModel $section)
    {
        $path = Bulletin_RepositoryModel::getInstance()->getFilePathForCollegeMajorYear(
            $section->getCollegeLong(),
            $section->getMajor(),
            $section->getYear()
        );

        $html = Bulletin_RepositoryModel::getInstance()->getFileContents(
            'data/' . $path,
            $request->getId() ? 'request-' . $request->getId() : 'master'
        );
        $courses = UNL_UndergraduateBulletin_EPUB_Utilities::findCourses($html);
        if (
            !isset($courses[$oldCode['subject']]) ||
            !in_array($oldCode['courseNumber'] . $oldCode['courseLetter'], $courses[$oldCode['subject']])
        ) {
            return;
        }

        if ($oldCode['subject'] == $newCode['subject']) {
            $diffHtml = UNL_UndergraduateBulletin_EPUB_Utilities::updateCourseNumber(
                $html,
                $oldCode['subject'],
                $oldCode['courseNumber'] . $oldCode['courseLetter'],
                '<del>' . $oldCode['courseNumber'] . $oldCode['courseLetter'] . '</del>' .
                '<ins>' . $newCode['courseNumber'] . $newCode['courseLetter'] . '</ins>'
            );
            $html = UNL_UndergraduateBulletin_EPUB_Utilities::updateCourseNumber(
                $html,
                $oldCode['subject'],
                $oldCode['courseNumber'] . $oldCode['courseLetter'],
                $newCode['courseNumber'] . $newCode['courseLetter']
            );
        } else {
            $diffHtml = str_replace(
                $oldCode['subject'] . ' ' . $oldCode['courseNumber'] . $oldCode['courseLetter'],
                '<del>' . $oldCode['subject'] . ' ' . $oldCode['courseNumber'] . $oldCode['courseLetter'] . '</del>' .
                '<ins>' . $newCode['subject'] . ' ' . $newCode['courseNumber'] . $newCode['courseLetter'] . '</ins>',
                $html
            );
            $html = str_replace(
                $oldCode['subject'] . ' ' . $oldCode['courseNumber'] . $oldCode['courseLetter'],
                $newCode['subject'] . ' ' . $newCode['courseNumber'] . $newCode['courseLetter'],
                $html
            );
        }

        if (Zend_Auth::getInstance()->hasIdentity()) {
            $user = Auth_UserModel::findCurrentUser();
        } else {
            $user = Auth_UserModel::findByUsername('bglenn2');
        }

        $justification = 'Updating course code from '
                       . $oldCode['subject'] . ' ' . $oldCode['courseNumber'] . $oldCode['courseLetter']
                       . ' to '
                       . $newCode['subject'] . ' ' . $newCode['courseNumber'] . $newCode['courseLetter'];

        if (!$request->getId()) {
            $request->setType('BulletinAdminEdit');
            $request->setModule('bulletin');
            $request->setJustification($justification);
            $request->setOwner($user);
            Requests_RequestModel::save($request);
            $section->setRequest($request);
            Bulletin_SectionModel::save($section);
        }

        Bulletin_RepositoryModel::getInstance()->saveFileContents(
            'data/' . $path,
            'request-' . $request->getId(),
            $user,
            $html,
            $justification
        );

        Bulletin_RepositoryModel::getInstance()->saveFileNotes(
            'data/' . $path,
            'request-' . $request->getId(),
            $diffHtml
        );

        if ($request->getType() == 'BulletinAdminEdit') {
            return $request;
        }
    }
}