<?php

class Bulletin_UtilityModel implements Creq_Model_UtilityInterface
{
    static public function loadAdditionalData(&$data)
    {
        $requests = array();
        foreach ($data as $i => $record) {
            $requests[$i] = $record['request'];
        }

        $requestSections = Bulletin_SectionModel::findWithRequest($requests);

        foreach ($data as $i => &$record) {
            $section = $requestSections[$record['request']->getId()];
            $record['extra']['Section'] = $section->getMajor() ? $section->getMajor() : $section->getCollegeLong();
            $record['extra']['College'] = $section->getCollege();
        }
        unset($record);
    }

    static public function getCollegesForRequests($requests)
    {
        $requestColleges = array();

        $requestSections = Bulletin_SectionModel::findWithRequest($requests);
        foreach ($requests as $request) {
            $section = $requestSections[$request->getId()];
            $requestColleges[$request->getId()] = $section->getCollege();
        }

        return $requestColleges;
    }

    static public function getMajorsForRequests($requests)
    {
        $requestMajors = array();

        $requestSections = Bulletin_SectionModel::findWithRequest($requests);
        foreach ($requests as $request) {
            $section = $requestSections[$request->getId()];
            $requestMajors[$request->getId()] = $section->getMajor();
        }

        return $requestMajors;
    }

    static public function getTextBetweenTokens($text, $start_token, $end_token)
    {
        $content_start = strpos($text, $start_token);
        $content_end = strpos($text, $end_token, $content_start + 1);
        $content = substr(
            $text,
            $content_start + strlen($start_token),
            $content_end - $content_start - strlen($start_token)
        );
        $content = trim($content);

        if ($content && $content_start !== FALSE && $content_end !== FALSE) {
            return $content;
        }

        return FALSE;
    }

    static protected $_majors;
    static protected $_colleges;

    static public function getColleges()
    {
        self::getMajors();
        return self::$_colleges;
    }

    /**
     * @return array
     *
     * Returns a two-dimentional array of all Majors, keyed by college, then majorId.
     */
    static public function getMajors()
    {
        if (self::$_majors) {
            return self::$_majors;
        }

        $db = Zend_Db_Table::getDefaultAdapter();
        $select = $db->select();
        $select->from(array('m' => 'creqMajors'), array('majorId', 'major' => 'name'));
	$select->join(array('c' => 'creqColleges'), 'm.college = c.collegeId', array('collegeId', 'college' => 'description'));
	$select->where("m.removed = 'no'");
        $select->order('c.description');
        $select->order('m.name');

        foreach ($select->query()->fetchAll() as $row) {
            self::$_majors[$row['college']][$row['majorId']] = $row['major'];
            self::$_colleges[$row['collegeId']] = $row['college'];
        }

        return self::$_majors;
    }

    /**
     * @return array
     * Returns an array of all Majors, keyed by majorId.
     */
    static public function getAllMajors()
    {
        $collegeMajors = self::getMajors();
        $allMajors = array();

        foreach ($collegeMajors as $majors)
        {
            $allMajors += $majors;
        }
        asort($allMajors);

        return $allMajors;
    }
}
