<?php

/**
 * PublicViewController
 *
 * @author
 * @version
 */


class Courses_PublicViewController extends Creq_Controller_Action {
	/**
	 * The default action - show the home page
	 */
	public function allCoursesAction()
    {
        $this->_disableLayoutAndView();
        
        $subject = $this->_getParam('subject');
        if ($subject) {
            $criteria = array('subject' => $subject);
            $courses = Courses_CourseModel::findWithCriteria($criteria);
            foreach ($courses as $course) {
            	$course->setEffectiveHomeSubject($subject);
            }
        } else {
            $courses = Courses_CourseModel::findAllActive();
        }
        
        $courses->orderBy('getCourseCode');
        $dom = new DOMDocument('1.0', 'UTF-8');
        $dom->formatOutput = true;
        $root = $dom->createElement('courses');
        $root->setAttribute('xmlns', 'http://courseapproval.unl.edu/courses');
        $root->setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
        $root->setAttribute('xsi:schemaLocation', 'http://courseapproval.unl.edu/courses ' . Zend_Controller_Front::getBaseUrl() . '/schema/courses.xsd');
        $dom->appendChild($root);

        foreach ($courses as $course) {
            $courseNode = $dom->createElement('course');

            $courseNode->appendChild($dom->createElement('title', htmlspecialchars($course->getTitle())));

            $crosslistings = $course->getCrosslistings();
            if (count($crosslistings) > 0) {
                $courseCodesNode = $dom->createElement('courseCodes');
                $courseNode->appendChild($courseCodesNode);
                foreach ($crosslistings as $crosslisting) {
                    $courseCodeNode = $dom->createElement('courseCode');
                    $courseCodesNode->appendChild($courseCodeNode);
                    $courseCodeNode->setAttribute('type', $crosslisting['type']);
                    $courseCodeNode->appendChild($dom->createElement('subject', $crosslisting['subject']));
                    $courseCodeNode->appendChild($dom->createElement('courseNumber', $crosslisting['courseNumber']));
                    if ($crosslisting['courseLetter']) {
                        $courseCodeNode->appendChild($dom->createElement('courseLetter', $crosslisting['courseLetter']));
                    }
                    foreach ($crosslisting['courseGroups'] as $courseGroup) {
                        $courseCodeNode->appendChild($dom->createElement('courseGroup', $courseGroup['name']));
                    }
                }
            } else {
                continue;
            }

            $courseNode->appendChild($dom->createElement('gradingType', htmlspecialchars($course->getGradingType(), null, 'UTF-8')));
            if ($course->getDfRemoval() == 'yes') {
                $courseNode->appendChild($dom->createElement('dfRemoval', 'true'));
            } else {
                $courseNode->appendChild($dom->createElement('dfRemoval', 'false'));
            }
            $courseNode->appendChild($dom->createElement('effectiveSemester', htmlspecialchars($course->getEffectiveSemester(), null, 'UTF-8')));

            if ($divNode = $this->_getNodeFromHtmlFragment($dom, 'div', $course->getPrerequisite())) {
                $divNode->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
                $node = $dom->createElement('prerequisite');
                $node->appendChild($divNode);
                $courseNode->appendChild($node);
            }
            if ($divNode = $this->_getNodeFromHtmlFragment($dom, 'div', $course->getNotes())) {
                $divNode->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
                $node = $dom->createElement('notes');
                $node->appendChild($divNode);
                $courseNode->appendChild($node);
            }
            if ($divNode = $this->_getNodeFromHtmlFragment($dom, 'div', $course->getDescription())) {
                $divNode->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
                $node = $dom->createElement('description');
                $node->appendChild($divNode);
                $courseNode->appendChild($node);
            }

            if (count($course->getCampuses()) > 0) {
                $campusesNode = $dom->createElement('campuses');
                $courseNode->appendChild($campusesNode);
                foreach ($course->getCampuses() as $campus) {
                    $campusesNode->appendChild($dom->createElement('campus', $campus));
                }
            }

            if (count($course->getDeliveryMethods()) > 0) {
                $deliveryMethodsNode = $dom->createElement('deliveryMethods');
                $courseNode->appendChild($deliveryMethodsNode);
                foreach ($course->getDeliveryMethods() as $deliveryMethod) {
                    $deliveryMethodsNode->appendChild($dom->createElement('deliveryMethod', $deliveryMethod));
                }
            }

            if (count($course->getTermsOffered()) > 0) {
                $termsOfferedNode = $dom->createElement('termsOffered');
                $courseNode->appendChild($termsOfferedNode);
                foreach ($course->getTermsOffered() as $term) {
                    $termsOfferedNode->appendChild($dom->createElement('term', $term));
                }
            }

            $activities = $course->getActivities();
            if (count($crosslistings) > 0) {
                $activitiesNode = $dom->createElement('activities');
                $courseNode->appendChild($activitiesNode);
                foreach ($activities as $activity) {
                    $activityNode = $dom->createElement('activity');
                    $activitiesNode->appendChild($activityNode);
                    $activityNode->appendChild($dom->createElement('type', $activity['type']));
                    if ($activity['hours']) {
                        $activityNode->appendChild($dom->createElement('hours', $activity['hours']));
                    }
                }
            }

            $credits = $course->getCredits();
            if (count($credits) > 0) {
                $creditsNode = $dom->createElement('credits');
                $courseNode->appendChild($creditsNode);
                foreach ($credits as $credit) {
                    $creditNode = $dom->createElement('credit', $credit['hours']);
                    $creditsNode->appendChild($creditNode);
                    $creditNode->setAttribute('type', $credit['description']);
                }
            }
            
            $aceOutcomes = $course->getAceOutcomes();
            if (count($aceOutcomes) > 0) {
                $aceNode = $dom->createElement('aceOutcomes');
                $courseNode->appendChild($aceNode);
                foreach ($aceOutcomes as $aceOutcome) {
                    $sloNode = $dom->createElement('slo');
                    $sloNode->appendChild($dom->createTextNode(substr($aceOutcome['slo'], 3)));
                    $aceNode->appendChild($sloNode);
                }
            }
            

            $gradTieIn = $course->getGradTieIn();
            if ($gradTieIn['credits']) {
                $courseNode->appendChild($dom->createElement('gradCredits', htmlspecialchars($gradTieIn['credits'], null, 'UTF-8')));
            }
            if ($divNode = $this->_getNodeFromHtmlFragment($dom, 'div', $gradTieIn['notes'])) {
                $divNode->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
                $node = $dom->createElement('gradNotes');
                $node->appendChild($divNode);
                $courseNode->appendChild($node);
            }
            if ($divNode = $this->_getNodeFromHtmlFragment($dom, 'div', $gradTieIn['prerequisites'])) {
                $divNode->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
                $node = $dom->createElement('gradPrerequisite');
                $node->appendChild($divNode);
                $courseNode->appendChild($node);
            }


            $root->appendChild($courseNode);
        }

        header('Content-type: text/xml');
        $dom->normalize();
        echo $dom->saveXML();
    }

	protected function _getNodeFromHtmlFragment($dom, $nodeName, $fragment)
	{
		$fragment = trim($fragment);
        if (!$fragment) {
        	return null;
        }

        $node = new DOMDocument('1.0', 'UTF-8');
        $fragment = '<head><meta http-equiv="Content-Type" content="text/html; charset=utf-8"/></head><body>' . $fragment . '</body>';
        $node = DOMDocument::loadHTML(strtr($fragment, array('&' => '&amp;')));
        if (!$node->documentElement) {
        	return null;
        }

        $node = $dom->importNode($node->documentElement, true);
        if (count($node->childNodes->item(1)->childNodes) == 1 && $node->childNodes->item(1)->childNodes->item(0)->nodeName == 'p') {
        	$realNode = $node->childNodes->item(1)->childNodes->item(0);
        } else {
            $realNode = $node->childNodes->item(1);
        }

        $newNode = $dom->createElement($nodeName);
        while ($realNode->hasChildNodes()) {
        	$newNode->appendChild($realNode->childNodes->item(0));
        }

        return $newNode;
	}

	/**
	 * This action syncronizes the qreqCourses.currentGeneration field with data from creqCourseGenerations
	 *
	 */
	public function testAction()
	{
		$this->_disableLayoutAndView();
        //header('Content-type: text/plain');


		$db = Zend_Registry::get('db');

		$sql = 'SELECT * FROM creqCourseGenerations';
		$data = $db->query($sql)->fetchAll();

		$courses = array();
		foreach ($data as $row) {
			$courseId = $row['course'];
			$generationId = $row['courseGenerationId'];
			$courses[$courseId][$generationId] = $row;
		}

		$multipleLatestCount = 0;
		$courseLatest = array();
		foreach ($courses as $courseId => $course) {
			$parent = null;
			$parentIds = array();
			foreach ($course as $generation) {
				if ($generation['parent']) {
				    $parentIds[] = $generation['parent'];
				}
			}
			$latestCount = 0;
			foreach ($course as $generation) {
				if (!in_array($generation['courseGenerationId'], $parentIds)) {
					$latest = $generation;
					$latestCount++;
				}
			}
			if ($latestCount > 1) {
				echo $courseId . ': Multiple Latest! Skipping.' . PHP_EOL;
				$multipleLatestCount++;
				continue;
			}
			if ($latestCount == 0) {
				echo $courseId . ': No Latest! Skipping.' . PHP_EOL;
				continue;
			}
			while ($latest['parent'] && $latest['type'] != 'official') {
				$latest = $course[$latest['parent']];
			}
			if ($latest['type'] != 'official') {
				$latestId = null;
			} else {
				$latestId = $latest['courseGenerationId'];
			}
			$courseLatest[$courseId] = $latestId;
		}
		echo 'Multiple Latest Count: ' . $multipleLatestCount . PHP_EOL;

		foreach ($courseLatest as $courseId => $latestId) {
			$data = array('currentGeneration' => $latestId);
			$where = 'courseId = ' . $courseId;
			$db->update('creqCourses', $data, $where);
		}
	}

	public function searchAction()
	{
	    $this->_loadSearchBoxViewData();
        
		$in = $this->_getAllParams();
		$criteria = $in;
		$criteria['title'] = $criteria['courseTitle'];
		$courses = Courses_CourseModel::findWithCriteria($criteria, true);

        $data = new Unl_Model_Collection('Unl_Model_Array');
        foreach ($courses as $requestId => $course) {
        	$aceOutcomes = $course->getAceOutcomes();
        	if (!$aceOutcomes) {
        		$aceOutcomes = array('-');
        	}
        	$aceCepText = '';
        
            if ($course->getAceOutcomes()) {
                $aceCepText .= '(ACE ';
                $aceOutcomeList = array();
                foreach ($course->getAceOutcomes() as $aceOutcome) {
                    $aceOutcomeList[] = substr($aceOutcome['slo'], 3);
                }
                $aceCepText .= implode(', ', $aceOutcomeList) . ')';
            }
            if ($course->isEssentialStudies()) {
                $aceCepText .= '[ES]';
            }
            if ($course->isIntegratedStudies()) {
                $aceCepText .= '[IS]';
            }
        
            $title = $course->getTitle();
            if ($course->getCrosslistingsText()) {
                $title .=  ' (' . $course->getCrosslistingsText() . ')';
            }
            $data[] = new Unl_Model_Array(array(
                'slo' => $aceCepText,
                'courseCode' => $course->getCourseCode(),
                'college' => $course->getCollegeLong(),
                'department' => $course->getDepartment(),
                'title' => $title,
                'courseId' => $course->getCourseId(),
                'className' => ($course->getRemoved() ? 'removed': '')
            ));
        }
	
        $session = new Zend_Session_Namespace(__CLASS__);
        
        $session->recentSearchCriteria = $criteria;
        
        $sortBy = $session->sortBy;
        if (!Unl_Util::isArray($sortBy)) {
        	$sortBy = array();
        }
        if ($in['sort']) {
        	if (($key = array_search($in['sort'], $sortBy)) !== FALSE) {
        		unset($sortBy[$key]);
        	}
        	$sortBy[] = $in['sort'];
        }
        $session->sortBy = $sortBy;
        
        foreach ($sortBy as $sortKey) {
        	$data->orderBy('get' . $sortKey);
        }
        
        $this->view->data = $data;
        $this->view->criteria = $criteria;
        
        
        if (isset($criteria['college'])) {
            $this->render('results');
        }
        
	}

	public function viewAction()
	{
	    $in = $this->_getAllParams();
	    $id = $in['id'];
	    $subject = $in['subject'];
	    $courseNumber = $in['number'];
	    $courseLetter = $in['letter'];

	    if ($id) {
	        $course = Courses_CourseModel::findByCourseId($id);
	    } else if ($subject && $courseNumber) {
	        $course = Courses_CourseModel::findByCourseCode($subject, $courseNumber, $courseLetter);
	        
	        if (!$course) {
    	        $criteria = array('subject' => $subject,
                                  'courseNumber' => $courseNumber,
                                  'courseLetter' => $courseLetter);
    	        $courses = Courses_CourseModel::findWithCriteria($criteria, true);
    	        if (count($courses) > 0) {
    	            $course = $courses->pop();
    	        }
	        }
	    }
	    
	    if (!$course) {
	        return;
	    }
	    

	    $session = new Zend_Session_Namespace(__CLASS__);
        $hierarchy = Courses_CourseModel::getSubjectDepartmentCollegeHierarchy();
        $colleges = array();
        $departments = array();
        $departmentsSerial = array();
        $subjects = array();
        $subjectsSerial = array();
        foreach ($hierarchy as $collegeId => $college) {
            $colleges[$collegeId] = $college['name'];
            foreach ($college['departments'] as $departmentId => $department) {
                $departments[$college['name']][$departmentId] = $department['name'];
                $departmentsSerial[$departmentId] = $department['name'];
                foreach ($department['subjects'] as $subjectCode => $subjectName) {
                    $subjects[$department['name']][$subjectCode] = $subjectCode . ' - ' . $subjectName;
                    $subjectsSerial[$subjectCode] = $subjectCode . ' - ' . $subjectName;
                }
            }
        }
        
        
	    $this->view->criteria = $session->recentSearchCriteria;
	    $this->view->course = $course;
	    
        $this->_loadSearchBoxViewData();
	    
	    $previousRequests = Courses_CourseModel::findAllRequestsForCourse($course);
	    $previousRequests->orderBy('getLastApprovalTimestamp', SORT_DESC);
	    
	    $this->view->effectiveDate = new Unl_Date_Null('Prior to April, 2008.');
	    foreach ($previousRequests as $previousRequest) {
	        $this->view->effectiveDate = $previousRequest->getLastApprovalTime();
	        break;
	    }
	    
	    $requestParentCourses = Courses_CourseModel::findParentOfRequest($previousRequests);
	    $requestFinalCourses = Courses_CourseModel::findLatestOfRequest($previousRequests);
	    
	    $this->view->previousRequests = $previousRequests;
	    $this->view->requestParentCourses = $requestParentCourses;
	    $this->view->requestFinalCourses = $requestFinalCourses;
	    
	    $requestComments = Requests_CommentsModel::findByRequest($previousRequests);
	    $this->view->requestComments = $requestComments;
	    
	    //$offerings = Courses_ScheduleModel::fetchForCourses($course);
	
	    $controllerSession = new Zend_Session_Namespace(__CLASS__);
	    $this->view->recentSearchCriteria = $controllerSession->recentSearchCriteria;
	    
	    $session = new Zend_Session_Namespace(__CLASS__ . __METHOD__);
	    
	    // sort offerings
	    
        $sortBy = $session->sortBy;
        if (!Unl_Util::isArray($sortBy)) {
            $sortBy = array('semesterCode');
        }
        if ($in['sort']) {
            if (($key = array_search($in['sort'], $sortBy)) !== FALSE) {
                unset($sortBy[$key]);
            }
            $sortBy[] = $in['sort'];
        }
        $session->sortBy = $sortBy;
        
        // sort request history
        
        $sortBy = $session->sortHistoryBy;
        if (!Unl_Util::isArray($sortBy)) {
            $sortBy = array();
        }
        if ($in['sort-history']) {
            if (($key = array_search($in['sort-history'], $sortBy)) !== FALSE) {
                unset($sortBy[$key]);
            }
            $sortBy[] = $in['sort-history'];
        }
        $session->sortHistoryBy = $sortBy;
        
        foreach ($sortBy as $sortKey) {
            if ($sortKey == 'lastApprovalTimestamp') {
                $previousRequests->orderBy('get' . $sortKey, SORT_DESC);
            } else {
                $previousRequests->orderBy('get' . $sortKey);
            }
        }
	    
        
        
	    $this->view->offerings = $offerings;
	    
	    $relatedCourses = array();
	    $relatedMajors = array();
	    foreach ($course->getCrosslistings() as $crosslisting) {
	        $myRelatedCourses = Courses_CourseModel::getRelatedCourses($crosslisting['subject'], $crosslisting['courseNumber'], $crosslisting['courseLetter']);
	        $myRelatedMajors = Courses_CourseModel::getRelatedMajors($crosslisting['subject'], $crosslisting['courseNumber'], $crosslisting['courseLetter']);
	        if (count($myRelatedCourses) > 0) {
	            $relatedCourses[$crosslisting['subject'] . ' ' . $crosslisting['courseNumber'] . $crosslisting['courseLetter']] = $myRelatedCourses;
	        }
	        if ($myRelatedMajors) {
	            $relatedMajors[$crosslisting['subject'] . ' ' . $crosslisting['courseNumber'] . $crosslisting['courseLetter']] = $myRelatedMajors;
	        }
	    }
	    $this->view->relatedCourses = $relatedCourses;
	    $this->view->relatedMajors = $relatedMajors;
	    
	}
	
	public function printAction()
	{
        $this->_helper->layout->setLayout('naked-layout');
        $this->viewAction();
        $this->view->print = true;
        $this->render('view');
	}
	
	protected function _loadSearchBoxViewData()
	{
        $hierarchy = Courses_CourseModel::getSubjectDepartmentCollegeHierarchy();
        $colleges = array();
        $departments = array();
        $departmentsSerial = array();
        $subjects = array();
        $subjectsSerial = array();
        foreach ($hierarchy as $collegeId => $college) {
            $colleges[$collegeId] = $college['name'];
            foreach ($college['departments'] as $departmentId => $department) {
                $departments[$department['name']] = array('collegeId' => $collegeId, 'departmentId' => $departmentId);
                $departmentsSerial[$departmentId] = $department['name'];
                foreach ($department['subjects'] as $subjectCode => $subjectName) {
                    $subjects[$subjectCode . ' - ' . $subjectName] = array('departmentId' => $departmentId, 'subjectCode' => $subjectCode);
                    $subjectsSerial[$subjectCode] = $subjectCode . ' - ' . $subjectName;
                }
            }
        }
        
        ksort($departments);
        ksort($subjects);
        
        $this->view->colleges = $colleges;
        $this->view->departments = $departments;
        $this->view->departmentsSerial = $departmentsSerial;
        $this->view->subjects = $subjects;
        $this->view->subjectsSerial = $subjectsSerial;
        
	    $this->view->activityTypes = array(
            1 => 'Lec - Lecture',
            2 => 'Lab - Lab',
            3 => 'Quz - Quiz',
            4 => 'Rct - Recitation',
            5 => 'Stu - Studio',
            6 => 'Fld - Field',
            7 => 'Ind - Independent Study',
            8 => 'Psi - Personalized System of Instruction'
        );
	}
}
