<?php

class Courses_ApprovalActionEmailCourseInfoModel extends Requests_ApprovalActionEmailModel
{
    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('e' => 'creqApprovalActionsEmail'), 'a.approvalActionId = e.approvalActionId');
        $select->join(array('i' => 'creqApprovalActionsEmailCourseInfo'), 'e.approvalActionId = i.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        
        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }

    }
    
    static public function fetchNew()
    {
        $newParent = parent::fetchNew();
        $data = $newParent->_data;
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionEmailCourseInfo';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
        
        parent::save($models);
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        //until this table has more fields, there's no reason to ever have to update it.
        return;
        
        
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsEmailCourseInfoUpdate '
             . 'SELECT * FROM creqApprovalActionsEmailCourseInfo LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsEmailCourseInfoUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?,'  , $model->_data['alpha'])
                        . $db->quoteInto('?)'  , $model->_data['bravo']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsEmailCourseInfo AS a, '
             . '       creqApprovalActionsEmailCourseInfoUpdate AS b '
             . 'SET a.alpha = b.alpha, '
             . '    a.bravo = b.bravo '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
        
        $db->query('DROP TABLE creqApprovalActionsEmailCourseInfoUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsEmailCourseInfo (approvalActionId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?)', $model->_data['approvalActionId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    protected $_requestCourses;
    protected $_requestParentCourses;
    protected $_requestPreviousRequests;
    protected $_owners;
    protected $_collegesTable;
    protected $_departmentsTable;
    protected $_subjectsTable;
    
    /*
     * This doesn't actually do anything other than pre-load data that will be used in the _substitute call.
     */
    public function consider($requests)
    {
        $this->_collegesTable = new Zend_Db_Table('creqColleges');
        $this->_departmentsTable = new Zend_Db_Table('creqDepartments');
        $this->_subjectsTable = new Zend_Db_Table('creqSubjects');
        
        $this->_requestCourses = Courses_CourseModel::findLatestOfRequest($requests);
        $this->_requestParentCourses = Courses_CourseModel::findParentOfRequest($requests);
        $previousRequestIds = array();
        foreach ($this->_requestParentCourses as $requestId => $course) {
            if (!$course) {
                continue;
            }
            $previousRequestIds[$requestId] = $course->getRequest();
        }
        $previousRequests = Requests_RequestModel::find($previousRequestIds);
        $this->_requestPreviousRequests = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($previousRequestIds as $childId => $parentId) {
        	if ($previousRequests[$parentId]) {
                $this->_requestPreviousRequests[$childId] = $previousRequests[$parentId];
        	}
        }
        
        $ownerIds = array();
        foreach ($requests as $request) {
        	$ownerIds[] = $request->getOwner();
        }
        foreach ($previousRequests as $request) {
        	$ownerIds[] = $request->getOwner();
        }
        $this->_owners = Auth_UserModel::find($ownerIds);
        
        return parent::consider($requests);
    }

    protected function _substitute($text, $request)
    {
        $course = $this->_requestCourses[$request->getId()];
        $owner = $this->_owners[$request->getOwner()];
        $parentCourse = $this->_requestParentCourses[$request->getId()];
        $parentRequest = $this->_requestPreviousRequests[$request->getId()];
        if ($parentRequest) {
            $parentOwner = $this->_owners[$parentRequest->getOwner()];
        }
        
        $text = parent::_substitute($text, $request);
        $text = strtr($text, array(
            '%cc%' => $course->getCourseCode(),     // Course Code
            '%ri%' => $request->getId(),            // RequestId
            '%rj%' => $request->getJustification(), // Request Justification
            '%rse%' => $owner->getEmail(),          // Request Submitter Email
        ));
        
        if ($parentCourse) {
	        $text = strtr($text, array(
	            '%cc-%' => $parentCourse->getCourseCode(), // Course Code of Parent 
	        ));
        } else {
            $text = strtr($text, array(
                '%cc-%' => $course->getCourseCode(), // Course Code of Parent
            ));
        }
        
        if ($parentRequest) {
	        $text = strtr($text, array(
                '%ri-%' => $parentRequest->getId(),           // RequestId of Parent Request
                '%rj-%' => $parentRequest->getJustification() // Justuficaiton of Parent Request
	        ));
        }
        
        if ($parentOwner) {
	        $text = strtr($text, array(
	            '%rse-%' => $parentOwner->getEmail() // Submitter Email of Parent Request 
	        ));
        } else {
            $text = strtr($text, array(
                '%rse-%' => 'nobody@example.com' // Submitter Email of Parent Request (not found)
            ));
        }

        // Handle related departments/colleges
        if ($parentCourse) {
            $relatedCourses = new Unl_Model_Collection('Courses_CourseModel');
            $relatedDepartmentIds = array();
            $relatedCollegeIds = array();
            $relatedSubjectIds = array();
            foreach ($parentCourse->getCrosslistings() as $crosslisting) {
                $relatedCourses->merge(Courses_CourseModel::getRelatedCourses($crosslisting['subject'], $crosslisting['courseNumber'], $crosslisting['courseLetter']));
            }
            foreach ($relatedCourses as $relatedCourse) {
                $relatedCollegeIds[$relatedCourse->getCollegeId()] = $relatedCourse->getCollegeId();
                $relatedDepartmentIds[$relatedCourse->getDepartmentId()] = $relatedCourse->getDepartmentId();
                $relatedSubjectIds[$relatedCourse->getSubjectId()] = $relatedCourse->getSubjectId();
            }

            // Function to get related hierarchy (college/department/subject) email addresses
            $getHierarchyEmails = function($table, $idColumn, $relatedIds) {
                if (count($relatedIds) == 0) {
                    return 'nobody@example.com';
                }
                $hierarchyContactGroupIds = array();
                
                $select = $table->select();
                $select->where($idColumn . ' IN (?)', $relatedIds);

                foreach ($table->fetchAll($select) as $relatedHierarchy) {
                    if (!$relatedHierarchy->contactGroup) {
                        continue;
                    }
                    $hierarchyContactGroupIds[$relatedHierarchy->contactGroup] = $relatedHierarchy->contactGroup;
                }
                
                $relatedHierarchyContactGroups = Auth_GroupModel::find($hierarchyContactGroupIds);
                $relatedHierarchyContactUserGroups = Auth_UserModel::findByParentGroup($relatedHierarchyContactGroups);
                
                $relatedHierarchyEmails = array('nobody@example.com' => 'nobody@example.com');
                foreach ($relatedHierarchyContactUserGroups as $relatedHierarchyContactUserGroup) {
                    foreach ($relatedHierarchyContactUserGroup as $relatedHierarchyContactUser) {
                        $relatedHierarchyEmails[$relatedHierarchyContactUser->getEmail()] = $relatedHierarchyContactUser->getEmail();
                    }
                }
                $relatedHierarchyEmails = implode(',', $relatedHierarchyEmails);
                
                return $relatedHierarchyEmails;
            };

            $relatedCollegeEmails    = $getHierarchyEmails($this->_collegesTable, 'collegeId', $relatedCollegeIds);
            $relatedDepartmentEmails = $getHierarchyEmails($this->_departmentsTable, 'departmentId', $relatedDepartmentIds);
            $relatedSubjectEmails    = $getHierarchyEmails($this->_subjectsTable, 'subjectId', $relatedSubjectIds);
            

            $text = strtr($text, array(
                '%rcce%' => $relatedCollegeEmails, // Related Colleges Contact Emails
                '%rdce%' => $relatedDepartmentEmails, // Related Departments Contact Emails
                '%rsce%' => $relatedSubjectEmails, // Related Subjects Contact Emails
            ));
        } else {
            $text = strtr($text, array(
                '%rcce%' => 'nobody@example.com', // Related Colleges Contact Emails (not found)
                '%rdce%' => 'nobody@example.com', // Related Departments Contact Emails (not found)
                '%rsce%' => 'nobody@example.com', // Related Subjects Contact Emails (not found)
            ));
        }
        
        return $text;
    }

}
