<?php

class Courses_ApprovalActionForkAceModel extends Requests_ApprovalActionModel
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsForkAce'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionForkAce';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        // Nothing to update
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsForkAce (approvalActionId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?) ', $model->_data['approvalActionId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function getResultStatusStrings()
    {
        return array('ACE' => 'ACE');
    }
    
    public function consider($requests)
    {
        $courses = Courses_CourseModel::findLatestOfRequest($requests);
        $parentCourses = Courses_CourseModel::findParentOfRequest($requests);
        
        $newCourseRequests = array();
        $newCourses = new Unl_Model_Collection('Courses_CourseModel');
        $newRequests = new Unl_Model_Collection('Requests_RequestModel');
        
        $childParentRequestMap = array();
        
        foreach ($requests as $request) {
            
            // check to see what each of the new request types will be
            switch ($request->getType()) {
                case 'NewCourseWithACE':
                    $request->setType('NewCourse');
                    $addAce = true;
                    break;
                case 'AddACEAndChangeCourse':
                    $request->setType('ChangeCourse');
                    $addAce = true;
                    break;
                case 'RemoveACEAndChangeCourse':
                    $request->setType('ChangeCourse');
                    $addAce = false;
                    break;
                default:
                    continue 2;
                    break;
            }
            
            $course = $courses[$request->getId()];
            $parentCourse = $parentCourses[$request->getId()];
            
            // create a new generation of the course from the input request to base the new request on.
            $aceCourse = clone $course;
            
            // either remove or add the ACE data to the non-ACE request's course.
            if ($addAce) {
                $course->removeAce();
            } else {
                foreach ($parentCourse->getAceOutcomes() as $aceOutcome) {
                    $course->setAceOutcome($aceOutcome['name'],
                                           $aceOutcome['justification'],
                                           $aceOutcome['studentWork'],
                                           $aceOutcome['assesmentPlan']
                                           );
                }
                foreach ($parentCourse->getAceReinforcements() as $aceReinforcement) {
                    $course->setAceReinforcement($aceReinforcement['name'],
                                                 $aceReinforcement['description']);
                }
            }
            
            // create the new request to contain only the ACE data.
            $newRequest = clone $request;
            $childParentRequestMap[] = array('child' => $newRequest, 'parent' => $request);
            if ($addAce) {
                $newRequest->setType('AddACEToCourse');
            } else {
                $newRequest->setType('RemoveACEFromCourse');
            }
            $newRequest->setState('ACE');
            
            $newRequests[] = $newRequest;
            $newCourses[] = $aceCourse;
            $newCourseRequests[] = array(
                'request' => $newRequest,
                'course' => $aceCourse
            );
        }
        Courses_CourseModel::save($courses);
        Requests_RequestModel::save($requests);
        Requests_RequestModel::save($newRequests);
        
        foreach ($newCourseRequests as $newCourseRequest) {
            $request = $newCourseRequest['request'];
            $course = $newCourseRequest['course'];
            
            $course->setRequest($request);
        }
        
        Courses_CourseModel::save($newCourses);
        
        // Copy the watchers from the parent requests to the child requests
        $db = Zend_Registry::get('db');
        foreach ($childParentRequestMap as $childParentRequests) {
        	$parent = $childParentRequests['parent'];
        	$child = $childParentRequests['child'];
        	$sql = 'INSERT INTO creqRequestWatchers (request, watchingApprovalRole) '
        	     . $db->quoteInto('SELECT ?, watchingApprovalRole ', $child->getId())
        	     . 'FROM creqRequestWatchers AS w '
        	     . $db->quoteInto('WHERE w.request=? ', $parent->getId());
        	     ;
            $db->query($sql);
        }
        
        //  all done... advance all requests and return them for consideration
        $allRequests = $requests->merge($newRequests);
        Requests_ApprovalChainModel::advance($requests);
        return $requests;
    }

    public function setExtendedData($in)
    {
        // No extended data
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

}
