<?php

class Courses_ApprovalActionGradLevelRouterModel extends Requests_ApprovalActionModel
{

    const LEVEL_UNDERGRADUATE_ONLY = 'Undergraduate Only';
    const LEVEL_GRADUATE_ONLY = 'Graduate Only';
    const LEVEL_BOTH = 'Both';
    
    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsGradLevelRouter'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }

    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionGradLevelRouter';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        // Nothing to update
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsGradLevelRouter (approvalActionId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?) ', $model->_data['approvalActionId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {   
        $requestParentCourses = Courses_CourseModel::findParentOfRequest($requests);
        $requestCurrentCourses = Courses_CourseModel::findLatestOfRequest($requests);
        foreach ($requests as $request) {
            $isUndergraduate = FALSE;
            $isGraduate = FALSE;
        
            $parentCourse = $requestParentCourses[$request->getId()];
            $currentCourse = $requestCurrentCourses[$request->getId()];
            if ($parentCourse && $parentCourse->isUndergraduateLevel() || $currentCourse && $currentCourse->isUndergraduateLevel()) {
                $isUndergraduate = TRUE;
            }
            if ($parentCourse && $parentCourse->isGraduateLevel() || $currentCourse && $currentCourse->isGraduateLevel()) {
                $isGraduate = TRUE;
            }
            if ($isUndergraduate && !$isGraduate) {
                $request->setState(self::LEVEL_UNDERGRADUATE_ONLY);
            } else if (!$isUndergraduate && $isGraduate) {
                $request->setState(self::LEVEL_GRADUATE_ONLY);
            } else if ($isUndergraduate && $isGraduate) {
                $request->setState(self::LEVEL_BOTH);
            } else {
                $request->setState('Uh oh!');
                //throw new Unl_Exception('While routing a course based on undergrad/grad level, a course that was neither was found!');
            }
        }
        Requests_RequestModel::save($requests);
        Requests_ApprovalChainModel::advance($requests);
        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

    public function setExtendedData($in)
    {
        // No extended data
    }

    public function getResultStatusStrings()
    {
        return array(
            self::LEVEL_UNDERGRADUATE_ONLY => self::LEVEL_UNDERGRADUATE_ONLY,
            self::LEVEL_GRADUATE_ONLY      => self::LEVEL_GRADUATE_ONLY,
            self::LEVEL_BOTH               => self::LEVEL_BOTH
        );
    }
}
