<?php

class Courses_ApprovalActionMakeOfficialModel extends Requests_ApprovalActionModel
{
	static public function find($id)
	{
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsMakeOfficial'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
	}
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionMakeOfficial';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        // Nothing to update
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsMakeOfficial (approvalActionId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?) ', $model->_data['approvalActionId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        foreach ($requests as $request) {
            $request->setState('Approve');
        }
        Requests_RequestModel::save($requests);
        
        $courses = Courses_CourseModel::findLatestOfRequest($requests);
        $parentCourses = Courses_CourseModel::findParentOfRequest($requests);
        
        // Update all references to changed course codes in other courses.
        foreach ($requests as $request) {
            $course = $courses[$request->getId()];
            $parentCourse = $parentCourses[$request->getId()];
            if (!$parentCourse) {
                continue;
            }
            
            foreach ($course->getChangedCourseCodes($parentCourse) as $change) {
                if ($change['from']['subject'] != $change['to']['subject']) {
                    continue;
                }
                Courses_CourseModel::updateCourseNumber(
                    $change['from']['subject'],
                    $change['from']['courseNumber'],
                    $change['from']['courseLetter'],
                    $change['to']['courseNumber'],
                    $change['to']['courseLetter']
                );

                $cronJob = Cron_JobsModel::fetchNew();
                $cronJob->setDate(new Zend_Date());
                $cronJob->setClass('Bulletin_SectionModel');
                $cronJob->setMethod('updateCourseCode');
                $cronJob->setArguments($change['from'], $change['to']);

                $collection = new Unl_Model_Collection('Cron_JobsModel');
                $collection[] = $cronJob;
                Cron_JobsModel::save($collection);
            }
        }
        
        foreach ($courses as $course) {
            $course->setType('official');
        }
        Courses_CourseModel::save($courses);
        Courses_CourseModel::makeCurrent($courses);
        Requests_ApprovalChainModel::advance($requests);
        return $requests;
    }

    public function setExtendedData($in)
    {
        // No extended data
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

}
