<?php

class Courses_ScheduleModel extends Unl_Model 
{

    static public function find($courseIds)
    {
        $db = Zend_Registry::get('scheduleDb');

        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'courses'));
        if (Unl_Util::isArray($courseIds)) {
            $select->where('c.courseid IN(?)', $courseIds);
        } else {
            $select->where('c.courseid = ?', $courseIds);
        }

        $records = $db->query($select)->fetchAll();
        
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $object->_data['instructors'] = array();
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }
        
        
        $select = new Zend_Db_Select($db);
        $select->from(array('s' => 'staff_assign'), array('courseid'));
        $select->join(array('p' => 'people'), 's.nuid = p.nuid', array('firstname', 'lastname'));
        $select->where('s.courseid IN (?)', $objects->getId());
        $select->where('primaryrole IN (?)', array('staff', 'faculty'));
        $select->order('lastname');
        
        $records = $db->query($select)->fetchAll();
        foreach ($records as $record) {
            $objects[$record['courseid']]->_data['instructors'][] = $record['firstname'] . ' ' . $record['lastname'];
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function fetchForCourses($courses, $semester = NULL)
    {
        $multiple = true;
        if (!Unl_Util::isArray($courses)) {
            $multiple = false;
            $collection = new Unl_Model_Collection('Courses_CourseModel');
            $collection[] = $courses;
            $courses = $collection;
        }
        
        $db = Zend_Registry::get('scheduleDb');
        
        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'courses'), 'courseid');
        
        $courseCodes = array();
        foreach ($courses as $course) {
            $courseCodes[$course->getId()] = $course->getSubject() . $course->getCourseNumber() . $course->getCourseLetter();
        }
        $select->where('SUBSTRING(c.courseid, 7, LENGTH(c.courseid)-9) IN(?)', $courseCodes);
        
        if ($semester) {
            $select->where('semester_code = ?', $semester);
        }

        $records = $db->query($select)->fetchAll();
        
        $courseIds = array();
        foreach ($records as $record) {
            $courseIds[] = $record['courseid'];
        }
        
        $models = self::find($courseIds);
        
        
        $results = array();
        foreach ($courseCodes as $courseId => $courseCode) {
            $results[$courseId] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($models as $model) {
            $courseCode = substr($model->getId(), 6, -3);
            $courseId = array_search($courseCode, $courseCodes);
            $results[$courseId][$model->getId()] = $model;
        }
        
        if ($multiple) {
            return $results;
        } else {
            return array_pop($results);
        }
    }
    
    public function getId()
    {
        return $this->getCourseId();
    }
    
    public function getCourseId()
    {
        return $this->_data['courseid'];
    }
    
    public function getSemesterCode()
    {
        return $this->_data['semester_code'];
    }
    
    public function getTitle()
    {
        return $this->_data['title'];
    }
    
    public function getSection()
    {
        return $this->_data['section'];
    }
    
    public function getCredit()
    {
        return $this->_data['creidt'];
    }
    
    public function getDepartment()
    {
        return $this->_data['department'];
    }
    
    public function getCollege()
    {
        return $this->_data['college'];
    }
    
    public function getBuilding()
    {
        return $this->_data['building'];
    }
    
    public function getRoom()
    {
        return $this->_data['room'];
    }
    
    public function getDays()
    {
        return $this->_data['days'];
    }
    
    public function getStartTime()
    {
        return $this->_parseTime($this->_data['starttime']);
    }
    
    public function getEndTime()
    {
        return $this->_parseTime($this->_data['endtime']);
    }
    
    public function getStartDate()
    {
        return $this->_parseDate($this->_data['startdate']);
    }
    
    public function getEndDate()
    {
        return $this->_parseDate($this->_data['enddate']);
    }
    
    public function _parseDate($dateText)
    {
        if (!$dateText || $dateText == 'ARR') {
            $dateText = '00/00/0000';
        }
        
        $date = new Zend_Date();
        $date->setYear(substr($dateText, -4));
        $date->setMonth(substr($dateText, 0, 2));
        $date->setDay(substr($dateText, 3, 2));
        $date->setHour(0);
        $date->setMinute(0);
        $date->setSecond(0);
        
        return $date;
    }
    
    public function _parseTime($timeText)
    {   
        if (!$timeText || $timeText == 'ARR') {
            $timeText = '1200AM';
        }
        
        $date = new Zend_Date();
        $date->setYear(0);
        $date->setMonth(0);
        $date->setDay(0);
        $date->setMinute(substr($timeText, 2, 2));
        $date->setSecond(0);
        
        $hour = intval(substr($timeText, 0, 2));
        $amPm = substr($timeText, -2);
        if ($amPm == 'AM' && $hour == 12) {
            $date->setHour(0);
        } else if ($amPm == 'AM' || $hour == 12) {
            $date->setHour($hour);
        } else if ($amPm == 'PM') {
            $date->setHour($hour + 12);
        }
        
        return $date;
    }
    
    public function getInstructors()
    {
        return $this->_data['instructors'];
    }
    
}

















