<?php

class Cron_JobsModel extends Unl_Model
{
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }


        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqCronJobs'));
        if (Unl_Util::isArray($id)) {
            $select->where('a.cronJobId IN(?)', $id);
        } else {
            $select->where('a.cronJobId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    /**
     * Returns a new Cron_JobsModel
     *
     * @return Cron_JobsModel
     */
    static public function fetchNew()
    {
        $data = array(
            'date'       => NULL,
            'hasStarted' => 'no',
            'hasRun'     => 'no',
            'class'      => NULL,
            'method'     => NULL,
            'arguments'  => serialize(array())
        );
    	
    	$new = new self($data);
    	$new->_setClean();
    	return $new;
    }

    static public function runPendingJobs()
    {

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqCronJobs'), array('cronJobId'));
        $select->where('date <= ?', time());
        $select->where('hasStarted = "no"');
        $select->where('hasRun = "no"');

        $records = $select->query()->fetchAll();
        $cronJobIds = array();
        foreach ($records as $record) {
            $cronJobIds[] = $record['cronJobId'];
        }
        $jobs = self::find($cronJobIds);

        foreach ($jobs as $job) {
            $class = $job->_data['class'];
            if ($class == 'ApprovalActionsQueue') {
                $class = 'Requests_ApprovalActionQueueModel';
            }
            $method = $job->_data['method'];
            $arguments = $job->_data['arguments'];
            if ($arguments) {
                $arguments = unserialize($arguments);
            } else {
                $arguments = null;
            }
            if (!class_exists($class)) {
                throw new Nmc_Exception('Cannot run cron job from nonexistant class: ' . $class);
            }

            if (!method_exists($class, $method)) {
                throw new Nmc_Exception('Cannot run cron job of nonexistant method: ' . $class . '::' . $method);
            }

            $job->setHasStarted();
            self::save($jobs);

            call_user_func_array(array($class, $method), $arguments);

            $job->setHasRun();
            self::save($jobs);
        }

    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
    	$modelsToUpdate = new Unl_Model_Collection(__CLASS__);

    	if (!Unl_Util::isArray($models)) {
    	    $model = $models;
    	    $models = new Unl_Model_Collection(__CLASS__);
    	    $models[$model->getId()] = $model;
    	}

    	foreach ($models as $model) {
    		if ($model->getId()) {
    			if ($model->_cleanData != $model->_data) {
    			    $modelsToUpdate[] = $model;
    			}
    		} else {
    			$modelsToInsert[] = $model;
    		}
    	}

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
    	$db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqCronJobsUpdate '
             . 'SELECT * FROM creqCronJobs LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqCronJobsUpdate VALUES ';
        $sqlParts = array();
    	foreach ($models as $model) {
    		$sqlParts[] = $db->quoteInto('(?, ', $model->_data['cronJobId'])
                        . $db->quoteInto('?, ' , $model->_data['date'])
                        . $db->quoteInto('?, ' , $model->_data['hasStarted'])
                        . $db->quoteInto('?, ' , $model->_data['hasRun'])
                        . $db->quoteInto('?, ' , $model->_data['class'])
                        . $db->quoteInto('?, ' , $model->_data['method'])
                        . $db->quoteInto('?)'  , $model->_data['arguments']);
    	}
    	$sql .= implode(', ', $sqlParts);
    	$db->query($sql);

    	$sql = 'UPDATE creqCronJobs AS a, '
    	     . '       creqCronJobsUpdate AS b '
    	     . 'SET a.date       = b.date, '
             . '    a.hasStarted = b.hasStarted, '
             . '    a.hasRun     = b.hasRun, '
             . '    a.class      = b.class, '
             . '    a.method     = b.method, '
             . '    a.arguments  = b.arguments '
             . 'WHERE a.cronJobId = b.cronJobId ';
        $db->query($sql);
        $db->query('DROP TABLE creqCronJobsUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

    	$sql = 'INSERT INTO creqCronJobs (date, hasStarted, hasRun, class, method, arguments) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['date'])
                        . $db->quoteInto('?, ' , $model->_data['hasStarted'])
                        . $db->quoteInto('?, ' , $model->_data['hasRun'])
                        . $db->quoteInto('?, ' , $model->_data['class'])
                        . $db->quoteInto('?, ' , $model->_data['method'])
                        . $db->quoteInto('?)'  , $model->_data['arguments']);
        }
        $sql .= implode(', ', $sqlParts);
        try {
            $db->query($sql);
        } catch (Zend_Db_Statement_Exception $e) {
            if (strpos($e->getMessage(), 'Duplicate entry') === FALSE) {
                throw $e;
            }
        }

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['cronJobId'] = $lastId;
            $lastId++;
        }
    }

    public function getId()
    {
        return $this->_data['cronJobId'];
    }

    public function setDate(Zend_Date $date)
    {
    	$this->_data['date'] = $date->getTimestamp();
    }
    
    public function setHasStarted($running = true)
    {
        if ($running) {
            $this->_data['hasStarted'] = 'yes';
        } else {
            $this->_data['hasStarted'] = 'no';
        }
    }

    public function setHasRun($running = true)
    {
        if ($running) {
            $this->_data['hasRun'] = 'yes';
        } else {
            $this->_data['hasRun'] = 'no';
        }
    }
    
    public function setClass($class)
    {
        $this->_data['class'] = $class;
    }
    
    public function setMethod($method)
    {
        $this->_data['method'] = $method;
    }
    
    public function setArguments()
    {
        $this->_data['arguments'] = serialize(func_get_args());
    }
}