<?php

class RegistryModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('r' => 'creqRegistry'));

        if (Unl_Util::isArray($id)) {
            if (count($id) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('registryId IN(?)', $id);
        } else {
            $select->where('registryId = ?', $id);
        }
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($id)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$id];
        }
    }

    static public function findByNamespaceAndKey($namespace, $key)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('r' => 'creqRegistry'), array('registryId'));
        $select->where('`namespace` = ?', $namespace);
        $select->where('`key` = ?', $key);
        $data = $select->query()->fetchAll();

        $registryId = $data[0]['registryId'];

        if ($registryId) {
            return self::find($registryId);
        } else {
            $new = self::fetchNew();
            $new->_data['namespace'] = $namespace;
            $new->_data['key'] = $key;
            return $new;
        }
        
    }

    /**
     * Returns a new, empty model.
     *
     * @return RegistryModel
     */
    static public function fetchNew()
    {
        $data = array(
            'registryId' => NULL,
            'namespace'  => NULL,
            'key'        => NULL,
            'data'       => NULL
        );

        $new = new self($data);
        $new->_setClean();

        return $new;
    }
    
    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqRegistryUpdate '
             . 'SELECT * FROM creqRegistry LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqRegistryUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['registryId'])
                        . $db->quoteInto('?, ' , $model->_data['namespace'])
                        . $db->quoteInto('?, ' , $model->_data['key'])
                        . "'" . base64_encode(serialize($model->_data['data'])) . "')";
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqRegistry AS a, '
             . '       creqRegistryUpdate AS b '
             . 'SET a.namespace = b.namespace, '
             . '    a.key       = b.key, '
             . '    a.data      = b.data '
             . 'WHERE a.registryId = b.registryId ';
        $db->query($sql);
        $db->query('DROP TABLE creqRegistryUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqRegistry (`namespace`, `key`, `data`) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['namespace'])
                        . $db->quoteInto('?, ' , $model->_data['key'])
                        . $db->quoteInto('?)'  , base64_encode(serialize($model->_data['data'])));
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['registryId'] = $lastId;
            $lastId++;
        }
    }

    static public function delete($models)
    {
        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');
        $sql = $db->quoteInto('DELETE FROM creqRegistry WHERE registryId IN (?)', $models->getId());
        $db->query($sql);
    }
    
    public function __construct($data)
    {
    	parent::__construct($data);
    	$this->_data['data'] = unserialize(base64_decode($this->_data['data']));
    }
    
    public function getId()
    {
    	return $this->_data['registryId'];
    }
    
    public function getData()
    {
    	return $this->_data['data'];
    }
    
    public function setData($data)
    {
        $this->_data['data'] = $data;
    }
}