<?php

class FourYearPlans_EditController extends Creq_Controller_Action
{
    public function indexAction()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $fourYearPlan = $session->fourYearPlan;
        $request = $session->request;

        $this->view->fourYearPlan = $fourYearPlan;
        $this->view->request = $request;
        $this->view->formSessionId = $in['id'];
        $this->view->courseList = $this->_getCourseList();
    }

    public function editPostAction()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $fourYearPlan = $session->fourYearPlan;

        $fourYearPlan->clearCourses();
        foreach ($in['courses'] as $concentrationId => $semesters) {
            $concentration = $in['concentrations'][$concentrationId];
            foreach ($semesters as $semesterNumber => $semesterCourses) {
                foreach ($semesterCourses as $key => $course) {
                    if ($course['type'] == 'course') {
                        $fourYearPlan->addCourse($concentration, $semesterNumber, $course['value'], isset($course['hours']) ? $course['hours'] : NULL);
                    } else {
                        $fourYearPlan->addCourseText($concentration, $semesterNumber, $course['value'], $course['hours']);
                    }
                }
            }
        }

        $fourYearPlan->clearNotes();
        foreach ($in['notes'] as $concentrationId => $notes) {
            if ($concentrationId == 'stub') {
                continue;
            }
            $concentration = $in['concentrations'][$concentrationId];
            $fourYearPlan->setNotes($concentration, $notes);
        }

        $this->redirect('/fouryearplans/view/index/type/session/id/' . $in['id']);
    }

    public function getCourseListAction()
    {
        $courseList = $this->_getCourseList();

        $this->_response->setHeader('Content-type', 'application/json');
        $this->_response->setBody(json_encode($courseList));
        $this->_disableLayoutAndView();
    }

    protected function _getCourseList()
    {
        $cache = Zend_Controller_Front::getInstance()->getParam('bootstrap')->getPluginResource('cachemanager')->getCacheManager()->getCache('daily');

        if ($cache->test(__CLASS__ . '_courseList')) {
            $courseList = $cache->load(__CLASS__ . '_courseList');
        } else {
            $courses = Courses_CourseModel::findAllActive();
            $courses->orderBy('getCourseCode');
            $courseList = array();
            foreach ($courses as $course) {
                $hours = $course->getCreditsText();
                $hours = substr($hours, 1, strpos($hours, 'cr') - 1);
                $hours = trim($hours);
                $courseList[$course->getCourseId()] = array(
                    'code' => $course->getCourseCode(),
                    'hours' => $hours,
                );
            }
            $cache->save($courseList, __CLASS__ . '_courseList');
        }

        return $courseList;
    }

    public function submitPostAction()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);

        if ($in['submitType'] == 'Submit') {
            set_time_limit(5);
            $this->_saveRequest();
            $this->_startApproval();
            $this->redirect('/requests');
        } else if ($in['submitType'] == 'Return to Edit') {
            $this->redirect('/fouryearplans/edit/index/id/' . $in['id']);
        } else if ($in['submitType'] == 'Save Changes') {
            $this->_saveRequest();
            $this->redirect('/fouryearplans/view/index/id/' . $session->request->getId());
        } else if ($in['submitType'] == 'Save for Later Submission') {
            $this->_saveForLater();
            $this->redirect('/requests');
        } else if ($in['submitType'] == 'Abandon Changes') {
            $this->redirect('/requests');
        } else if ($in['submitType'] == 'Cancel Request') {
            $this->redirect('/requests');
        }
        exit;
    }

    protected function _saveRequest()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);

        if ($session->currentSavedRequestId !== null) {
            $id = $session->currentSavedRequestId;
            $user = Auth_UserModel::findCurrentUser();

            $savedRequestRegistry = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests');
            $savedRequests = $savedRequestRegistry->getData();
            unset($savedRequests[$id]);
            $savedRequestRegistry->setData($savedRequests);
            $collection = new Unl_Model_Collection('RegistryModel');
            $collection[] = $savedRequestRegistry;
            RegistryModel::save($collection);
            unset($session->currentSavedRequestId);
        }

        Requests_RequestModel::save($session->request);
        $session->fourYearPlan->setRequest($session->request);
        FourYearPlans_FourYearPlanModel::save($session->fourYearPlan);
    }

    protected function _startApproval()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        Requests_ApprovalChainModel::consider($session->request);
    }

    public function loadAction()
    {
        $in = $this->getRequest()->getParams();

        $tempId = hash('md5', openssl_random_pseudo_bytes(16));
        $session = new Zend_Session_Namespace('Edit Request ' . $tempId);

        if ($in['sessionId'] !== null) {
            $id = $in['sessionId'];
            $user = Auth_UserModel::findCurrentUser();
            $savedRequests = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests')->getData();
            $request = $savedRequests[$id]['request'];
            $fourYearPlan = $savedRequests[$id]['fourYearPlan'];
            $previousFourYearPlan = NULL;
            /*
            if ($this->_isSavedRequestStale($request, $course)) {
                $controllerSession = new Zend_Session_Namespace(__CLASS__);
                $controllerSession->errorMessage = 'Unfortunately another request modifying this course has been submitted since you saved your request.  You will need to create a new request to continue.';
                $this->_redirect('/courses/edit/error');
                exit;
            }
            */
            $session->currentSavedRequestId = $id;
        } else {
            $id = $in['id'];
            $request = Requests_RequestModel::find($id);
            $fourYearPlan = FourYearPlans_FourYearPlanModel::findLatestOfRequest($request);
            $previousFourYearPlan = FourYearPlans_FourYearPlanModel::findParentOfRequest($request);
            $fourYearPlan = clone $fourYearPlan;
            unset($session->currentSavedRequestId);
        }

        $session->request = $request;
        $session->fourYearPlan = $fourYearPlan;
        $session->previousFourYearPlan = $previousFourYearPlan;

        $this->_redirect('/fouryearplans/edit/index/id/' . $tempId);
    }

    protected function _saveForLater()
    {
        $in = $this->getAllParams();
        $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
        $user = Auth_UserModel::findCurrentUser();
        $savedRequestRegistry = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests');
        $savedRequests = $savedRequestRegistry->getData();

        if (!Unl_Util::isArray($savedRequests)) {
            $savedRequests = array();
        }
        $newSavedRequest = array(
            'request'      => $session->request,
            'fourYearPlan' => $session->fourYearPlan,
        );

        if ($session->currentSavedRequestId !== null) {
            $id = $session->currentSavedRequestId;
            $savedRequests[$id] = $newSavedRequest;
        } else {
            $savedRequests[] = $newSavedRequest;
        }

        $savedRequestRegistry->setData($savedRequests);
        $collection = new Unl_Model_Collection('RegistryModel');
        $collection[] = $savedRequestRegistry;
        RegistryModel::save($collection);
    }
}
