<?php

require_once APPLICATION_PATH . '/modules/requests/controllers/ViewController.php';

class FourYearPlans_ViewController extends Requests_ViewController
{
    public function indexAction()
    {
        $this->_commonIndexAction();

        $request = $this->view->request;

        $in = $this->getAllParams();
        if ($in['type'] == 'session' && isset($in['id']) && $in['id']) {
            $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
            $fourYearPlan = $session->fourYearPlan;
            $previousFourYearPlan = $session->previousFourYearPlan;

            if ($request->getId()) {
                $this->view->initialRequest = false;
            } else {
                $this->view->initialRequest = true;
            }

            $this->view->preview = true;
        } else {
            $fourYearPlan = FourYearPlans_FourYearPlanModel::findLatestOfRequest($request);
            if (in_array($request->getType(), array('ChangeFourYearPlan'))) {
                $previousFourYearPlan = FourYearPlans_FourYearPlanModel::findParentOfRequest($request);
            }
            $this->view->initialRequest = false;
            $this->view->preview = false;
        }

        $courseIds = array(-1);
        foreach ($fourYearPlan->getChangedCourses($previousFourYearPlan) as $planCourse) {
            if (!$planCourse['course']) {
                continue;
            }
            $courseIds[$planCourse['course']] = $planCourse['course'];
        }
        $planCourses = Courses_CourseModel::findByCourseId($courseIds);

        $this->_totalHoursForFyp($fourYearPlan, $planCourses);

        $this->view->request = $request;
        $this->view->id = $in['id'];
        $this->view->isRequestValid = $request->isValid();
        $this->view->fourYearPlan = $fourYearPlan;
        $this->view->previousFourYearPlan = $previousFourYearPlan;
        $this->view->planCourses = $planCourses;
    }

    public function feedAction()
    {
        $plans = array();

        $courseIds = array();
        foreach (FourYearPlans_FourYearPlanModel::findAllActive() as $fourYearPlan) {
            foreach ($fourYearPlan->getCourses() as $course) {
                if ($course['course']) {
                    $courseIds[$course['course']] = $course['course'];
                }
            }
        }
        $courseDetails = Courses_CourseModel::findByCourseId($courseIds);

        foreach (FourYearPlans_FourYearPlanModel::findAllActive() as $fourYearPlan) {
            $plan = array();
            $plan['creq-id'] = $fourYearPlan->getFourYearPlanId();
            $plan['major'] = $fourYearPlan->getMajor();

            foreach ($fourYearPlan->getCourses() as $course) {
                if ($course['course'] && $courseDetails[$course['course']]) {
                    $courseDetail = $courseDetails[$course['course']];
                    if (!$course['text']) {
                        $course['text'] = $courseDetail->getCourseCode();
                    }
                    if (!$course['hours']) {
                        $hours = $courseDetail->getCreditsText();
                        $hours = substr($hours, 1, strpos($hours, 'cr') - 1);
                        $hours = trim($hours);
                        $course['hours'] = $hours;
                    }
                    $course['title'] = $courseDetail->getTitle();
                }

                $plan['concentrations'][$course['concentration']]['semesters'][$course['semester']]['courses'][] = array(
                    'creq-id' => $course['course'],
                    'course' => $course['text'],
                    'hours' => $course['hours'],
                    'title' => $course['title'],
                );
            }

            foreach ($fourYearPlan->getNotes() as $concentration => $notes) {
                $plan['concentrations'][$concentration]['notes'] = $notes;
            }

            $plans[] = $plan;
        }

        header('Content-type: application/json');
        echo json_encode($plans);
        exit;
    }

    public function listAction()
    {
        $this->view->fourYearPlans = FourYearPlans_FourYearPlanModel::findAllActive();
    }

    public function publicAction()
    {
        $fourYearPlan = FourYearPlans_FourYearPlanModel::findByFourYearPlanId($this->getParam('id'));
        $requests = FourYearPlans_FourYearPlanModel::findAllRequestsForMajor($fourYearPlan);
        $requestFourYearPlans = FourYearPlans_FourYearPlanModel::findLatestOfRequest($requests);
        $requestParentFourYearPlans = FourYearPlans_FourYearPlanModel::findParentOfRequest($requests);

        $courseIds = array(-1);
        foreach ($requestFourYearPlans as $requestId => $requestFourYearPlan) {
            foreach ($requestFourYearPlan->getChangedCourses() as $planCourse) {
                if (!$planCourse['course']) {
                    continue;
                }
                $courseIds[$planCourse['course']] = $planCourse['course'];
            }
        }
        $planCourses = Courses_CourseModel::findByCourseId($courseIds);

        $this->_totalHoursForFyp($fourYearPlan, $planCourses);

        $this->view->fourYearPlan = $fourYearPlan;
        $this->view->requests = $requests;
        $this->view->requestFourYearPlans = $requestFourYearPlans;
        $this->view->requestParentFourYearPlans = $requestParentFourYearPlans;
        $this->view->planCourses = $planCourses;
    }

    protected function _totalHoursForFyp(FourYearPlans_FourYearPlanModel $fourYearPlan, $planCourses)
    {
        foreach ($fourYearPlan->getConcentrations as $concentration) {
            $totalHoursMin[$concentration] = 0;
            $totalHoursMax[$concentration] = 0;
        }
        foreach ($fourYearPlan->getCourses() as $course) {
            if (!$course['course'] || $course['hours']) {
                if (Zend_Validate::is($course['hours'], 'Float')) {
                    $totalHoursMin[$course['concentration']] += $course['hours'];
                    $totalHoursMax[$course['concentration']] += $course['hours'];
                } else if (preg_match('/(\d*) *- *(\d)/', $course['hours'], $matches)) {
                    $totalHoursMin[$course['concentration']] += $matches[1];
                    $totalHoursMax[$course['concentration']] += $matches[2];
                }
            } else {
                $minHours = 99;
                $maxHours = 0;
                $credits = $planCourses[$course['course']]->getCredits();
                foreach ($credits as $credit) {
                    if (in_array($credit['type'], array(1,2)) && $credit['hours'] < $minHours) {
                        $minHours = $credit['hours'];
                    }
                    if (in_array($credit['type'], array(1,3)) && $credit['hours'] > $maxHours) {
                        $maxHours = $credit['hours'];
                    }
                }

                if ($minHours < 99) {
                    $totalHoursMin[$course['concentration']] += $minHours;
                }
                if ($minHours > 0) {
                    $totalHoursMax[$course['concentration']] += $maxHours;
                }
            }
        }

        $this->view->totalHours = array();
        foreach ($fourYearPlan->getConcentrations() as $concentration) {
            if ($totalHoursMin[$concentration] == $totalHoursMax[$concentration]) {
                $this->view->totalHours[$concentration] = $totalHoursMin[$concentration];
            } else {
                $this->view->totalHours[$concentration] = $totalHoursMin[$concentration] . '-' . $totalHoursMax[$concentration];
            }
        }
    }
}
