<?php

require_once ('Unl/Model.php');

class FourYearPlans_FourYearPlanModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('a' => 'creqAssets'));
        $select->join(array('g' => 'creqFourYearPlanGenerations'), 'a.assetId = g.assetId');

        if (Unl_Util::isArray($id)) {
            if (count($id) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('fourYearPlanGenerationId IN(?)', $id);
        } else {
            $select->where('fourYearPlanGenerationId = ?', $id);
        }

        $records = $select->query()->fetchAll();
        $fourYearPlanIds = array();
        $fourYearPlans = array();
        foreach ($records as $record) {
            $record['courses'] = array();
            $fourYearPlanId = $record['fourYearPlanGenerationId'];
            $fourYearPlanIds[] = $fourYearPlanId;
            $fourYearPlans[$fourYearPlanId] = $record;
        }

        // Courses
        $select = new Zend_Db_Select($db);
        $select->from(array('fc' => 'creqFourYearPlanCourses'));
        $select->joinLeft(array('c' => 'creqCourses'), 'fc.course = c.courseId', array());
        $select->joinLeft(array('x' => 'creqCourseCrosslistings'), 'c.currentGeneration = x.generation', array());
        $select->joinLeft(array('cc' => 'creqCourseCodes'), 'x.courseCode = cc.courseCodeId', array('subject', 'courseLetter', 'courseNumber'));
        $select->order('fc.fourYearPlanCourseId');
        $select->where('fc.generation IN (?)', $fourYearPlanIds);

        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $fourYearPlanId = $record['generation'];
            $fourYearPlanCourseId = $record['fourYearPlanCourseId'];
            $fourYearPlans[$fourYearPlanId]['courses'][$fourYearPlanCourseId] = $record;
        }

        // Notes
        $select = new Zend_Db_Select($db);
        $select->from(array('fn' => 'creqFourYearPlanNotes'));
        $select->where('fn.generation IN (?)', $fourYearPlanIds);

        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $fourYearPlans[$record['generation']]['notes'][$record['concentration']] = $record['notes'];
        }

        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($fourYearPlans as $fourYearPlan) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($fourYearPlan));
            $object->_setClean();
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects[$id];
        }
    }

    static public function fetchNew()
    {
        $data = array(
            'assetId'      => null,
            'creationTime' => null,
            'modifiedTime' => null,

            'fourYearPlanGenerationId' => null,
            'parent'                   => null,
            'fourYearPlan'             => null,
            'request'                  => null,
            'type'                     => 'proposed',
            'removed'                  => 'no',
            'majorId'                  => null,
            'courses'                  => array(),
            'notes'                    => array(),
        );

        $new = new self($data);
        return $new;
    }

    static public function findByFourYearPlanId($fourYearPlanId)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('fourYearPlanGenerationId'));
        $select->join(array('f' => 'creqFourYearPlans'), 'g.fourYearPlanGenerationId = f.currentGeneration', array());
        if (Unl_Util::isArray($fourYearPlanId)) {
            $select->where('f.fourYearPlanId IN (?)', $fourYearPlanId);
        } else {
            $select->where('f.fourYearPlanId = ?', $fourYearPlanId);
        }

        $records = $select->query()->fetchAll();
        if (!Unl_Util::isArray($fourYearPlanId)) {
            if (count($records) == 0) {
                return null;
            }
            $generationId = $records[0]['fourYearPlanGenerationId'];
        } else {
            $generationId = array();
            foreach ($records as $record) {
                $generationId[] = $record['fourYearPlanGenerationId'];
            }
        }

        $models = self::find($generationId);

        if (!Unl_Util::isArray($fourYearPlanId)) {
            return $models;
        }

        $indexedModels = new Unl_Model_Collection(__CLASS__);
        foreach ($models as $model) {
            $indexedModels[$model->getFourYearPlanId()] = $model;
        }

        return $indexedModels;
    }

    static public function findParentOfRequest($request)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $requestId = array();
            foreach ($request as $aRequest) {
                $requestId[] = $aRequest->getId();
            }
        } else {
            $requestId = $request->getId();
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('request'));
        $select->join(array('p' => 'creqFourYearPlanGenerations'), 'g.parent = p.fourYearPlanGenerationId', array('fourYearPlanGenerationId'));
        $select->where('p.request != g.request OR p.request IS NULL');
        $select->where('p.type = ?', 'official');
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $fourYearPlanIds = array();
        foreach ($records as $record) {
            $rId = $record['request'];
            $fourYearPlanIds[$rId] = $record['fourYearPlanGenerationId'];
        }

        self::find($fourYearPlanIds);

        if (Unl_Util::isArray($requestId)) {
            $rv = array();
            foreach ($requestId as $rId) {
                $rv[$rId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $fourYearPlanIds[$rId]);
            }
            return $rv;
        } else {
            return Unl_Model_Registry::getInstance()->get(__CLASS__, $fourYearPlanIds[$requestId]);
        }
    }

    static public function findLatestOfRequest($request)
    {
        /**
         * TODO:
         * Make sure that if two of the requests given are on subsequent generations,
         * that we can get the latest for EACH and not just the later one.
         */

        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $requestId = array();
            foreach ($request as $aRequest) {
                $requestId[] = $aRequest->getId();
            }
        } else {
            $requestId = $request->getId();
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('parent', 'request'));
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $parentIds = array();
        foreach ($records as $record) {
            if (!Unl_Util::isArray($parentIds[$record['request']])) {
                $parentIds[$record['request']] = array();
            }
            if (!$record['parent']) {
                continue;
            }
            $parentIds[$record['request']][] = $record['parent'];
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('fourYearPlanGenerationId', 'request'));
        $whereOrs = array();
        foreach ($parentIds as $rId => $pIds) {
            $whereOr = $db->quoteInto('g.request = ? ', $rId);
            if (count($pIds) > 0) {
                $whereOr .= $db->quoteInto('AND g.fourYearPlanGenerationId NOT IN(?)', $pIds);
            }
            $whereOrs[] = $whereOr;
        }
        $whereOrs = '(' . implode(') OR (', $whereOrs) . ')';
        $select->where($whereOrs);
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $fourYearPlanIds = array();
        foreach ($records as $record) {
            $rId = $record['request'];
            $fourYearPlanIds[$rId] = $record['fourYearPlanGenerationId'];
        }

        self::find($fourYearPlanIds);

        if (Unl_Util::isArray($requestId)) {
            $rv = new Unl_Model_Collection(__CLASS__);
            foreach ($requestId as $rId) {
                $rv[$rId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $fourYearPlanIds[$rId]);
            }
            return $rv;
        } else {
            return Unl_Model_Registry::getInstance()->get(__CLASS__, $fourYearPlanIds[$requestId]);
        }
    }

    static public function findAllActive()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('fourYearPlanGenerationId'));
        $select->join(array('p' => 'creqFourYearPlans'), 'g.fourYearPlanGenerationId = p.currentGeneration', array());
        $select->where('g.removed = "no"');

        $records = $select->query()->fetchAll();
        $generationIds = array();
        foreach ($records as $row) {
            $generationIds[] = $row['fourYearPlanGenerationId'];
        }
        return self::find($generationIds);
    }

    static public function findByMajorId($majorId)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqFourYearPlanGenerations'), array('fourYearPlanGenerationId'));
        $select->join(array('f' => 'creqFourYearPlans'), 'g.fourYearPlanGenerationId = f.currentGeneration', array());
        $select->where('g.majorId = ?', $majorId);

        $records = $select->query()->fetchAll();
        if (count($records) == 0) {
            return null;
        }
        $generationId = $records[0]['fourYearPlanGenerationId'];
        return self::find($generationId);
    }

    static public function findAllRequestsForMajor($fourYearPlans, $completeApproveOnly = true)
    {
        $multiple = true;
        if (!Unl_Util::isArray($fourYearPlans)) {
            if (!($fourYearPlans instanceof self)) {
                throw new Exception('You must pass a ' . __CLASS__ . ' object or a collection of them.');
            }
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $fourYearPlans;
            $fourYearPlans = $collection;
            $multiple = false;
        }

        if (count($fourYearPlans) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->distinct();
        $select->from(array('g1' => 'creqFourYearPlanGenerations'), array('fourYearPlanGenerationId'));
        $select->join(array('g2' => 'creqFourYearPlanGenerations'), 'g1.fourYearPlan = g2.fourYearPlan', array('request'));
        $select->join(array('r' => 'creqRequests'), 'g2.request = r.requestId', array());
        $select->where('g1.fourYearPlanGenerationId IN (?)', $fourYearPlans->getId());
        if ($completeApproveOnly) {
            $select->where('r.complete = ?', 'yes');
            $select->where('r.state = ?', 'Approve');
        }
        $records = $select->query()->fetchAll();

        $requestIds = array();
        foreach ($records as $row) {
            $requestIds[] = $row['request'];
        }

        $requests = Requests_RequestModel::find($requestIds);

        $fourYearPlanRequests = array();
        foreach ($fourYearPlans as $fourYearPlan) {
            $fourYearPlanRequests[$fourYearPlan->getId()] = new Unl_Model_Collection('Requests_RequestModel');
        }
        foreach ($records as $row) {
            $fourYearPlanRequests[$row['fourYearPlanGenerationId']][$row['request']] = $requests[$row['request']];
        }

        if (!$multiple) {
            return array_pop($fourYearPlanRequests);
        } else {
            return $fourYearPlanRequests;
        }

    }

    static public function findActiveRequestByMajorId($majorId)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('f' => 'creqFourYearPlanGenerations'), array());
        $select->join(array('r' => 'creqRequests'), 'f.request = r.requestId', array('requestId'));
        $select->where('f.majorId = ?', $majorId);
        $select->where('r.complete = ?', 'no');

        $records = $select->query()->fetchAll();
        $requestIds = array();
        foreach ($records as $record) {
            $requestIds[$record['requestId']] = $record['requestId'];
        }

        return Requests_RequestModel::find($requestIds);
    }

    static public function save($models)
    {

        if (!Unl_Util::isArray($models)) {
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $models;
            $models = $collection;
        }

        self::_migrateMajorIds($models);

        $db = Zend_Registry::get('db');
        $db->beginTransaction();

        $unsavedData = array();
        foreach ($models as $id => $model) {
            $unsavedData[$id] = $model->_data;
        }

        try {
            $modelsToInsert = array();
            $modelsToUpdate = array();
            foreach ($models as $model) {
                if ($model->getId()) {
                    $modelsToUpdate[] = $model;
                } else {
                    $modelsToInsert[] = $model;
                }
            }
            self::_insertBaseRows($modelsToInsert);
            self::_updateBaseRows($modelsToUpdate);


            $coursesToInsert = array();
            $coursesToUpdate = array();
            $coursesToDelete = array();
            foreach ($models as $model) {
                if (!Unl_Util::isArray($model->_data['courses'])) {
                    $model->_data['courses'] = array();
                }
                foreach ($model->_data['courses'] as $id => &$course) {
                    $course['generation'] = $model->_data['fourYearPlanGenerationId'];

                    $courseRef = array(
                        'model' => $model,
                        'id'    => $id
                    );

                    if ($course['fourYearPlanCourseId']) {
                        $coursesToUpdate[] = $courseRef;
                    } else {
                        $coursesToInsert[] = $courseRef;
                    }
                }

                if (is_array($model->_cleanData['courses'])) {
                    $coursesToDelete += array_diff_key($model->_cleanData['courses'], $model->_data['courses']);
                }
            }
            self::_insertCourses($coursesToInsert);
            self::_updateCourses($coursesToUpdate);
            self::_deleteCourses($coursesToDelete);

            $notesToInsert = array();
            $notesToUpdate = array();
            $notesToDelete = array();
            foreach ($models as $model) {
                $existingNotes = array();
                foreach ($model->_cleanData['notes'] as $concentration => $notes) {
                    $existingNotes[] = array(
                        'generation' => $model->_data['fourYearPlanGenerationId'],
                        'concentration' => $concentration,
                        'notes' => $notes,
                    );
                }
                $newNotes = array();
                foreach ($model->_data['notes'] as $concentration => $notes) {
                    $newNotes[] = array(
                        'generation' => $model->_data['fourYearPlanGenerationId'],
                        'concentration' => $concentration,
                        'notes' => $notes,
                    );
                }

                $notesToInsert += array_diff($newNotes, $existingNotes);
                $notesToUpdate += array_intersect($newNotes, $existingNotes);
                $notesToDelete += array_diff($existingNotes, $newNotes);
            }
            self::_insertNotes($notesToInsert);
            self::_updateNotes($notesToUpdate);
            self::_deleteNotes($notesToDelete);

            $db->commit();
        } catch (Exception $e) {
            $db->rollback();
            foreach ($models as $id => $model) {
                $model->_data = $unsavedData[$id];
            }
            throw $e;
        }
    }

    static protected function _migrateMajorIds($models)
    {
        $majors = Bulletin_UtilityModel::getAllMajors();

        foreach ($models as $model) {
            if (!$model->_data['majorId'] && $model->_data['major']) {
                $model->_data['majorId'] = array_search($model->_data['major'], $majors);
            }
        }
    }

    static public function _insertBaseRows($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqAssets (creationTime, modifiedTime) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', time())
                        . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['assetId'] = $lastId;
            $lastId++;
        }

        $sql = 'INSERT INTO creqFourYearPlans (currentGeneration) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            if ($model->_data['fourYearPlan']) {
                continue;
            }
            $sqlParts[] = '(NULL)';
        }
        if (count($sqlParts) > 0) {
            $sql .= implode(', ', $sqlParts);
            $db->query($sql);

            $lastId = $db->lastInsertId();
            foreach ($models as $model) {
                if ($model->_data['fourYearPlan']) {
                    continue;
                }
                $model->_data['fourYearPlan'] = $lastId;
                $model->_data['fourYearPlanId'] = $lastId;
                $lastId++;
            }
        }

        $sql = 'INSERT INTO creqFourYearPlanGenerations (assetId, parent, fourYearPlan, request, type, removed, majorId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['assetId'])
                        . $db->quoteInto('?, ' , $model->_data['parent'])
                        . $db->quoteInto('?, ' , $model->_data['fourYearPlan'])
                        . $db->quoteInto('?, ' , $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['type'])
                        . $db->quoteInto('?, ' , $model->_data['removed'])
                        . $db->quoteInto('?)'  , $model->_data['majorId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['fourYearPlanGenerationId'] = $lastId;
            $model->_data['generation'] = $lastId;
            $lastId++;
        }
    }

    static public function _updateBaseRows($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');


        // Update the assets table
        $sql = 'CREATE TEMPORARY TABLE creqAssetsUpdate '
            . 'SELECT * FROM creqAssets LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqAssetsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['assetId'])
                . $db->quoteInto('?, ' , time())
                . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqAssets AS a, '
             . '       creqAssetsUpdate AS b '
             . 'SET a.modifiedTime = b.modifiedTime '
             . 'WHERE a.assetId = b.assetId ';
        $db->query($sql);
        $db->query('DROP TABLE creqAssetsUpdate');


        // Update the fourYearPlans table
        $sql = 'CREATE TEMPORARY TABLE creqFourYearPlansUpdate '
             . 'SELECT * FROM creqFourYearPlans LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqFourYearPlansUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['fourYearPlan'])
                . $db->quoteInto('?)'  , $model->_data['currentGeneration']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqFourYearPlans AS a, '
             . '       creqFourYearPlansUpdate AS b '
             . 'SET a.currentGeneration = b.currentGeneration '
             . 'WHERE a.fourYearPlanId = b.fourYearPlanId ';
        $db->query($sql);
        $db->query('DROP TABLE creqFourYearPlansUpdate');


        // Update the fourYearPlanGenerations table
        $sql = 'CREATE TEMPORARY TABLE creqFourYearPlanGenerationsUpdate '
             . 'SELECT * FROM creqFourYearPlanGenerations LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqFourYearPlanGenerationsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['fourYearPlanGenerationId'])
                . $db->quoteInto('?, ' , $model->_data['assetId'])
                . $db->quoteInto('?, ' , $model->_data['parent'])
                . $db->quoteInto('?, ' , $model->_data['fourYearPlan'])
                . $db->quoteInto('?, ' , $model->_data['request'])
                . $db->quoteInto('?, ' , $model->_data['type'])
                . $db->quoteInto('?, ' , $model->_data['removed'])
                . $db->quoteInto('?)'  , $model->_data['majorId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqFourYearPlanGenerations AS a, '
             . '       creqFourYearPlanGenerationsUpdate AS b '
             . 'SET a.assetId       = b.assetId, '
             . '    a.parent        = b.parent, '
             . '    a.fourYearPlan  = b.fourYearPlan, '
             . '    a.request       = b.request, '
             . '    a.type          = b.type, '
             . '    a.removed       = b.removed, '
             . '    a.majorId       = b.majorId '
             . 'WHERE a.fourYearPlanGenerationId = b.fourYearPlanGenerationId ';
        $db->query($sql);
        $db->query('DROP TABLE creqFourYearPlanGenerationsUpdate');
    }

    static public function _insertCourses($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqFourYearPlanCourses (generation, concentration, semester, course, text, hours) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $course = $model['model']->_data['courses'][$model['id']];
            $sqlParts[] = $db->quoteInto('(?, ', $course['generation'])
                        . $db->quoteInto('?, ' , $course['concentration'])
                        . $db->quoteInto('?, ' , $course['semester'])
                        . $db->quoteInto('?, ' , $course['course'])
                        . $db->quoteInto('?, ' , $course['text'])
                        . $db->quoteInto('?)'  , $course['hours']);
        }
        $sql .= implode(', ', $sqlParts);

        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model['model']->_data['courses'][$model['id']]['fourYearPlanCourseId'] = $lastId;
            $lastId++;
        }
    }

    static public function _updateCourses($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqFourYearPlanCoursesUpdate '
             . 'SELECT * FROM creqFourYearPlanCourses LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqFourYearPlanCoursesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $course = $model['model']->_data['courses'][$model['id']];
            $sqlParts[] = $db->quoteInto('(?, ', $course['fourYearPlanCourseId'])
                        . $db->quoteInto('?, ' , $course['generation'])
                        . $db->quoteInto('?, ' , $course['concentration'])
                        . $db->quoteInto('?, ' , $course['semester'])
                        . $db->quoteInto('?, ' , $course['course'])
                        . $db->quoteInto('?, ' , $course['text'])
                        . $db->quoteInto('?)'  , $course['hours']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqFourYearPlanCourses AS a, '
             . '       creqFourYearPlanCoursesUpdate AS b '
             . 'SET a.generation    = b.generation, '
             . '    a.concentration = b.concentration, '
             . '    a.semester      = b.semester, '
             . '    a.course        = b.course, '
             . '    a.text          = b.text, '
             . '    a.hours         = b.hours '
             . 'WHERE a.fourYearPlanCourseId = b.fourYearPlanCourseId ';

        $db->query($sql);
        $db->query('DROP TABLE creqFourYearPlanCoursesUpdate');
    }

    static public function _deleteCourses($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = $db->quoteInto('DELETE FROM creqFourYearPlanCourses WHERE fourYearPlanCourseId IN (?)', array_keys($models));
        $db->query($sql);
    }

    static public function _insertNotes($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqFourYearPlanNotes (generation, concentration, notes) VALUES ';
        $sqlParts = array();
        foreach ($models as $notes) {
            $sqlParts[] = $db->quoteInto('(?, ', $notes['generation'])
                        . $db->quoteInto('?, ' , $notes['concentration'])
                        . $db->quoteInto('?)'  , $notes['notes']);
        }
        $sql .= implode(', ', $sqlParts);

        $db->query($sql);
    }

    static public function _updateNotes($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqFourYearPlanNotesUpdate '
             . 'SELECT * FROM creqFourYearPlanNotes LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqFourYearPlanNotesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $notes) {
            $sqlParts[] = $db->quoteInto('(?, ', $notes['generation'])
                        . $db->quoteInto('?, ' , $notes['concentration'])
                        . $db->quoteInto('?)'  , $notes['notes']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqFourYearPlanNotes AS a, '
             . '       creqFourYearPlanNotesUpdate AS b '
             . 'SET a.notes = b.notes '
             . 'WHERE (a.generation, a.concentration) = (b.generation, b.concentration) ';

        $db->query($sql);
        $db->query('DROP TABLE creqFourYearPlanNotesUpdate');
    }

    static public function _deleteNotes($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'DELETE FROM creqFourYearPlanNotes '
             . 'WHERE (generation, concentration) IN '
             ;

        $sqlParts = array();
        foreach ($models as $notes) {
            $sqlParts[] = $db->quoteInto('(?, ', $notes['generation'])
                        . $db->quoteInto('?)'  , $notes['concentration']);
        }
        $sql .= '(' . implode(', ', $sqlParts) . ')';

        $db->query($sql);
    }

    public function __clone()
    {
        $this->_data['parent'] = $this->getId();
        $this->_data['type'] = 'proposed';

        unset($this->_data['assetId']);
        unset($this->_data['fourYearPlanGenerationId']);

        foreach ($this->_data['courses'] as &$course) {
            unset ($course['fourYearPlanCourseId']);
            unset ($course['generation']);
        }

        unset($this->_cleanData);
    }

    static public function makeCurrent($fourYearPlans)
    {
        if (!Unl_Util::isArray($fourYearPlans)) {
            if (!($fourYearPlans instanceof self)) {
                throw Exception(__CLASS__ . '::' . __METHOD__ . '(): invalid argument.');
            }
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $fourYearPlans;
            $fourYearPlans = $collection;
        }

        if (count($fourYearPlans) == 0) {
            return;
        }

        $fypOfficialGenerations = array();
        foreach ($fourYearPlans as $fourYearPlan) {
            $fourYearPlanId = $fourYearPlan->_data['fourYearPlan'];
            $generationId = $fourYearPlan->getId();
            $fypOfficialGenerations[$fourYearPlanId] = $generationId;
        }

        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqFourYearPlansUpdate '
             . 'SELECT * FROM creqFourYearPlans LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqFourYearPlansUpdate VALUES ';
        $sqlParts = array();
        foreach ($fypOfficialGenerations as $fourYearPlanId => $generationId) {
            $sqlParts[] = $db->quoteInto('(?, ', $fourYearPlanId)
                        . $db->quoteInto('?)'  , $generationId);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqFourYearPlans AS a, '
             . '       creqFourYearPlansUpdate AS b '
             . 'SET a.currentGeneration = b.currentGeneration '
             . 'WHERE a.fourYearPlanId = b.fourYearPlanId ';

        $db->query($sql);
        $db->query('DROP TABLE creqFourYearPlansUpdate');
    }

    public function getId()
    {
        return $this->_data['fourYearPlanGenerationId'];
    }

    public function getAssetId()
    {
        return $this->_data['assetId'];
    }

    public function getParentId()
    {
        return $this->_data['parent'];
    }

    public function getFourYearPlanId()
    {
        return $this->_data['fourYearPlan'];
    }

    public function getRequest()
    {
        return $this->_data['request'];
    }

    public function setRequest(Requests_RequestModel $request)
    {
        $this->_data['request'] = $request->getId();
    }

    public function getType()
    {
        return $this->_data['type'];
    }

    public function setType($type)
    {
        $this->_data['type'] = $type;
    }

    public function getRemoved()
    {
        if ($this->_data['removed'] == 'yes') {
            return true;
        }
        return false;
    }

    public function setRemoved($removed = true)
    {
        if ($removed) {
            $this->_data['removed'] = 'yes';
        } else {
            $this->_data['removed'] = 'no';
        }
    }

    static protected $_majorCollegeMap = array();
    static protected function _getMajorCollegeMap()
    {
        if (!self::$_majorCollegeMap) {
            $db = Zend_Registry::get('db');
            $select = new Zend_Db_Select($db);

            $select->from(array('c' => 'creqColleges'), array(
                'collegeId',
                'college'     => 'name',
                'collegeLong' => 'description'
            ));
            $select->join(array('m' => 'creqMajors'), 'c.collegeId = m.college', array(
                'majorId',
                'major' => 'name'
            ));

	    $select->where("m.removed = 'no'");

            foreach ($select->query()->fetchAll() as $row) {
                self::$_majorCollegeMap[$row['majorId']] = $row;
            }
        }
        return self::$_majorCollegeMap;
    }

    public function getCollege()
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['college'];
    }

    public function getCollegeLong()
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['collegeLong'];
    }

    public function getMajor()
    {
        if (!$this->_data['majorId'] && $this->_data['major']) {
            return $this->_data['major'];
        }

        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['major'];
    }

    public function setMajorId($majorId)
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$majorId])) {
            throw new Exception('Major not found.');
        }

        $this->_data['majorId'] = $majorId;
    }

    public function setMajor($major)
    {
        $majorId = NULL;
        foreach (self::_getMajorCollegeMap() as $item) {
            if ($item['major'] == $major) {
                $majorId = $item['majorId'];
            }
        }

        if (!$majorId) {
            throw new Exception ('The major "' . $major . '" does not exist.');
        }

        $this->_data['majorId'] = $majorId;
    }

    public function getConcentrations()
    {
        $concentrations = array();
        foreach ($this->_data['courses'] as $course) {
            $hyphenated = $course['concentration'];
            $hyphenated = strtr($hyphenated, array(' ' => '-'));
            $hyphenated = strtolower($hyphenated);
            $hyphenated = preg_replace('/[^a-z0-9\-]/', '', $hyphenated);
            $hyphenated = preg_replace('/-+/', '-', $hyphenated);
            $concentrations[$hyphenated] = $course['concentration'];
        }
        if (!$concentrations) {
            $concentrations = array('default' => 'Default');
        }

        return $concentrations;
    }

    public function getCourses()
    {
        return $this->_data['courses'];
    }

    public function clearCourses()
    {
        $this->_data['courses'] = array();
    }

    public function addCourse($concentration, $semester, $courseId, $hours = NULL)
    {
        $linkId = $this->_getCourseLinkId($concentration, $semester, $courseId);
        if (array_key_exists($linkId, $this->_data['courses'])) {
            $this->_data['courses'][$linkId]['concentration'] = $concentration;
            $this->_data['courses'][$linkId]['semester'] = $semester;
            $this->_data['courses'][$linkId]['hours'] = $hours;
        } else {
            $linkId = min(min(array_keys($this->_data['courses'])), 0) - 1;
            $this->_data['courses'][$linkId] = array(
                'generation'    => null,
                'concentration' => $concentration,
                'semester'      => $semester,
                'course'        => $courseId,
                'text'          => null,
                'hours'         => $hours,
            );
        }
    }

    public function removeCourse($concentration, $semester, $courseId)
    {
        $linkId = $this->_getCourseLinkId($concentration, $semester, $courseId);
        $this->removeCourseText($linkId);
    }

    protected function _getCourseLinkId($concentration, $semester, $courseId)
    {
        $linkId = null;
        foreach ($this->_data['courses'] as $id => $record) {
            if ($record['concentration'] == $concentration &&
                $record['semester'] == $semester &&
                $record['course'] == $courseId)
            {
                $linkId = $id;
            }
        }
        return $linkId;
    }

    public function addCourseText($concentration, $semester, $text, $hours, $linkId = null)
    {
        if (array_key_exists($linkId, $this->_data['courses'])) {
            if ($this->_data['courses'][$linkId]['course']) {
                throw new Exception('Cannot add text to a course link!');
            }
            $this->_data['courses'][$linkId]['concentration'] = $concentration;
            $this->_data['courses'][$linkId]['semester']      = $semester;
            $this->_data['courses'][$linkId]['text']          = $text;
            $this->_data['courses'][$linkId]['hours']         = $hours;
        } else {
            $linkId = min(min(array_keys($this->_data['courses'])), 0) - 1;
            $this->_data['courses'][$linkId] = array(
                'generation'    => null,
                'concentration' => $concentration,
                'semester'      => $semester,
                'course'        => null,
                'text'          => $text,
                'hours'         => $hours,
            );
        }
    }

    public function removeCourseText($linkId)
    {
        if (!$linkId) {
            return;
        }
        unset($this->_data['courses'][$linkId]);
    }

    public function getNotes($concentration = NULL)
    {
        if (!$concentration) {
            return $this->_data['notes'];
        }
        if (isset($this->_data['notes'][$concentration])) {
            return $this->_data['notes'][$concentration];
        }
        return NULL;
    }

    public function setNotes($concentration, $notes)
    {
        $this->_data['notes'][$concentration] = $notes;
    }

    public function clearNotes()
    {
        $this->_data['notes'] = array();
    }

    public function getChangedCourses(self $previous = NULL)
    {
        $proposedCourses = $this->getCourses();
        if (!$previous) {
            foreach ($proposedCourses as &$proposed) {
                $proposed['change'] = 'none';
            }
            unset($proposed);
            return $proposedCourses;
        }

        $currentCourses = $previous->getCourses();

        foreach ($proposedCourses as &$proposed) {
            $proposed['used'] = false;
        }
        unset($proposed);

        foreach ($currentCourses as &$current) {
            $found = false;
            foreach ($proposedCourses as &$proposed) {
                if ($proposed['used'] == false && $current['concentration'] == $proposed['concentration'] && $current['text'] == $proposed['text'] && $current['course'] == $proposed['course'] && $current['hours'] == $proposed['hours'] && $current['semester'] == $proposed['semester']) {
                    $proposed['used'] = true;
                    $found = true;
                    break;
                }
            }
            unset($proposed);

            if ($found) {
                $current['change'] = 'none';
            } else {
                $current['change'] = 'removed';
            }
        }
        unset($current);


        foreach ($proposedCourses as $linkId => $proposed) {
            if ($proposed['used']) {
                continue;
            }
            $proposed['change'] = 'added';
            $currentCourses[$linkId] = $proposed;
        }


        return $currentCourses;
    }
}

