<?php

require_once ('Unl/Model.php');

class LearningOutcomes_LearningOutcomeModel extends Unl_Model
{
    static $outcomes = array(
        '1'     => 'Learning Outcome One',
        '2'     => 'Learning Outcome Two',
        '3'     => 'Learning Outcome Three',
        '4'     => 'Learning Outcome Four',
        '5'     => 'Learning Outcome Five',
        '6'     => 'Learning Outcome Six',
        '7'     => 'Learning Outcome Seven',
        '8'     => 'Learning Outcome Eight',
        '9'     => 'Learning Outcome Nine',
        '10'    => 'Learning Outcome Ten',
        '11'    => 'Learning Outcome Eleven',
        '12'    => 'Learning Outcome Twelve',
        'notes' => 'Notes',
    );

    static public function find($id)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('a' => 'creqAssets'));
        $select->join(array('l' => 'creqLearningOutcomeGenerations'), 'a.assetId = l.assetId');

        if (Unl_Util::isArray($id)) {
            if (count($id) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('learningOutcomeGenerationId IN(?)', $id);
        } else {
            $select->where('learningOutcomeGenerationId = ?', $id);
        }

        $records = $select->query()->fetchAll();
        $learningOutcomeIds = array();
        $learningOutcomes = array();
        foreach ($records as $record) {
            $record['courses'] = array();
            $learningOutcomeId = $record['learningOutcomeGenerationId'];
            $learningOutcomeIds[] = $learningOutcomeId;
            $learningOutcomes[$learningOutcomeId] = $record;
        }

        // Details
        $select = new Zend_Db_Select($db);
        $select->from(array('ld' => 'creqLearningOutcomeDetails'));
        $select->order('ld.learningOutcomeDetailId');
        $select->where('ld.generation IN (?)', $learningOutcomeIds);

        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $learningOutcomeId = $record['generation'];
            $learningOutcomes[$learningOutcomeId]['details'][$record['concentration']][$record['name']] = array(
                'id'   => $record['learningOutcomeDetailId'],
                'text' => $record['text'],
            );
        }

        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($learningOutcomes as $learningOutcome) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($learningOutcome));
            $object->_setClean();
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects[$id];
        }
    }

    static public function fetchNew()
    {
        $data = array(
            'assetId'      => null,
            'creationTime' => null,
            'modifiedTime' => null,

            'learningOutcomeGenerationId' => null,
            'parent'                      => null,
            'learningOutcome'             => null,
            'request'                     => null,
            'type'                        => 'proposed',
            'removed'                     => 'no',
            'majorId'                     => null,
            'details'                     => array(),
        );

        $new = new self($data);
        return $new;
    }

    static public function findByLearningOutcomeId($learningOutcomeId)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('learningOutcomeGenerationId'));
        $select->join(array('l' => 'creqLearningOutcomes'), 'g.learningOutcomeGenerationId = l.currentGeneration', array());
        if (Unl_Util::isArray($learningOutcomeId)) {
            $select->where('l.learningOutcomeId IN (?)', $learningOutcomeId);
        } else {
            $select->where('l.learningOutcomeId = ?', $learningOutcomeId);
        }

        $records = $select->query()->fetchAll();
        if (!Unl_Util::isArray($learningOutcomeId)) {
            if (count($records) == 0) {
                return null;
            }
            $generationId = $records[0]['learningOutcomeGenerationId'];
        } else {
            $generationId = array();
            foreach ($records as $record) {
                $generationId[] = $record['learningOutcomeGenerationId'];
            }
        }

        $models = self::find($generationId);

        if (!Unl_Util::isArray($learningOutcomeId)) {
            return $models;
        }

        $indexedModels = new Unl_Model_Collection(__CLASS__);
        foreach ($models as $model) {
            $indexedModels[$model->getLearningOutcomeId()] = $model;
        }

        return $indexedModels;
    }

    static public function findParentOfRequest($request)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $requestId = array();
            foreach ($request as $aRequest) {
                $requestId[] = $aRequest->getId();
            }
        } else {
            $requestId = $request->getId();
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('request'));
        $select->join(array('l' => 'creqLearningOutcomeGenerations'), 'g.parent = l.learningOutcomeGenerationId', array('learningOutcomeGenerationId'));
        $select->where('l.request != g.request OR l.request IS NULL');
        $select->where('l.type = ?', 'official');
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $learningOutcomeIds = array();
        foreach ($records as $record) {
            $rId = $record['request'];
            $learningOutcomeIds[$rId] = $record['learningOutcomeGenerationId'];
        }

        self::find($learningOutcomeIds);

        if (Unl_Util::isArray($requestId)) {
            $rv = array();
            foreach ($requestId as $rId) {
                $rv[$rId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $learningOutcomeIds[$rId]);
            }
            return $rv;
        } else {
            return Unl_Model_Registry::getInstance()->get(__CLASS__, $learningOutcomeIds[$requestId]);
        }
    }

    static public function findLatestOfRequest($request)
    {
        /**
         * TODO:
         * Make sure that if two of the requests given are on subsequent generations,
         * that we can get the latest for EACH and not just the later one.
         */

        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $requestId = array();
            foreach ($request as $aRequest) {
                $requestId[] = $aRequest->getId();
            }
        } else {
            $requestId = $request->getId();
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('parent', 'request'));
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $parentIds = array();
        foreach ($records as $record) {
            if (!Unl_Util::isArray($parentIds[$record['request']])) {
                $parentIds[$record['request']] = array();
            }
            if (!$record['parent']) {
                continue;
            }
            $parentIds[$record['request']][] = $record['parent'];
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('learningOutcomeGenerationId', 'request'));
        $whereOrs = array();
        foreach ($parentIds as $rId => $pIds) {
            $whereOr = $db->quoteInto('g.request = ? ', $rId);
            if (count($pIds) > 0) {
                $whereOr .= $db->quoteInto('AND g.learningOutcomeGenerationId NOT IN(?)', $pIds);
            }
            $whereOrs[] = $whereOr;
        }
        $whereOrs = '(' . implode(') OR (', $whereOrs) . ')';
        $select->where($whereOrs);
        if (Unl_Util::isArray($request)) {
            $select->where('g.request IN(?)', $requestId);
        } else {
            $select->where('g.request = ?', $requestId);
        }

        $records = $select->query()->fetchAll();
        $learningOutcomeIds = array();
        foreach ($records as $record) {
            $rId = $record['request'];
            $learningOutcomeIds[$rId] = $record['learningOutcomeGenerationId'];
        }

        self::find($learningOutcomeIds);

        if (Unl_Util::isArray($requestId)) {
            $rv = new Unl_Model_Collection(__CLASS__);
            foreach ($requestId as $rId) {
                $rv[$rId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $learningOutcomeIds[$rId]);
            }
            return $rv;
        } else {
            return Unl_Model_Registry::getInstance()->get(__CLASS__, $learningOutcomeIds[$requestId]);
        }
    }

    static public function findAllActive()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('learningOutcomeGenerationId'));
        $select->join(array('l' => 'creqLearningOutcomes'), 'g.learningOutcomeGenerationId = l.currentGeneration', array());
        $select->where('g.removed = "no"');

        $records = $select->query()->fetchAll();
        $generationIds = array();
        foreach ($records as $row) {
            $generationIds[] = $row['learningOutcomeGenerationId'];
        }
        return self::find($generationIds);
    }

    static public function findByMajorId($majorId)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('g' => 'creqLearningOutcomeGenerations'), array('learningOutcomeGenerationId'));
        $select->join(array('f' => 'creqLearningOutcomes'), 'g.learningOutcomeGenerationId = f.currentGeneration', array());
        $select->where('g.majorId = ?', $majorId);

        $records = $select->query()->fetchAll();
        if (count($records) == 0) {
            return null;
        }
        $generationId = $records[0]['learningOutcomeGenerationId'];
        return self::find($generationId);
    }

    static public function findAllRequestsForMajor($learningOutcomes, $completeApproveOnly = true)
    {
        $multiple = true;
        if (!Unl_Util::isArray($learningOutcomes)) {
            if (!($learningOutcomes instanceof self)) {
                throw new Exception('You must pass a ' . __CLASS__ . ' object or a collection of them.');
            }
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $learningOutcomes;
            $learningOutcomes = $collection;
            $multiple = false;
        }

        if (count($learningOutcomes) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->distinct();
        $select->from(array('g1' => 'creqLearningOutcomeGenerations'), array('learningOutcomeGenerationId'));
        $select->join(array('g2' => 'creqLearningOutcomeGenerations'), 'g1.learningOutcome = g2.learningOutcome', array('request'));
        $select->join(array('r' => 'creqRequests'), 'g2.request = r.requestId', array());
        $select->where('g1.learningOutcomeGenerationId IN (?)', $learningOutcomes->getId());
        if ($completeApproveOnly) {
            $select->where('r.complete = ?', 'yes');
            $select->where('r.state = ?', 'Approve');
        }
        $records = $select->query()->fetchAll();

        $requestIds = array();
        foreach ($records as $row) {
            $requestIds[] = $row['request'];
        }

        $requests = Requests_RequestModel::find($requestIds);

        $learningOutcomeRequests = array();
        foreach ($learningOutcomes as $learningOutcome) {
            $learningOutcomeRequests[$learningOutcome->getId()] = new Unl_Model_Collection('Requests_RequestModel');
        }
        foreach ($records as $row) {
            $learningOutcomeRequests[$row['learningOutcomeGenerationId']][$row['request']] = $requests[$row['request']];
        }

        if (!$multiple) {
            return array_pop($learningOutcomeRequests);
        } else {
            return $learningOutcomeRequests;
        }

    }

    static public function findActiveRequestByMajorId($majorId)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('l' => 'creqLearningOutcomeGenerations'), array());
        $select->join(array('r' => 'creqRequests'), 'l.request = r.requestId', array('requestId'));
        $select->where('l.majorId = ?', $majorId);
        $select->where('r.complete = ?', 'no');

        $records = $select->query()->fetchAll();
        $requestIds = array();
        foreach ($records as $record) {
            $requestIds[$record['requestId']] = $record['requestId'];
        }

        return Requests_RequestModel::find($requestIds);
    }

    static public function save($models)
    {

        if (!Unl_Util::isArray($models)) {
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $models;
            $models = $collection;
        }

        $db = Zend_Registry::get('db');
        $db->beginTransaction();

        $unsavedData = array();
        foreach ($models as $id => $model) {
            $unsavedData[$id] = $model->_data;
        }

        try {
            $modelsToInsert = array();
            $modelsToUpdate = array();
            foreach ($models as $model) {
                if ($model->getId()) {
                    $modelsToUpdate[] = $model;
                } else {
                    $modelsToInsert[] = $model;
                }
            }
            self::_insertBaseRows($modelsToInsert);
            self::_updateBaseRows($modelsToUpdate);


            $detailsToInsert = array();
            $detailsToUpdate = array();
            $detailsToDelete = array();
            foreach ($models as $model) {
                if (!Unl_Util::isArray($model->_data['details'])) {
                    $model->_data['details'] = array();
                }
                foreach ($model->_data['details'] as $concentration => $details) {
                    foreach ($details as $name => $text) {
                        $detail = array(
                            'learningOutcome'         => $model,
                            'learningOutcomeDetailId' => isset($text['id']) ? $text['id'] : NULL,
                            'generation'              => $model->_data['learningOutcomeGenerationId'],
                            'concentration'           => $concentration,
                            'name'                    => $name,
                            'text'                    => $text['text'],
                        );
                        if ($detail['learningOutcomeDetailId']) {
                            $detailsToUpdate[] = $detail;
                        } else {
                            $detailsToInsert[] = $detail;
                        }
                    }

                }

            }
            self::_insertDetails($detailsToInsert);
            self::_updateDetails($detailsToUpdate);
            self::_deleteDetails($detailsToDelete);

            $db->commit();
        } catch (Exception $e) {
            $db->rollback();
            foreach ($models as $id => $model) {
                $model->_data = $unsavedData[$id];
            }
            throw $e;
        }
    }

    static public function _insertBaseRows($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqAssets (creationTime, modifiedTime) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', time())
                . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['assetId'] = $lastId;
            $lastId++;
        }

        $sql = 'INSERT INTO creqLearningOutcomes (currentGeneration) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            if ($model->_data['learningOutcome']) {
                continue;
            }
            $sqlParts[] = '(NULL)';
        }
        if (count($sqlParts) > 0) {
            $sql .= implode(', ', $sqlParts);
            $db->query($sql);

            $lastId = $db->lastInsertId();
            foreach ($models as $model) {
                if ($model->_data['learningOutcome']) {
                    continue;
                }
                $model->_data['learningOutcome'] = $lastId;
                $model->_data['learningOutcomeId'] = $lastId;
                $lastId++;
            }
        }

        $sql = 'INSERT INTO creqLearningOutcomeGenerations (assetId, parent, learningOutcome, request, type, removed, majorId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['assetId'])
                . $db->quoteInto('?, ' , $model->_data['parent'])
                . $db->quoteInto('?, ' , $model->_data['learningOutcome'])
                . $db->quoteInto('?, ' , $model->_data['request'])
                . $db->quoteInto('?, ' , $model->_data['type'])
                . $db->quoteInto('?, ' , $model->_data['removed'])
                . $db->quoteInto('?)'  , $model->_data['majorId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['learningOutcomeGenerationId'] = $lastId;
            $model->_data['generation'] = $lastId;
            $lastId++;
        }
    }

    static public function _updateBaseRows($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');


        // Update the assets table
        $sql = 'CREATE TEMPORARY TABLE creqAssetsUpdate '
             . 'SELECT * FROM creqAssets LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqAssetsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['assetId'])
                        . $db->quoteInto('?, ' , time())
                        . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqAssets AS a, '
             . '       creqAssetsUpdate AS b '
             . 'SET a.modifiedTime = b.modifiedTime '
             . 'WHERE a.assetId = b.assetId ';
        $db->query($sql);
        $db->query('DROP TABLE creqAssetsUpdate');


        // Update the learningOutcomes table
        $sql = 'CREATE TEMPORARY TABLE creqLearningOutcomesUpdate '
             . 'SELECT * FROM creqLearningOutcomes LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqLearningOutcomesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['learningOutcome'])
                        . $db->quoteInto('?)'  , $model->_data['currentGeneration']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqLearningOutcomes AS a, '
             . '       creqLearningOutcomesUpdate AS b '
             . 'SET a.currentGeneration = b.currentGeneration '
             . 'WHERE a.learningOutcomeId = b.learningOutcomeId ';
        $db->query($sql);
        $db->query('DROP TABLE creqLearningOutcomesUpdate');


        // Update the learningOutcomeGenerations table
        $sql = 'CREATE TEMPORARY TABLE creqLearningOutcomeGenerationsUpdate '
             . 'SELECT * FROM creqLearningOutcomeGenerations LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqLearningOutcomeGenerationsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['learningOutcomeGenerationId'])
                        . $db->quoteInto('?, ' , $model->_data['assetId'])
                        . $db->quoteInto('?, ' , $model->_data['parent'])
                        . $db->quoteInto('?, ' , $model->_data['learningOutcome'])
                        . $db->quoteInto('?, ' , $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['type'])
                        . $db->quoteInto('?, ' , $model->_data['removed'])
                        . $db->quoteInto('?)'  , $model->_data['majorId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqLearningOutcomeGenerations AS a, '
             . '       creqLearningOutcomeGenerationsUpdate AS b '
             . 'SET a.assetId          = b.assetId, '
             . '    a.parent           = b.parent, '
             . '    a.learningOutcome  = b.learningOutcome, '
             . '    a.request          = b.request, '
             . '    a.type             = b.type, '
             . '    a.removed          = b.removed, '
             . '    a.majorId          = b.majorId '
             . 'WHERE a.learningOutcomeGenerationId = b.learningOutcomeGenerationId ';
        $db->query($sql);
        $db->query('DROP TABLE creqLearningOutcomeGenerationsUpdate');
    }

    static public function _insertDetails($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqLearningOutcomeDetails (generation, concentration, name, text) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model['generation'])
                        . $db->quoteInto('?, ' , $model['concentration'])
                        . $db->quoteInto('?, ' , $model['name'])
                        . $db->quoteInto('?)'  , $model['text']);
        }
        $sql .= implode(', ', $sqlParts);

        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model['learningOutcome']->_data['details'][$model['concentration']][$model['name']]['id'] = $lastId;
            $lastId++;
        }
    }

    static public function _updateDetails($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqLearningOutcomeDetailsUpdate '
             . 'SELECT * FROM creqLearningOutcomeDetails LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqLearningOutcomeDetailsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model['learningOutcomeDetailId'])
                        . $db->quoteInto('?, ' , $model['generation'])
                        . $db->quoteInto('?, ' , $model['concentration'])
                        . $db->quoteInto('?, ' , $model['name'])
                        . $db->quoteInto('?)'  , $model['text']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqLearningOutcomeDetails AS a, '
             . '       creqLearningOutcomeDetailsUpdate AS b '
             . 'SET a.generation    = b.generation, '
             . '    a.concentration = b.concentration, '
             . '    a.name          = b.name, '
             . '    a.text          = b.text '
             . 'WHERE a.learningOutcomeDetailId = b.learningOutcomeDetailId ';

        $db->query($sql);
        $db->query('DROP TABLE creqLearningOutcomeDetailsUpdate');
    }

    static public function _deleteDetails($models)
    {
        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');

        $sql = $db->quoteInto('DELETE FROM creqLearningOutcomeCourses WHERE learningOutcomeCourseId IN (?)', array_keys($models));
        $db->query($sql);
    }

    public function __clone()
    {
        $this->_data['parent'] = $this->getId();
        $this->_data['type'] = 'proposed';

        unset($this->_data['assetId']);
        unset($this->_data['learningOutcomeGenerationId']);

        foreach ($this->_data['courses'] as &$course) {
            unset ($course['learningOutcomeCourseId']);
            unset ($course['generation']);
        }

        unset($this->_cleanData);
    }

    static public function makeCurrent($learningOutcomes)
    {
        if (!Unl_Util::isArray($learningOutcomes)) {
            if (!($learningOutcomes instanceof self)) {
                throw Exception(__CLASS__ . '::' . __METHOD__ . '(): invalid argument.');
            }
            $collection = new Unl_Model_Collection(__CLASS__);
            $collection[] = $learningOutcomes;
            $learningOutcomes = $collection;
        }

        if (count($learningOutcomes) == 0) {
            return;
        }

        $fypOfficialGenerations = array();
        foreach ($learningOutcomes as $learningOutcome) {
            $learningOutcomeId = $learningOutcome->_data['learningOutcome'];
            $generationId = $learningOutcome->getId();
            $fypOfficialGenerations[$learningOutcomeId] = $generationId;
        }

        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqLearningOutcomesUpdate '
             . 'SELECT * FROM creqLearningOutcomes LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqLearningOutcomesUpdate VALUES ';
        $sqlParts = array();
        foreach ($fypOfficialGenerations as $learningOutcomeId => $generationId) {
            $sqlParts[] = $db->quoteInto('(?, ', $learningOutcomeId)
                        . $db->quoteInto('?)'  , $generationId);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqLearningOutcomes AS a, '
             . '       creqLearningOutcomesUpdate AS b '
             . 'SET a.currentGeneration = b.currentGeneration '
             . 'WHERE a.learningOutcomeId = b.learningOutcomeId ';

        $db->query($sql);
        $db->query('DROP TABLE creqLearningOutcomesUpdate');
    }

    public function getId()
    {
        return $this->_data['learningOutcomeGenerationId'];
    }

    public function getAssetId()
    {
        return $this->_data['assetId'];
    }

    public function getParentId()
    {
        return $this->_data['parent'];
    }

    public function getLearningOutcomeId()
    {
        return $this->_data['learningOutcome'];
    }

    public function getRequest()
    {
        return $this->_data['request'];
    }

    public function setRequest(Requests_RequestModel $request)
    {
        $this->_data['request'] = $request->getId();
    }

    public function getType()
    {
        return $this->_data['type'];
    }

    public function setType($type)
    {
        $this->_data['type'] = $type;
    }

    public function getRemoved()
    {
        if ($this->_data['removed'] == 'yes') {
            return true;
        }
        return false;
    }

    public function setRemoved($removed = true)
    {
        if ($removed) {
            $this->_data['removed'] = 'yes';
        } else {
            $this->_data['removed'] = 'no';
        }
    }

    static protected $_majorCollegeMap = array();
    static protected function _getMajorCollegeMap()
    {
        if (!self::$_majorCollegeMap) {
            $db = Zend_Registry::get('db');
            $select = new Zend_Db_Select($db);

            $select->from(array('c' => 'creqColleges'), array(
                'collegeId',
                'college'     => 'name',
                'collegeLong' => 'description'
            ));
            $select->join(array('m' => 'creqMajors'), 'c.collegeId = m.college', array(
                'majorId',
                'major' => 'name'
            ));

	    $select->where("m.removed = 'no'");

            foreach ($select->query()->fetchAll() as $row) {
                self::$_majorCollegeMap[$row['majorId']] = $row;
            }
        }
        return self::$_majorCollegeMap;
    }

    public function getCollege()
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['college'];
    }

    public function getCollegeLong()
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['collegeLong'];
    }

    public function getMajor()
    {
        if (!$this->_data['majorId'] && $this->_data['major']) {
            return $this->_data['major'];
        }

        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$this->_data['majorId']])) {
            return NULL;
        }

        return $majorCollegeMap[$this->_data['majorId']]['major'];
    }

    public function setMajorId($majorId)
    {
        $majorCollegeMap = self::_getMajorCollegeMap();
        if (!isset($majorCollegeMap[$majorId])) {
            throw new Exception('Major not found.');
        }

        $this->_data['majorId'] = $majorId;
    }

    public function setMajor($major)
    {
        $majorId = NULL;
        foreach (self::_getMajorCollegeMap() as $item) {
            if ($item['major'] == $major) {
                $majorId = $item['majorId'];
            }
        }

        if (!$majorId) {
            throw new Exception ('The major "' . $major . '" does not exist.');
        }

        $this->_data['majorId'] = $majorId;
    }

    public function getConcentrations()
    {
        $concentrations = array();
        foreach ($this->_data['details'] as $concentration => $details) {
            $hyphenated = $concentration;
            $hyphenated = strtr($hyphenated, array(' ' => '-'));
            $hyphenated = strtolower($hyphenated);
            $hyphenated = preg_replace('/[^a-z0-9\-]/', '', $hyphenated);
            $hyphenated = preg_replace('/-+/', '-', $hyphenated);
            $concentrations[$hyphenated] = $concentration;
        }
        if (!$concentrations) {
            $concentrations = array('default' => 'Default');
        }

        return $concentrations;
    }

    public function getDetail($concentration, $name)
    {
        return $this->_data['details'][$concentration][$name]['text'];
    }

    public function setDetail($concentration, $name, $text)
    {
        $this->_data['details'][$concentration][$name]['text'] = $text;
    }

    public function clearDetails()
    {
        $this->_data['details'] = array();
    }
}

