<?php

class Requests_ApprovalChainManagerController extends Creq_Controller_Action
{
    public function preDispatch()
    {
        $user = Auth_UserModel::findCurrentUser();
        $roles = Auth_GroupModel::findByUser($user);
        if (!in_array(1, $roles->getId())) {
            throw new Exception('You must be logged in to view this page.');
        }
    }
    
    public function indexAction()
    {
        $in = $this->getRequest()->getParams();
        $this->_loadViewWithChainData($in['id']);
    }

    public function getActionDetailXmlAction()
    {
        $in = $this->getRequest()->getParams();
        $id = $in['id'];

        $action = Requests_ApprovalActionModel::find($id);
        if (!$action) {
            exit;
        }

        $dom = new DOMDocument();
        $actionNode = $dom->createElement('action');
        $dom->appendChild($actionNode);
        $actionNode->setAttribute('id', $action->getId());
        $actionNode->setAttribute('name', $action->getName());
        foreach ($action->getResultStatusStrings() as $key => $value) {
            $result = $dom->createElement('result');
            $result->setAttribute('value', $key);
            $result->setAttribute('name', $value);
            $actionNode->appendChild($result);
        }

        header('Content-type: text/xml');
        echo $dom->saveXML();

        exit;
    }

    public function editChainPostAction()
    {
        $in = $this->getAllParams();

        $chain = Requests_ApprovalChainModel::find($in['id']);
        if (!$chain) {
            $chain = Requests_ApprovalChainModel::fetchNew();
        }

        $chain->setName($in['name']);
        $chain->setOwnerBodyId($in['owner']);
        $chain->setModule($in['chain-module']);
        Requests_ApprovalChainModel::save($chain);

        $this->redirect('/requests/approval-chain-manager/index/id/' . $chain->getId());
    }

    public function addLinkPostAction()
    {
        $in = $this->getRequest()->getParams();

        $link = Requests_ApprovalLinkModel::fetchNew();
        $link->setChainId($in['approvalChain']);
        if ($in['currentAction'] != '_null') {
            $link->setCurrentActionId($in['currentAction']);
        }
        if ($in['currentState'] != '_null') {
            $link->setCurrentState($in['currentState']);
        }
        if ($in['nextAction'] != '_null') {
            $link->setNextActionId($in['nextAction']);
        }

        Requests_ApprovalLinkModel::save($link);
        $this->_redirect('/requests/approval-chain-manager/index/id/' . $in['approvalChain']);
    }

    public function editLinksPostAction()
    {
        $in = $this->getRequest()->getParams();

        $linkIds = array_keys($in['currentAction']);
        $links = Requests_ApprovalLinkModel::find($linkIds);
        $linksToDelete = new Unl_Model_Collection('Requests_ApprovalLinkModel');
        $linksToSave = new Unl_Model_Collection('Requests_ApprovalLinkModel');
        foreach ($links as $link) {
            if ($in['currentAction'][$link->getId()] != '_null') {
                $link->setCurrentActionId($in['currentAction'][$link->getId()]);
            } else {
            	$link->setCurrentActionId(NULL);
            }
            
            if ($in['currentState'][$link->getId()] != '_null') {
                $link->setCurrentState($in['currentState'][$link->getId()]);
            } else {
                $link->setCurrentState(NULL);
            }
            
            if ($in['nextAction'][$link->getId()] != '_null') {
                $link->setNextActionId($in['nextAction'][$link->getId()]);
            } else {
                $link->setNextActionId(NULL);
            }
            
            if ($in['delete'][$link->getId()]) {
                $linksToDelete[] = $link;
            } else {
                $linksToSave[] = $link;
            }
        }
        Requests_ApprovalLinkModel::save($linksToSave);
        Requests_ApprovalLinkModel::delete($linksToDelete);

        $this->_redirect('/requests/approval-chain-manager/index/id/' . $in['approvalChain']);
    }

    public function addActionPostAction()
    {
        $in = $this->_getAllParams();
        
        $approvalChain = Requests_ApprovalChainModel::find($in['id']);
        $actionClass = $in['type'];
        
        if (!class_exists($actionClass)) {
            exit;
        }
        
        if ($actionClass instanceof Requests_ApprovalActionModel) {
            exit;
        }
        
        $new = call_user_func(array($actionClass, 'fetchNew'));
        $new->setName($in['name']);
        $new->setApprovalChain($approvalChain);
        $new->setParticipatingRoleIds($in['participatingRoles']);
        $new->setEditingRoleIds($in['editingRoles']);
        
        $new->setExtendedData($in);
        
        Requests_ApprovalActionModel::save($new);
        
        $this->_redirect('/requests/approval-chain-manager/index/id/' . $approvalChain->getId());
    }
    
    public function editActionsPostAction()
    {
        $in = $this->_getAllParams();
        
        $approvalChain = Requests_ApprovalChainModel::find($in['id']);
        
        $actionIds = array_keys($in['edit']);
        $approvalActions = Requests_ApprovalActionModel::find($actionIds);
        
        foreach ($in['edit'] as $actionId => $data) {
            $approvalAction = $approvalActions[$actionId];
            $approvalAction->setName($data['name']);
            $approvalAction->setParticipatingRoleIds($data['participatingRoles']);
            $approvalAction->setEditingRoleIds($data['editingRoles']);
            $approvalAction->setExtendedData($data);
        }
        
        Requests_ApprovalActionModel::save($approvalActions);
        
        $this->_redirect('/requests/approval-chain-manager/index/id/' . $approvalChain->getId());
    }
    
    public function getActionEditXhtmlAction()
    {
        $this->_loadViewWithChainData($this->_getParam('chain-id'));
        $actionClass = $this->_getParam('action-class');
        
        if (!class_exists($actionClass)) {
            exit;
        }
        
        if ($actionClass instanceof Requests_ApprovalActionModel) {
            exit;
        }
        
        $action = call_user_func(array($actionClass, 'fetchNew'));
        $xhtml = $action->getEditXhtml($this->view);
        
        echo $xhtml;
        
        exit;
    }
    
    protected function _loadViewWithChainData($chainId)
    {
        $approvalChains = Requests_ApprovalChainModel::findAll();

        $approvalChain = null;
        if ($chainId) {
            $approvalChain = Requests_ApprovalChainModel::find($chainId);
            if (!$approvalChain) {
                $approvalChain = Requests_ApprovalChainModel::fetchNew();
            }
            $user = Auth_UserModel::findCurrentUser();
            $userGroups = Auth_GroupModel::findByUser($user);
            $approvalBodies = Requests_ApprovalBodyModel::findWithAdminGroups($userGroups);
            $bodies = new Unl_Model_Collection('Requests_ApprovalBodyModel');
            foreach ($approvalBodies as $approvalBody) {
                $bodies->merge($approvalBody);
            }

            $approvalLinks = Requests_ApprovalLinkModel::findForApprovalChains($approvalChain);
            $approvalActions = Requests_ApprovalActionModel::findForApprovalChains($approvalChain);

            $approvalActionClasses = Requests_ApprovalActionModel::getApprovalActionModelClasses($approvalChain->getModule());
            
            $approvalTypes = array();
            foreach ($approvalActionClasses as $className => $fullClassName) {
                $approvalTypes[$fullClassName] = substr($className, 14);
            }
            $this->view->approvalTypes = $approvalTypes;
            
            $ownerBody = Requests_ApprovalBodyModel::find($approvalChain->getOwnerBody());

            $approvalRoles = Requests_ApprovalRoleModel::findByParentBody($ownerBody);
            $approvalRolesArray = array();
            foreach ($approvalRoles as $approvalRole) {
                $approvalRolesArray[$approvalRole->getId()] = $approvalRole->getName();
            }
            $this->view->approvalRoles = $approvalRolesArray;
        }
        
        $this->view->approvalChains = $approvalChains;
        $this->view->approvalChain = $approvalChain;
        $this->view->approvalBodies = $bodies;
        $this->view->approvalLinks = $approvalLinks;
        $this->view->approvalActions = $approvalActions;
        $this->view->modules = Requests_ApprovalActionModel::getModulesWithApprovalActions();
    }

    public function getApprovalChainAction()
    {
        $json = array();
        header('Content-type: application/json');

        $chain = Requests_ApprovalChainModel::find($this->getParam('id'));
        if (!$chain) {
            echo json_encode(array('error' => 'not found'));
            header('HTTP/1.1 404 Not Found');
            exit;
        }

        $json['name'] = $chain->getName();
        $json['actions'] = array(
            'enter-chain' => array(
                'id' => 'enter-chain',
                'name' => 'Just Entered Chain',
                'position' => $chain->getEnterPosition(),
            ),
            'exit-chain' => array(
                'id' => 'exit-chain',
                'name' => 'Exit Chain',
                'position' => $chain->getExitPosition(),
            ),
        );
        $json['links'] = array();

        foreach (Requests_ApprovalActionModel::findForApprovalChains($chain) as $action) {

            $json['actions'][$action->getId()] = array(
                'id'       => $action->getId(),
                'name'     => $action->getName(),
                'position' => $action->getPosition(),
                'options'  => $action->getResultStatusStrings(),
            );
        }

        foreach(Requests_ApprovalLinkModel::findForApprovalChains($chain) as $link) {
            $json['links'][$link->getId()] = array(
                'id'    => $link->getId(),
                'from'  => $link->getCurrentAction(),
                'to'    => $link->getNextAction(),
                'state' => $link->getCurrentState(),
            );
        }

        echo json_encode($json);
        exit;

    }

    public function saveApprovalChainAction()
    {
        $in = $this->getAllParams();

        $chain = Requests_ApprovalChainModel::find($in['id']);
        if (!$chain) {
            echo json_encode(array('error' => 'not found'));
            header('HTTP/1.1 404 Not Found');
            exit;
        }

        $chain->setEnterPosition($in['actions']['enter-chain']['x'], $in['actions']['enter-chain']['y']);
        $chain->setExitPosition($in['actions']['exit-chain']['x'], $in['actions']['exit-chain']['y']);
        $actions = Requests_ApprovalActionModel::find(array_keys($in['actions']));
        foreach ($in['actions'] as $data) {
            $action = $actions[$data['id']];
            if (!$action) {
                continue;
            }

            $action->setPosition($data['x'], $data['y']);
        }

        $links = Requests_ApprovalLinkModel::find(array_keys($in['links']));
        foreach ($in['links'] as $data) {
            $link = $links[$data['id']];
            if (!$link) {
                $link = Requests_ApprovalLinkModel::fetchNew();
                $link->setChain($chain);
                $links[$data['id']] = $link;
            }
            $link->setCurrentActionId(NULL);
            if ($data['from']) {
                $link->setCurrentActionId($data['from']);
            }
            $link->setCurrentState(NULL);
            if ($data['state']) {
                $link->setCurrentState($data['state']);
            }
            $link->setNextActionId(NULL);
            if ($data['to']) {
                $link->setNextActionId($data['to']);
            }
        }

        if ($in['removedLinks']) {
            $removedLinks = Requests_ApprovalLinkModel::find($in['removedLinks']);
            Requests_ApprovalLinkModel::delete($removedLinks);
        }

        Requests_ApprovalChainModel::save($chain);
        Requests_ApprovalActionModel::save($actions);
        Requests_ApprovalLinkModel::save($links);

        exit;
    }
}
