<?php

/**
 * IndexController
 *
 * @author
 * @version
 */

require_once 'Zend/Controller/Action.php';

class Requests_IndexController extends Creq_Controller_Action
{
	/**
	 * The default action - show the home page
	 */
	public function indexAction()
	{
        $user = Auth_UserModel::findCurrentUser();
        $this->view->user = $user;
        
        $userRoles = Requests_ApprovalRoleModel::findByUser($user);
        $this->view->roles = $userRoles;
        
        // Saved Requests
        $savedRequestsData = RegistryModel::findByNamespaceAndKey($user->getId(), 'savedRequests')->getData();
        if (!Unl_Util::isArray($savedRequestsData)) {
            $savedRequestsData = array();
		}
		
		$savedRequests = array();
		foreach ($savedRequestsData as $i => $savedRequest) {
		    $savedRequests[$savedRequest['request']->getModule()][$i] = $savedRequest;
		}
		$this->view->savedRequests = $savedRequests;
		  
		$allRequests = new Unl_Model_Collection('Requests_RequestModel');
		$allRequestRecords = array();
		$moduleRequests = array();
		
		// Utility function to populate arrays with request data.
		$loadRequestArray = function(&$requests) use (&$moduleRequests, &$allRequestRecords, &$allRequests) {
		    $requests->orderBy('getTypeDescription');
		    $destination = array();
		    foreach ($requests as $i => $request) {
		        if ($allRequestRecords[$i]) {
		            $destination[$request->getModule()][$i] = &$allRequestRecords[$i];
		            continue;
		        }
		        $record = array('request' => $request);
		        $destination[$request->getModule()][$i] = &$record;
		        $moduleRequests[$request->getModule()][$i] = &$record;
		        $allRequestRecords[$i] = &$record;
		        $allRequests[$i] = $request;
		        unset($record);
		    }
		    return $destination;
		};
		
		// Owned Requests
		$this->view->ownedRequests = $loadRequestArray(Requests_RequestModel::findByUser($user));

		// Role Requests
		$roleRequests = array();
		foreach (Requests_RequestModel::findByRole($userRoles) as $role => $roleRequestData) {
		    $roleRequests[$role]['requests'] = $loadRequestArray($roleRequestData);
		}

		// Watching Requests
		foreach (Requests_RequestModel::findByWatchingRole($userRoles) as $role => $roleWatchingData) {
		    $roleRequests[$role]['watching'] = $loadRequestArray($roleWatchingData);
		}
		
		$this->view->roleRequests = $roleRequests; 
		
		// Populate additional request data
		$requestActions = Requests_ApprovalActionModel::findByRequest($allRequests);
		$requestComments = Requests_CommentsModel::findByRequest($allRequests, true);
		$requestViewTimes = Requests_ViewTimeModel::findByUserAndRequest($user, $allRequests);
		$requestVotes = Requests_ApproverVoteModel::findUsersVotesForRequests($user, $allRequests);
		$requestHistories = Requests_ApprovalHistoryModel::findByRequest($allRequests);
		
		foreach ($allRequestRecords as $i => &$record) {
		    $record['action'] = $requestActions[$i];
		    $record['comments'] = $requestComments[$i];
		    $record['vote'] = $requestVotes[$i];
		    
		    if ($requestViewTimes[$i] instanceof Requests_ViewTimeModel) {
		        $record['viewTime'] = $requestViewTimes[$i]->getTime();
		        $record['hidden'] = $requestViewTimes[$i]->isHidden();
		    } else {
		        $record['viewTime'] = 0;
		        $record['hidden'] = false;
		    }

		    $record['lastApprovalTime'] = new Zend_Date(0);
		    $histories = $requestHistories[$i];
		    $histories->orderBy('getUnixtime', SORT_DESC);
		    foreach ($histories as $history) {
		        $record['lastApprovalTime'] = $history->getTime();
		        break;
		    }
		}
		unset($record);
		
		// Populate additional module-based data
		foreach ($moduleRequests as $module => &$requests) {
		    $model = ucfirst($module) . '_UtilityModel';
		    $model::loadAdditionalData($requests);
		}

		$this->view->xsrfToken = Unl_XsrfToken::getInstance()->create();
		// save the order of requests to the session so we can have next/previous links on other pages.
		$session = new Zend_Session_Namespace('My Requests');
		$session->requestOrder = array();
		foreach ($roleRequests as $roleId => $requestsOrWatching) {
		    $session->requestOrder[$roleId] = array();
		    $session->requestOrder[$roleId]['normal'] = array();
		    if (Unl_Util::isArray($requestsOrWatching['requests'])) {
		        foreach ($requestsOrWatching['requests'] as $module => $requestList) {
		            foreach ($requestList as $request) {
		                $requestId = $request['request']->getId();
		                $session->requestOrder[$roleId]['normal'][] = $requestId;
		            }
		        }
		    }
		    $session->requestOrder[$roleId]['watching'] = array();
		    if (Unl_Util::isArray($requestsOrWatching['watching'])) {
		        foreach ($requestsOrWatching['watching'] as $module => $requestList) {
		            foreach ($requestList as $request) {
		                $requestId = $request['request']->getId();
		                $session->requestOrder[$roleId]['watching'][] = $requestId;
		            }
		        }
		    }
		}

		// Display any messages.
		$decisionSession = new Zend_Session_Namespace('decidePostAction');
		$this->view->decisionMessage = $decisionSession->message;
		unset($decisionSession->message);
	}

	public function decidePostAction()
	{
	    $in = $this->getRequest()->getParams();
	    
	    if (!Unl_XsrfToken::getInstance()->verify($in['xsrfToken'])) {
	    	throw new Zend_Exception('Invalid XSRF Token! Please go back click the Refresh button while holding Shift.');
	    }

	    $requestDecisions = array();
	    foreach ($in['decisions'] as $requestId => $decision) {
	        if ($decision == '_null') {
	            continue;
	        }
	        $requestDecisions[$requestId] = $decision;
	    }
	    $requests = Requests_RequestModel::find(array_keys($requestDecisions));
        $requestActions = Requests_ApprovalActionModel::findByRequest($requests);

        $actions = new Unl_Model_Collection('Requests_ApprovalActionModel');
        foreach ($requestActions as $requestAction) {
            $actions[$requestAction->getId()] = $requestAction;
        }

        $actionRequests = array();
        foreach ($requests as $request) {
            $actionId = $requestActions[$request->getId()]->getId();
            if (!$actionRequests[$actionId] instanceof Unl_Model_Collection) {
                $actionRequests[$actionId] = new Unl_Model_Collection('Requests_RequestModel');
            }
            $actionRequests[$actionId][$request->getId()] = $request;
        }

        foreach ($actionRequests as $actionId => $myRequests)
        {
            $actions[$actionId]->userMadeDecisions($myRequests, $requestDecisions);
        }

        Requests_ApprovalChainModel::consider($requests);
        
        $session = new Zend_Session_Namespace('decidePostAction');
        if (count($requests) == 1) {
            $session->message = 'Successfully submitted decision for 1 request.';
        } else {
            $session->message = 'Successfully submitted decisions for ' . count($requests) . ' requests.';
        }

        if (array_key_exists('returnUrl', $in)) {
            $this->_redirect($in['returnUrl']);
        } else {
            $this->_redirect('/requests');
        }
	}

	public function hideRequestPostAction()
	{
		$requestId = $this->getRequest()->getParam('id');
		
		$user = Auth_UserModel::findCurrentUser();
		$request = Requests_RequestModel::find($requestId);
		if ($request && $user) {
            $viewTime = Requests_ViewTimeModel::findByUserAndRequest($user, $request);
            if (!$viewTime) {
            	$viewTime = Requests_ViewTimeModel::fetchNew();
            	$viewTime->setUser($user);
            	$viewTime->setRequest($request);
            }
            $viewTime->setHidden();
            
            $collection = new Unl_Model_Collection('Requests_ViewTimeModel');
            $collection[] = $viewTime;
            Requests_ViewTimeModel::save($collection);
		}
		
		$this->_redirect('/requests');
	}
}
