<?php

class Requests_QueueManagerController extends Creq_Controller_Action
{
    public function preDispatch()
    {
        $user = Auth_UserModel::findCurrentUser();
        $roles = Auth_GroupModel::findByUser($user);
        if (!in_array(1, $roles->getId())) {
            throw new Exception('You must be logged in to view this page.');
        }
    }
    
    public function indexAction()
    {
    	$queues = Requests_ApprovalActionQueueModel::findAll();
    	
    	$this->view->queues = $queues;
    }

    public function viewAction()
    {
        $in = $this->_getAllParams();
        $queue = Requests_ApprovalActionModel::find($in['id']);
        $periodId = $in['period'];
        if ($periodId > 0) {
        	$periods = $queue->getPeriods();
        	$period = $periods[$periodId];
        	$periodStart = $period['start']->addSecond(1);
        	$periodEnd = $period['end'];
        	$allRequests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction(
                $periodStart, $periodEnd, $queue
            );
        } else {
            $allRequests = Requests_RequestModel::findWithCurrentApprovalAction($queue);
            $periodEnd = new Zend_Date();
        }
        $moduleRequests = array();
        foreach ($allRequests as $request) {
            $module = $request->getModule();
        	if (!$moduleRequests[$module]) {
        		$moduleRequests[$module] = new Unl_Model_Collection('Requests_RequestModel');
        	}
        	$moduleRequests[$module][$request->getId()] = $request;
        }
        
        $periods = $queue->getPeriods();
        $periodArray = array();
        foreach ($periods as $periodId => $period) {
        	if ($period['end']) {
                $periodArray[$periodId] = $period['end']->toString('YYYY-MM-dd');
        	} else {
                $periodArray[-1] = 'Current';
        	}
        }
        
        $requestVotes = Requests_ApproverVoteModel::findVotesForRequests($allRequests, $periodEnd, $queue);
    
		$requestUserVotes = array();
		$userIds = array();
		foreach ($requestVotes as $requestId => $votes) {
		    foreach ($votes as $voteId => $vote) {
		        $userIds[$vote->getUser()] = $vote->getUser();
		        $requestUserVotes[$requestId][$vote->getUser()] = $vote->getVote();
		    }
		}
        
		$users = Auth_UserModel::find($userIds);
		$users->orderBy('getLastName');
		
		$session = new Zend_Session_Namespace(__CLASS__);
        $this->view->message = $session->message;
        unset($session->message);
		
        $this->view->queue = $queue;
        $this->view->currentPeriod = $in['period'];
        $this->view->periods = $periodArray;
        $this->view->moduleRequests = $moduleRequests;
        $this->view->requestUserVotes = $requestUserVotes;
        $this->view->users = $users;
    }
    
    public function editPostAction()
    {
        $in = $this->_getAllParams();
    }

    public function advancePostAction()
    {
        $in = $this->_getAllParams();
        $requestIdsToAdvance = array();
        foreach ($in['requests'] as $requestId => $advance) {
            if (!$advance) {
                continue;
            }
            $requestIdsToAdvance[] = $requestId;
        }
        
        $requestsToAdvance = Requests_RequestModel::find($requestIdsToAdvance);
        $queue = Requests_ApprovalActionModel::find($in['queueId']);
        
        if ($queue && in_array($in['state'], $queue->getResultStatusStrings())) {
            foreach ($requestsToAdvance as $request) {
                $request->setState($in['state']);
            }
            Requests_RequestModel::save($requestsToAdvance);
        }
        
        Requests_ApprovalChainModel::advance($requestsToAdvance);
        Requests_ApprovalChainModel::consider($requestsToAdvance);
        
        $session = new Zend_Session_Namespace(__CLASS__);
        if (count($requestsToAdvance) == 1) {
            $session->message = 'Successfully advanced 1 request.';
        } else if (count($requestsToAdvance) > 1) {
            $session->message = 'Successfully advanced ' . count($requestsToAdvance) . ' requests.';
        }
        
        $this->_redirect('requests/queue-manager/view/id/' . $in['id']);
    }
}
