<?php

class Requests_ViewController extends Creq_Controller_Action
{

	public function indexAction()
	{
		$id = $this->getRequest()->getParam('id');
		$roleId = $this->getRequest()->getParam('role');
		$list = $this->getRequest()->getParam('list');
		
		$session = new Zend_Session_Namespace('My Requests');
		$request = Requests_RequestModel::find($id);
		if (!$request) {
			throw new Zend_Exception('The specified request does not exist.');
		}
        $user = Auth_UserModel::findCurrentUser();
        
        $viewTime = Requests_ViewTimeModel::findByUserAndRequest($user, $request);
        if (!$viewTime) {
        	$viewTime = Requests_ViewTimeModel::fetchNew();
        	$viewTime->setRequest($request);
        	$viewTime->setUser($user);
        }
        $viewTime->setTime(new Zend_Date());
        $collection = new Unl_Model_Collection('Requests_ViewTimeModel');
        $collection[] = $viewTime;
        Requests_ViewTimeModel::save($viewTime);
	
        if ($roleId) {
        	$session->lastRoleId = $roleId;
        }
        if ($list) {
        	$session->lastList = $list;
        }
		
		$module = $request->getModule();
		$redirectUrl = Zend_Controller_Action::getFrontController()->getBaseUrl()
		             . '/' . $module . '/view/index/id/' . $id;
        
		$this->getResponse()->setRedirect($redirectUrl);
	}

    protected function _commonIndexAction()
    {
        $in = $this->getAllParams();
        $requestId = $in['id'];
        $myRequestSession = new Zend_Session_Namespace('My Requests');
        if ($in['type'] == 'session') {
            $session = new Zend_Session_Namespace('Edit Request ' . $in['id']);
            $request = $session->request;
            if ($request->getId()) {
                $initialRequest = false;
            } else {
                $initialRequest = true;
            }
            $preview = true;
        } else {
            $request = Requests_RequestModel::find($requestId);
            $initialRequest = false;
            $preview = false;

            // if we can know our role, we can create next and prev links.
            $roleId = $myRequestSession->lastRoleId;
            $list = $myRequestSession->lastList;
            if ($roleId) {
                $requestOrder = $myRequestSession->requestOrder[$roleId][$list];
                $listPosition = array_search($request->getId(), $requestOrder);
                if ($listPosition == 0) {
                    $prevRequestId = $requestOrder[count($requestOrder) - 1];
                } else {
                    $prevRequestId = $requestOrder[$listPosition - 1];
                }
                if ($listPosition == count($requestOrder) - 1) {
                    $nextRequestId = $requestOrder[0];
                } else {
                    $nextRequestId = $requestOrder[$listPosition + 1];
                }
            }
        }

        $comments = Requests_CommentsModel::findByRequest($request);
        $user = Auth_UserModel::findCurrentUser();

        $action = null;
        $canEdit = false;
        if ($request->getId()) {
            $possibleAction = Requests_ApprovalActionModel::findByRequest($request);
            if ($possibleAction) {
                if ($possibleAction instanceof Requests_ApprovalActionSubmitterApprovalModel && $request->getOwner() == $user->getId()) {
                    $action = $possibleAction;
                }

                $userRoles = Requests_ApprovalRoleModel::findByUser($user);

                $allowedRoles = array_intersect(array_merge(array(1), $possibleAction->getParticipatingRoleIds()), $userRoles->getId());
                if (count($allowedRoles) > 0) {
                    $action = $possibleAction;
                    foreach ($allowedRoles as $allowedRole) {
                        $actionRoleNames[] = $userRoles[$allowedRole]->getName();
                    }
                }
                $editAllowedRoles = array_intersect(array_merge(array(1), $possibleAction->getEditingRoleIds()), $userRoles->getId());
                if (count($editAllowedRoles) > 0) {
                    $canEdit = true;
                }
            }


            if ($action) {
                $selectedDecision = '_null';
                $defaultDecision = '--Decision--';

                $votes = Requests_ApproverVoteModel::findUsersVotesForRequests($user, $request);
                $vote = $votes[$request->getId()];

                if ($vote) {
                    if ($vote->getApprovalAction() == $action->getId() && $request->getLastApprovalTime()->isEarlier($vote->getTime())) {
                        $selectedDecision = $vote->getVote();
                    } else {
                        $defaultDecision = 'PREVOTE: ' . $vote->getVote();
                    }
                }
                $this->view->selectedDecision = $selectedDecision;
                $this->view->defaultDecision = $defaultDecision;
            }
        }

        if ($request->getId()) {
            $allHistories = Requests_ApprovalHistoryModel::findByRequest($request);
            $allHistories->orderBy('getId');
            $mostRecentHistoryDate = new Zend_Date(0);

            $first = TRUE;
            $histories = new Unl_Model_Collection('Requests_ApprovalHistoryModel');
            foreach ($allHistories as $history) {
                if ($history->getTime()->getTimestamp() - $mostRecentHistoryDate->getTimestamp() < 60) {
                    continue;
                }
                $mostRecentHistoryDate = $history->getTime();
                if (!$first) {
                    $histories[$history->getId()] = $history;
                }
                $first = FALSE;
            }
        } else {
            $histories = array();
        }

        $this->view->request = $request;
        $this->view->comments = $comments;
        $this->view->user = $user;
        $this->view->initialRequest = $initialRequest;
        $this->view->preview = $preview;
        $this->view->prevRequestId = $prevRequestId;
        $this->view->nextRequestId = $nextRequestId;
        $this->view->canEdit = $canEdit;
        $this->view->lastList = $list;
        $this->view->lastRoleId = $roleId;
        $this->view->action = $action;
        $this->view->actionRoleNames = $actionRoleNames;
        $this->view->id = $in['id'];
        $this->view->histories = $histories;

        $decisionSession = new Zend_Session_Namespace('decidePostAction');
        $this->view->decisionMessage = $decisionSession->message;
        unset ($decisionSession->message);

        $session = new Zend_Session_Namespace(__CLASS__);
        if ($session->tabName) {
            $this->view->selectedCommentTab = $session->tabName;
        } else {
            $this->view->selectedCommentTab = 'commentsTab';
        }
    }
	
	public function getFileAction()
	{
		$in = $this->_getAllParams();
		$fileId = $this->getRequest()->getParam('fileId');
		if ($in['session']) {
			$session = new Zend_Session_Namespace('Edit Request ' . $in['session']);
			$request = $session->request;
			$file = $request->getFileById($fileId);
			$data = $file['data'];
			$title = $file['title'];
			$mimeType = $file['mimeType'];
		} else {
            $file = Requests_FileModel::find($fileId);
            $data = $file->getData();
            $title = $file->getTitle();
            $mimeType = $file->getMimeType();
		}
		$this->getResponse()->setBody($data);
		$this->getResponse()->setHeader('Content-Type:', $mimeType);
		$this->getResponse()->setHeader('Content-Disposition:', 'attachment; filename="' . $title . '"');
		
		$this->_helper->layout->disableLayout();
		$this->getFrontController()->setParam('noViewRenderer', true);
	}
	
	public function addCommentAction()
	{
		$requestId = $this->getRequest()->getParam('requestId');
		$request = Requests_RequestModel::find($requestId);
		$comments = Requests_CommentsModel::findByRequest($request);
		//$this->getFrontController()->setParam('noViewRenderer', true);
		
		$params = $this->getRequest()->getParams();
		
		$comment = Requests_CommentsModel::fetchNew();
		$comment->setTime(new Zend_Date());
		$comment->setVisibility($params['visibility']);
		$comment->setUser(Auth_UserModel::findCurrentUser());
		$comment->setRequest($request);
		$comment->setText($params['comment']);

		$comments[] = $comment;
		Requests_CommentsModel::save($comments);
		
		$redirectUrl = Zend_Controller_Action::getFrontController()->getBaseUrl()
		             . '/requests/view/index/id/' . $requestId;
		$this->getResponse()->setRedirect($redirectUrl);
	}

    public function setCommentTabAction()
    {
        $tabName = $this->getRequest()->getParam('tabName');
        $session = new Zend_Session_Namespace(__CLASS__);
        $session->tabName = $tabName;
        exit;
    }
}
