<?php

class Requests_ApprovalActionAutoModel extends Requests_ApprovalActionModel
{
	static public function find($id)
	{
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsAuto'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
	}
	
	static public function fetchNew()
	{
	    $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
	        'participatingRoles'  => array(),
	        'editingRoles'        => array(),
            'result'              => ''
	    );
	    
	    $new = new self($data);
	    $new->_setClean();
	    
	    $new->_data['className'] = 'ApprovalActionAuto';
	    
	    return $new;
	}

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsAutoUpdate '
             . 'SELECT * FROM creqApprovalActionsAuto LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsAutoUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?) ',  $model->_data['result']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsAuto AS a, '
             . '       creqApprovalActionsAutoUpdate AS b '
             . 'SET a.result = b.result '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsAuto (approvalActionId, result) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , $model->_data['result']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
	
    public function consider($requests)
    {
        foreach ($requests as $request) {
            $request->setState($this->getAutomaticResult());
        }
        Requests_RequestModel::save($requests);
        Requests_ApprovalChainModel::advance($requests);
        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

    public function getAutomaticResult()
    {
        return $this->_data['result'];
    }

    public function setExtendedData($in)
    {
        $this->setResult($in['result']);
    }
    
    public function getResult()
    {
        return $this->_data['result'];
    }
    
    public function setResult($result)
    {
        $this->_data['result'] = $result;
    }
}
