<?php

class Requests_ApprovalActionDelegateModel extends Requests_ApprovalActionModel
{
    protected $_targetChainMap;

    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsDelegate'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }

    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'delegateBody'        => ''
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionDelegate';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsDelegateUpdate '
             . 'SELECT * FROM creqApprovalActionsDelegate LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsDelegateUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?) ',  $model->_data['delegateBody']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsDelegate AS a, '
             . '       creqApprovalActionsDelegateUpdate AS b '
             . 'SET a.delegateBody = b.delegateBody '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsDelegate (approvalActionId, delegateBody) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , $model->_data['delegateBody']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        $stacks = Requests_StackModel::findForRequests($requests);
        $newStacks = new Unl_Model_Collection('Requests_StackModel');

        foreach ($requests as $request) {
            $currentStack = $stacks[$request->getId()];
            $newStack = Requests_StackModel::fetchNew();
            $newStack->setChainById($this->_getTargetChainForType($request->getTypeId()));
            $newStack->setPrevious($currentStack);
            $newStacks[$request->getId()] = $newStack;
        }
        Requests_StackModel::save($newStacks);

        foreach ($requests as $request)
        {
            $request->setStackPointer($newStacks[$request->getId()]);
        }
        Requests_RequestModel::save($requests);

        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

    public function setExtendedData($in)
    {
        $this->_data['delegateBody'] = ($in['delegateBody']);
    }
    
    protected function _getTargetChainForType($type)
    {
        $targetChainMap = $this->_getTargetChainMap();
        return $targetChainMap[$type];
    }

    protected function _getTargetChainMap()
    {
        if ($this->_targetChainMap) {
            return $this->_targetChainMap;
        }

        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqDefaultApprovalChains'), array('requestType', 'approvalChain'));
        $select->where('approvalBody = ?', $this->_data['delegateBody']);
        $records = $select->query()->fetchAll();

        $this->_targetChainMap = array();
        foreach ($records as $row) {
            $this->_targetChainMap[$row['requestType']] = $row['approvalChain'];
        }
        return $this->_targetChainMap;
    }

    public function getId()
    {
        return $this->_data['approvalActionId'];
    }
    
    public function getDelegateBody()
    {
        return $this->_data['delegateBody'];
    }
}
