<?php

class Requests_ApprovalActionEmailModel extends Requests_ApprovalActionModel
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsEmail'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            if (!is_array($object->_data['recipientGroups'])) {
                $object->_data['recipientGroups'] = explode(',', $object->_data['recipientGroups']);
            }
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'recipient'           => '',
            'recipientGroups'     => array(),
            'subject'             => '',
            'body'                => ''
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionEmail';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsEmailUpdate '
             . 'SELECT * FROM creqApprovalActionsEmail LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsEmailUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?, ',  $model->_data['recipient'])
                        . $db->quoteInto('?, ',  implode(',', $model->_data['recipientGroups']))
                        . $db->quoteInto('?, ',  $model->_data['subject'])
                        . $db->quoteInto('?) ',  $model->_data['body']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsEmail AS a, '
             . '       creqApprovalActionsEmailUpdate AS b '
             . 'SET a.recipient       = b.recipient, '
             . '    a.recipientGroups = b.recipientGroups, '
             . '    a.subject         = b.subject, '
             . '    a.body            = b.body '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
        $db->query('DROP TABLE creqApprovalActionsEmailUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsEmail (approvalActionId, recipient, recipientGroups, subject, body) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?, ',  $model->_data['recipient'])
                        . $db->quoteInto('?, ',  implode(',', $model->_data['recipientGroups']))
                        . $db->quoteInto('?, ',  $model->_data['subject'])
                        . $db->quoteInto('?)'  , $model->_data['body']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
    
    public function consider($requests)
    {
        foreach ($requests as $request) {
            $mail = new Zend_Mail('UTF-8');
            $recipients = $this->_substitute($this->getRecipient(), $request);
            if ($recipients) {
                $recipients = explode(',', $recipients);
                foreach ($recipients as $recipient) {
                    $recipient = trim($recipient);
                    if (!$recipient || $recipient == 'nobody@example.com') {
                        continue;
                    }
                    $mail->addTo($recipient);
                }
            } else {
                $recipients = array();
            }

            $recipientGroups = Auth_GroupModel::find($this->getRecipientGroups());
            $recipientGroupUsers = Auth_UserModel::findByParentGroup($recipientGroups);
            foreach ($recipientGroupUsers as $recipientUsers) {
                foreach ($recipientUsers as $user) {
                    $mail->addBcc($user->getEmail());
                }
            }
            
            // If this email wouldn't be sent to anyone, skip it.
            if (count($mail->getRecipients()) == 0) {
                continue;
            }
            
            $mail->setFrom('automated@courseapproval.unl.edu', 'Course Approval');
            $mail->setSubject($this->_substitute($this->getSubject(), $request));
            $mail->setBodyText($this->_substitute($this->getBody(), $request));
            $mail->send();
        }
        
        Requests_ApprovalChainModel::advance($requests);
        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

    public function setExtendedData($in)
    {
        $this->setRecipient($in['recipient']);
        $this->setRecipientGroups($in['recipientGroups']);
        $this->setSubject($in['subject']);
        $this->setBody($in['body']);
    }
    
    public function getRecipient()
    {
        return $this->_data['recipient'];
    }
    
    public function setRecipient($recipient)
    {
        $this->_data['recipient'] = $recipient;
    }
    
    public function getRecipientGroups()
    {
        return $this->_data['recipientGroups'];
    }
    
    public function setRecipientGroups($groups)
    {
        $this->_data['recipientGroups'] = $groups;
    }
    
    public function getSubject()
    {
        return $this->_data['subject'];
    }
    
    public function setSubject($subject)
    {
        $this->_data['subject'] = $subject;
    }
    
    public function getBody()
    {
        return $this->_data['body'];
    }
    
    public function setBody($body)
    {
        $this->_data['body'] = $body;
    }
    
    protected function _substitute($text, $request)
    {
        return $text;
    }
}
