<?php

class Requests_ApprovalActionModel extends Unl_Model
{
	static public function getApprovalActionModelClasses($onlyForModule = NULL)
	{
		$cache = Zend_Registry::get('cache');
        if ($cache->test(__CLASS__ . __FUNCTION__)) {
            $approvalActionModels = $cache->load(__CLASS__ . __FUNCTION__);
        } else {
            $front = Zend_Controller_Front::getInstance();
            $controllerDirs = $front->getControllerDirectory();
            $approvalActionModels = array();
            foreach ($controllerDirs as $moduleName => $controllerDir) {
                $modelDir = $controllerDir
                          . DIRECTORY_SEPARATOR . '..'
                          . DIRECTORY_SEPARATOR . 'models';
                if (!is_dir($modelDir)) {
                    continue;
                }
                $modelFiles = new DirectoryIterator($modelDir);
                foreach ($modelFiles as $modelFile) {
                    if (substr($modelFile->getFilename(), 0, 14) != 'ApprovalAction') {
                        continue;
                    }
                    if (substr($modelFile->getFilename(), -9) != 'Model.php') {
                        continue;
                    }
                    $tableName = substr($modelFile->getFilename(), 0, -9);
                    $className = ucfirst($moduleName) . '_' . substr($modelFile->getFilename(), 0, -4);
                    if (class_exists($className) && is_subclass_of($className, 'Requests_ApprovalActionModel')) {
                        $classes = get_declared_classes();
                        $className = $classes[array_search(strtolower($className), array_map('strtolower', $classes))];
                        $approvalActionModels[$tableName] = $className;
                    }
                }
            }
            $cache->save($approvalActionModels);
        }

        if ($onlyForModule) {
            foreach ($approvalActionModels as $key => $name) {
                $moduleName = strtolower(substr($name, 0, strpos($name, '_')));
                if (!in_array($moduleName, array('requests', $onlyForModule))) {
                    unset($approvalActionModels[$key]);
                }
            }
        }

		return $approvalActionModels;
	}

    static public function getModulesWithApprovalActions()
    {
        $modules = array('requests' => 'Generic');
        foreach (self::getApprovalActionModelClasses() as $approvalActionClass) {
            $moduleName = substr($approvalActionClass, 0, strpos($approvalActionClass, '_'));
            if ($moduleName == 'Requests') {
                continue;
            }
            $modules[strtolower($moduleName)] = $moduleName;
        }
        return $modules;
    }

	static public function find($id)
	{
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('c' => 'creqApprovalChains'), 'a.approvalChain = c.approvalChainId', array());
        $select->join(array('b' => 'creqApprovalBodies'), 'c.ownerBody = b.approvalBodyId', array('currentApprovalBody' => 'name'));
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $actionTypes = array();
        foreach ($records as $record) {
        	$actionType = $record['className'];
        	$actionId = $record['approvalActionId'];
        	$actionTypes[$actionType][] = $actionId;
        }

        $approvalActionClasses = self::getApprovalActionModelClasses();

        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($actionTypes as $actionType => $actionIds)
        {
        	$approvalActionClass = $approvalActionClasses[$actionType];
        	$objects->merge(call_user_func(array($approvalActionClass, 'find'), $actionIds));
        }

        foreach ($records as $record) {
        	$recordId = $record['approvalActionId'];
        	$object = $objects[$recordId];
        	$object->_data = array_merge($object->_data, $record);
        }


        $select = new Zend_Db_Select($db);
        $select->from(array('o' => 'creqApprovalActionsOptions'));
        if (Unl_Util::isArray($id)) {
            $select->where('o.approvalActionId IN (?)', $id);
        } else {
            $select->where('o.approvalActionId = ?', $id);
        }
        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $objects[$record['approvalActionId']]->_data['options'][$record['approvalActionsOptionId']] = $record['name'];
        }
        
        $select = new Zend_Db_Select($db);
        $select->from(array('p' => 'creqApprovalActionParticipants'));
	    if (Unl_Util::isArray($id)) {
            $select->where('p.approvalAction IN (?)', $id);
        } else {
            $select->where('p.approvalAction = ?', $id);
        }
        $records = $select->query()->fetchAll();
        foreach ($objects as $object) {
            $object->_data['participatingRoles'] = array();
            $object->_data['editingRoles'] = array();
        }
        foreach ($records as $record) {
            $objects[$record['approvalAction']]->_data['participatingRoles'][] = $record['approvalBodyRole'];
            if ($record['canEdit'] == 'yes') {
                $objects[$record['approvalAction']]->_data['editingRoles'][] = $record['approvalBodyRole'];
            }
        }

        foreach ($objects as $object) {
            $object->_setClean();
        }
        
        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects->pop();
        }
	}

	static public function findByRequest($request)
	{
		$db = Zend_Registry::get('db');

		if (Unl_Util::isArray($request)) {
			if (count($request) == 0) {
				return new Unl_Model_Collection(__CLASS__);
			}
			$requestId = array();
			foreach ($request as $aRequest) {
				$requestId[] = $aRequest->getId();
			}
		} else {
            $requestId = $request->getId();
		}

		$select = new Zend_Db_Select($db);
		$select->from(array('r' => 'creqRequests', array('requestId')));
		$select->join(array('s' => 'creqRequestStacks'), 'r.stackPointer = s.requestStackId', array('approvalActionId' => 'action'));
		if (Unl_Util::isArray($requestId)) {
			$select->where('r.requestId IN (?)', $requestId);
		} else {
			$select->where('r.requestId = ?', $requestId);
		}

		$records = $select->query()->fetchAll();

		$approvalActionIds = array();
		$requestActions = array();
		foreach($records as $record) {
            $approvalActionIds[] = $record['approvalActionId'];
            $requestActions[$record['requestId']] = $record['approvalActionId'];
		}

		$unsortedObjects = self::find($approvalActionIds);

		$objects = array();
		foreach ($requestActions as $requestId => $approvalActionId) {
            $objects[$requestId] = $unsortedObjects[$approvalActionId];
		}

		if (Unl_Util::isArray($request)) {
            return $objects;
		} else {
			return array_pop($objects);
		}
	}

	static public function findNextForRequest($request)
	{
		$db = Zend_Registry::get('db');

		if (Unl_Util::isArray($request)) {
			$requestId = array();
			foreach ($request as $aRequest) {
				$requestId[] = $aRequest->getId();
			}
		} else {
            $requestId = $request->getId();
		}

		/**
		 * TODO: Be sure that the sorting on this is correct so that
		 * more specific links are used in favor of less specific links.
		 */

		$select = new Zend_Db_Select($db);
		$select->from(array('r' => 'creqRequests'), array('requestId'));
		$select->join(array('s' => 'creqRequestStacks'), 'r.stackPointer = s.requestStackId', array());
		$select->join(array('l' => 'creqApprovalLinks'), 's.chain = l.chain', array('nextAction'));
		$select->where('s.action = l.currentAction OR (ISNULL(s.action) AND ISNULL(l.currentAction))');
		//$select->where('r.state = l.currentState OR (ISNULL(r.state) AND ISNULL(l.currentState))');
		//$select->where('s.action = l.currentAction OR (ISNULL(l.currentAction))');
		$select->where('r.state = l.currentState OR (ISNULL(l.currentState))');
		if (Unl_Util::isArray($requestId)) {
			$select->where('r.requestId IN (?)', $requestId);
		} else {
			$select->where('r.requestId = ?', $requestId);
		}
		$select->order('currentState');
		//$records = $select->query()->fetchAll();
		
		if (Unl_Util::isArray($requestId)) {
            $requestIdSql = $db->quoteInto('  r.requestId IN (?) ', $requestId);
		} else {
			$requestIdSql = $db->quoteInto('  r.requestId = ? ', $requestId);
		}
		$sql = 'SELECT r.requestId, l.nextAction, currentState '
		     . 'FROM creqRequests AS r '
		     . 'INNER JOIN creqRequestStacks AS s '
		     . '  ON r.stackPointer = s.requestStackId '
		     . 'INNER JOIN creqApprovalLinks AS l '
		     . '  ON s.chain = l.chain '
		     . 'WHERE ( '
		     . '  s.action = l.currentAction ' 
		     . ') AND ( '
		     . '  r.state = l.currentState '
		     . '  OR ( '
		     . '    l.currentState IS NULL '
		     . '  ) '
		     . ') AND ( '
		     . $requestIdSql
		     . ') '
		
		     . 'UNION '
		
             . 'SELECT r.requestId, l.nextAction, currentState '
		     . 'FROM creqRequests AS r '
		     . 'INNER JOIN creqRequestStacks AS s '
		     . '  ON r.stackPointer = s.requestStackId '
		     . 'INNER JOIN creqApprovalLinks AS l '
		     . '  ON s.chain = l.chain '
		     . 'WHERE ( '
		     . '  s.action IS NULL '
		     . '  AND ' 
		     . '  l.currentAction IS NULL '
		     . ') AND ( '
		     . '  r.state = l.currentState '
		     . '  OR ( '
		     . '    l.currentState IS NULL '
		     . '  ) '
		     . ') AND ( '
		     . $requestIdSql
		     . ') '
		     
		     . 'ORDER BY `currentState` ASC ';

        $records = $db->query($sql)->fetchAll();
		     
		$approvalActionIds = array();
		$requestActions = array();
		foreach($records as $record) {
            $approvalActionIds[] = $record['nextAction'];
            $requestActions[$record['requestId']] = $record['nextAction'];
		}

		$unsortedObjects = self::find($approvalActionIds);

		$objects = array();
		foreach ($requestActions as $requestId => $approvalActionId) {
            $objects[$requestId] = $unsortedObjects[$approvalActionId];
		}

		if (Unl_Util::isArray($request)) {
            return $objects;
		} else {
			return array_pop($objects);
		}
	}

    static public function findForApprovalChains($chains)
    {
        if (!$chains->getId()) {
            if (Unl_Util::isArray($chains)) {
                return array();
            }
            return NULL;
        }

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'), array('approvalActionId'));
        $select->where('approvalChain IN (?)', $chains->getId());
        $data = $select->query()->fetchAll();

        $actionIds = array();
        foreach ($data as $row) {
            $actionIds[] = $row['approvalActionId'];
        }

        $unsortedObjects = self::find($actionIds);

        $chainActions = array();
        if (Unl_Util::isArray($chains)) {
            foreach ($chains as $chain) {
                $chainActions[$chain->getId()] = new Unl_Model_Collection(__CLASS__);
            }
        } else {
            $chainActions[$chains->getId()] = new Unl_Model_Collection(__CLASS__);
        }

        foreach ($actionIds as $actionId) {
            $object = $unsortedObjects[$actionId];
            $chainActions[$object->getApprovalChain()][$object->getId()] = $object;
        }

        if (Unl_Util::isArray($chains)) {
            return $chainActions;
        } else {
            return array_pop($chainActions);
        }
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
        
        
        $participatingRolesToAdd = array();
        $participatingRolesToDelete = array();
        $editingRolesToAdd = array();
        $editingRolesToDelete = array();
        foreach ($models as $model) {
            $participatingRolesToAdd[$model->getId()] = array_diff($model->_data['participatingRoles'], $model->_cleanData['participatingRoles']);
            $participatingRolesToDelete[$model->getId()] = array_diff($model->_cleanData['participatingRoles'], $model->_data['participatingRoles']);
            $editingRolesToAdd[$model->getId()] = array_diff($model->_data['editingRoles'], $model->_cleanData['editingRoles']);
            $editingRolesToDelete[$model->getId()] = array_diff($model->_cleanData['editingRoles'], $model->_data['editingRoles']);
        }
        self::_addParticipatingRoles($participatingRolesToAdd);
        self::_deleteParticipatingRoles($participatingRolesToDelete);
        self::_addEditingRoles($editingRolesToAdd);
        self::_deleteEditingRoles($editingRolesToDelete);


        $modelsByClass = array();
        foreach ($models as $model) {
            $modelClass = get_class($model);
            $modelsByClass[$modelClass][] = $model;
        }
        
        foreach ($modelsByClass as $modelClass => $modelsToSave) {
            call_user_func(array($modelClass, 'save'), $modelsToSave);
        }
        
        
        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }
    
    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsUpdate '
             . 'SELECT * FROM creqApprovalActions LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['approvalChain'])
                        . $db->quoteInto('?, ' , $model->_data['className'])
                        . $db->quoteInto('?, ' , intval($model->_data['xPosition']))
                        . $db->quoteInto('?)'  , intval($model->_data['yPosition']));
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActions AS a, '
             . '       creqApprovalActionsUpdate AS b '
             . 'SET a.name          = b.name, '
             . '    a.approvalChain = b.approvalChain, '
             . '    a.className     = b.className, '
             . '    a.xPosition     = b.xPosition, '
             . '    a.yPosition     = b.yPosition '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
        
        $db->query('DROP TABLE creqApprovalActionsUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActions (name, approvalChain, className, xPosition, yPosition) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['approvalChain'])
                        . $db->quoteInto('?, ' , $model->_data['className'])
                        . $db->quoteInto('?, ' , intval($model->_data['xPosition']))
                        . $db->quoteInto('?)'  , intval($model->_data['yPosition']));
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
        
        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approvalActionId'] = $lastId;
            $lastId++;
        }
    }
    
    static protected function _addParticipatingRoles($data)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionParticipants (approvalAction, approvalBodyRole, canEdit) VALUES ';
        $sqlParts = array();
        foreach ($data as $actionId => $roleIds) {
            foreach ($roleIds as $roleId) {
                $sqlParts[] = $db->quoteInto('(?, ', $actionId)
                            . $db->quoteInto('?, ' , $roleId)
                            . $db->quoteInto('?)'  , 'no');
            }
        }
        if (count($sqlParts) == 0) {
            return;
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
    
    static protected function _deleteParticipatingRoles($data)
    {
        $db = Zend_Registry::get('db');

        $sql = 'DELETE FROM creqApprovalActionParticipants '
             . 'WHERE (approvalAction, approvalBodyRole) IN (';
        $sqlParts = array('(-1, -1)');
        foreach ($data as $actionId => $roleIds) {
            foreach ($roleIds as $roleId) {
                $sqlParts[] = $db->quoteInto('(?, ', $actionId)
                            . $db->quoteInto('?)'  , $roleId);
            }
        }
        $sql .= implode(', ', $sqlParts) . ')';
        $db->query($sql);
    }

    static protected function _addEditingRoles($data)
    {
        $db = Zend_Registry::get('db');

        $sql = 'UPDATE creqApprovalActionParticipants '
             . "SET canEdit = 'yes' "
             . 'WHERE (approvalAction, approvalBodyRole) IN (';
        $sqlParts = array('(-1, -1)');
        foreach ($data as $actionId => $roleIds) {
            foreach ($roleIds as $roleId) {
                $sqlParts[] = $db->quoteInto('(?, ', $actionId)
                            . $db->quoteInto('?)'  , $roleId);
            }
        }
        $sql .= implode(', ', $sqlParts) . ')';
        $db->query($sql);
    }
    
    static protected function _deleteEditingRoles($data)
    {
        $db = Zend_Registry::get('db');

        $sql = 'UPDATE creqApprovalActionParticipants '
             . "SET canEdit = 'no'"
             . 'WHERE (approvalAction, approvalBodyRole) IN (';
        $sqlParts = array('(-1, -1)');
        foreach ($data as $actionId => $roleIds) {
            foreach ($roleIds as $roleId) {
                $sqlParts[] = $db->quoteInto('(?, ', $actionId)
                            . $db->quoteInto('?)'  , $roleId);
            }
        }
        $sql .= implode(', ', $sqlParts) . ')';
        $db->query($sql);
    }
    
    static public function getDefaultStatusStrings()
    {
        return array('Approve' => 'Approve', 'Resubmit' => 'Resubmit');
    }
    
    public function getId()
    {
    	return $this->_data['approvalActionId'];
    }

    public function getName()
    {
    	return $this->_data['name'];
    }

    public function setName($name)
    {
        $this->_data['name'] = $name;
    }
    
    public function getResultStatusStrings()
    {
        if (Unl_Util::isArray($this->_data['options'])) {
            $optinos = array();
            foreach ($this->_data['options'] as $option)
            {
                $options[$option] = $option;
            }
        } else {
            $options = self::getDefaultStatusStrings();
        }
        return $options;
    }

    public function getCurrentApprovalBody()
    {
    	return $this->_data['currentApprovalBody'];
    }

    public function getApprovalChain()
    {
        return $this->_data['approvalChain'];
    }
    
    public function setApprovalChain(Requests_ApprovalChainModel $approvalChain)
    {
        $this->_data['approvalChain'] = $approvalChain->getId();
    }
    
    public function getParticipatingRoleIds()
    {
        return $this->_data['participatingRoles'];
    }
    
    public function setParticipatingRoleIds($roleIds)
    {
        if (!$roleIds) {
            $roleIds = array();
        }
        $this->_data['participatingRoles'] = $roleIds;
    }
    
    public function getEditingRoleIds()
    {
        return $this->_data['editingRoles'];
    }
    
    public function setEditingRoleIds($roleIds)
    {
        if (!$roleIds) {
            $roleIds = array();
        }
        $this->_data['editingRoles'] = $roleIds;
    }
    
    public function getEditXhtml(Zend_View $view)
    {
        $className = get_class($this);
        $className = explode('_', $className);
        $moduleName = strtolower(array_shift($className));
        $actionName = substr(implode('_', $className), 14, -5);
        $actionName[0] = strtolower($actionName[0]);
        
        $helpers = $view->getHelpers();
        $view->setBasePath(APPLICATION_PATH . '/modules/' . $moduleName . '/views');
        $view->addHelperPath(APPLICATION_PATH . '/../library/Unl/View/Helper', 'Unl_View_Helper');
        $view->approvalAction = $this;
        return $view->render('approval-chain-manager/edit-action/' . $actionName . '.phtml');
    }

    public function getPosition()
    {
        return array(
            'x' => $this->_data['xPosition'],
            'y' => $this->_data['yPosition'],
        );
    }

    public function setPosition($x, $y)
    {
        $this->_data['xPosition'] = $x;
        $this->_data['yPosition'] = $y;
    }
}
