<?php

class Requests_ApprovalActionQueueModel extends Requests_ApprovalActionModel
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('q' => 'creqApprovalActionsQueue'), 'a.approvalActionId = q.approvalActionId');
        $select->join(array('p' => 'creqApprovalActionsQueuePeriods'),
                      'a.approvalActionId = p.approvalAction AND p.endTime IS NULL',
                      array('startTime', 'currentPeriod' => 'approvalActionsQueuePeriodId'));
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $record['dates'] = array();
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }
        
        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqApprovalActionsQueueDates'));
        if (Unl_Util::isArray($id)) {
            $select->where('d.approvalAction IN(?)', $id);
        } else {
            $select->where('d.approvalAction = ?', $id);
        }
        $records = $db->query($select)->fetchAll();
        
        foreach ($records as $record) {
            $objectId = $record['approvalAction'];
            $date = $record['date'];
            $objects[$objectId]->_data['dates'][] = new Zend_Date($date); 
        }
        
        
        $select = new Zend_Db_Select($db);
        $select->from(array('p' => 'creqApprovalActionsQueuePeriods'));
        $select->order('startTime DESC');
        if (Unl_Util::isArray($id)) {
            $select->where('p.approvalAction IN(?)', $id);
        } else {
            $select->where('p.approvalAction = ?', $id);
        }
        $records = $db->query($select)->fetchAll();
        
        foreach ($records as $record) {
            $objectId = $record['approvalAction'];
            $periodId = $record['approvalActionsQueuePeriodId'];
            $objects[$objectId]->_data['periods'][$periodId] = array('start' => new Zend_Date($record['startTime']),
                                                                     'end'   => ($record['endTime'] ? new Zend_Date($record['endTime']) : NULL),
                                                                     'id'    => $record['approvalActionsQueuePeriodId']); 
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function findAll()
    {
    	$db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from('creqApprovalActionsQueue', array('approvalActionId'));
        
        $data = $select->query()->fetchAll();
        
        $approvalActionIds = array(-1);
        foreach ($data as $row) {
        	$approvalActionIds[] = $row['approvalActionId'];
        }
        
        return self::find($approvalActionIds);
    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'canChangeVote'       => NULL,
            'dates'               => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionQueue';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
        
        
        $datesToInsert = array();
        $datesToDelete = array();
        foreach ($models as $model) {
            $datesToInsert[$model->getId()] = array_diff($model->_data['dates'], $model->_cleanData['dates']);
            $datesToDelete[$model->getId()] = array_diff($model->_cleanData['dates'], $model->_data['dates']);
        }
        
        self::_insertDates($datesToInsert);
        self::_deleteDates($datesToDelete);
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsQueueUpdate '
             . 'SELECT * FROM creqApprovalActionsQueue LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsQueueUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , $model->_data['canChangeVote']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsQueue AS a, '
             . '       creqApprovalActionsQueueUpdate AS b '
             . 'SET a.canChangeVote = b.canChangeVote '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
        
        $db->query('DROP TABLE creqApprovalActionsQueueUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');
        
        // Insert the queue row
        $sql = 'INSERT INTO creqApprovalActionsQueue (approvalActionId, canChangeVote) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , $model->_data['canChangeVote']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
        
        
        
        // Insert the initial queue period starting at the current time.
        $sql = 'INSERT INTO creqApprovalActionsQueuePeriods (approvalAction, startTime) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }
    
    static protected function _insertDates(array $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsQueueDates (approvalAction, date) VALUES ';
        $sqlParts = array();
        foreach ($models as $approvalActionId => $dates) {
            foreach ($dates as $date) {
                $sqlParts[] = $db->quoteInto('(?, ', $approvalActionId)
                            . $db->quoteInto('?)'  , $date->getTimestamp());
            }
        }
        if (count($sqlParts) == 0) {
            return;
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
        
        $jobs = new Unl_Model_Collection('Cron_JobsModel');
        foreach ($models as $approvalActionId => $dates) {
            foreach ($dates as $date) {
                $job = Cron_JobsModel::fetchNew();
                $job->setDate($date);
                $job->setClass(__CLASS__);
                $job->setMethod('advanceQueues');
                $jobs[] = $job;
            }
        }
        Cron_JobsModel::save($jobs);
    }
     
    static protected function _deleteDates(array $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'DELETE FROM creqApprovalActionsQueueDates WHERE (approvalAction, date) IN (';
        $sqlParts = array('(-1, -1)');
        foreach ($models as $approvalActionId => $dates) {
            foreach ($dates as $date) {
                $sqlParts[] = $db->quoteInto('(?, ', $approvalActionId)
                            . $db->quoteInto('?)'  , $date->getTimestamp());
            }
        }
        $sql .= implode(', ', $sqlParts) . ')';
        $db->query($sql);
    }
    
    static public function advanceQueues()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('q' => 'creqApprovalActionsQueueDates'), array('approvalAction'));
        $select->join(array('s' => 'creqRequestStacks'), 'q.approvalAction = s.action', array());
        $select->join(array('r' => 'creqRequests'), 's.requestStackId = r.stackPointer', array('requestId'));
        $select->where('q.date <= ?', time());

        $records = $select->query()->fetchAll();
        $requestIds = array();
        $requestActionIds = array();
        foreach ($records as $record) {
            $requestId = $record['requestId'];
            $approvalActionId =  $record['approvalAction'];
            $requestIds[$requestId] = $requestId;
            $requestActionIds[$requestId] = $approvalActionId;
        }
        $requests = Requests_RequestModel::find($requestIds);
        $approvalActions = Requests_ApprovalActionModel::find($requestActionIds);

        $actionRequests = array();
        foreach ($requestActionIds as $requestId => $actionId) {
            if (!$actionRequests[$actionId] instanceof Unl_Model_Collection) {
                $actionRequests[$actionId] = new Unl_Model_Collection('Requests_RequestModel');
            }
            $actionRequests[$actionId][$requestId] = $requests[$requestId];
        }

        foreach ($actionRequests as $actionId => $myRequests) {
            $approvalActions[$actionId]->finalize($myRequests);
        }
        Requests_ApprovalChainModel::advance($requests);
        Requests_ApprovalChainModel::consider($requests);

        
        
        $select = new Zend_Db_Select($db);
        $select->from(array('q' => 'creqApprovalActionsQueueDates'), array('approvalAction'));
        $select->where('q.date <= ?', time());
        
        $queueIds = array();
        $records = $select->query()->fetchAll();
        if (count($records) == 0) {
        	return;
        }
        foreach ($records as $record) {
        	$queueIds[] = $record['approvalAction'];
        }
        
        
        $sql = 'UPDATE creqApprovalActionsQueuePeriods '
             . $db->quoteInto('SET endTime = ? ', time())
             . 'WHERE endTime IS NULL '
             . $db->quoteInto('AND approvalAction IN (?) ', $queueIds);
        $db->query($sql);
        
        $sql = 'INSERT INTO creqApprovalActionsQueuePeriods (approvalAction, startTime) VALUES ';
        $sqlParts = array();
        foreach ($queueIds as $queueId) {
        	$sqlParts[] = $db->quoteInto('(?, ', $queueId)
        	            . $db->quoteInto('?)'  , time());
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
        
        
        $db->query($db->quoteInto('DELETE FROM creqApprovalActionsQueueDates WHERE date <= ?', time()));
    }

    public function consider($requests)
    {
        return new Unl_Model_Collection('Requests_RequestModel');
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        $user = Auth_UserModel::findCurrentUser();
        $votes = Requests_ApproverVoteModel::findUsersVotesForRequests($user, $requests, $this->getCurrentPeriodId());
        foreach ($requests as $request) {
            if ($votes[$request->getId()]) {
            	if ($this->getCanChangeVote()) {
                    $vote = $votes[$request->getId()];
            	} else {
                    continue;
                }
            } else {
                $vote = Requests_ApproverVoteModel::fetchNew();
            }
            $vote->setVote($requestDecisions[$request->getId()]);
            $vote->setTime(new Zend_Date());
            $vote->setUser($user);
            $vote->setApprovalAction($this);
            $vote->setRequest($request);
            $votes[$request->getId()] = $vote;
        }
        Requests_ApproverVoteModel::save($votes);
    }

    public function finalize($requests)
    {
        // Do nothing in base class, useful for overriding in child classes
    }

    public function getCanChangeVote()
    {
        return (bool) ($this->_data['canChangeVote'] == 'yes');
    }
    
    public function getDates()
    {
        return $this->_data['dates'];
    }
    
    public function setDates($dates)
    {
        foreach ($dates as $date) {
            if (!($date instanceof Zend_Date)) {
                throw new Zend_Exception('Each date must be a Zend_Date object.');
            }
        }
        $this->_data['dates'] = $dates;
    }
    
    public function getPeriods()
    {
    	return $this->_data['periods'];
    }
    
    public function setExtendedData($data)
    {
        $this->_data['canChangeVote'] = $data['canChangeVote'];
        
        $dates = array();
        foreach ($data['schedule'] as $dateParts) {
            if ($dateParts['delete']) {
                continue;
            }
            $date = new Zend_Date();
            $date->setDay(1);

            $date->setYear($dateParts['year']);
            $date->setMonth($dateParts['month']);
            $date->setDay($dateParts['day']);
            $date->setHour($dateParts['hour']);
            $date->setMinute($dateParts['minute']);
            $date->setSecond(0);
            
            $dates[] = $date;
        }
        
        $this->setDates($dates);
    }
    
    public function getCurrentPeriodStartTime()
    {
    	return new Zend_Date($this->_data['startTime']);
    }
    
    public function getCurrentPeriodId()
    {
    	return $this->_data['currentPeriod'];
    }
}

