<?php

class Requests_ApprovalActionSendToChainModel extends Requests_ApprovalActionModel
{
    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsSendToChain'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }
        
        
        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }

    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'targetChain'         => NULL
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionSendToChain';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsSendToChainUpdate '
             . 'SELECT * FROM creqApprovalActionsSendToChain LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsSendToChainUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?) ',  $model->_data['targetChain']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsSendToChain AS a, '
             . '       creqApprovalActionsSendToChainUpdate AS b '
             . 'SET a.targetChain = b.targetChain '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsSendToChain (approvalActionId, targetChain) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?)'  , $model->_data['targetChain']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        $stacks = Requests_StackModel::findForRequests($requests);
        $newStacks = new Unl_Model_Collection('Requests_StackModel');
        foreach ($requests as $request) {
            $currentStack = $stacks[$request->getId()];
            $newStack = Requests_StackModel::fetchNew();
            $newStack->setChainById($this->getTargetChain());
            $newStack->setPrevious($currentStack);
            $newStacks[$request->getId()] = $newStack;
        }
        Requests_StackModel::save($newStacks);

        foreach ($requests as $request)
        {
            $request->setStackPointer($newStacks[$request->getId()]);
        }
        Requests_RequestModel::save($requests);

        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        // Don't do anything here, this is an automated action.
    }

    public function setExtendedData($in)
    {
        $this->_data['targetChain'] = $in['targetChain'];
    }

    public function getTargetChain()
    {
        return $this->_data['targetChain'];
    }
    
    static public function getPossibleChains(Requests_ApprovalChainModel $chain)
    {
        $db = Zend_Registry::get('db');
        
        $select = new Zend_Db_Select($db);
        $select->from(array('c1' => 'creqApprovalChains'), array());
        $select->join(array('c2' => 'creqApprovalChains'), 'c1.ownerBody = c2.ownerBody', array('approvalChainId', 'name'));
        $select->where('c1.approvalChainId != c2.approvalChainId');
        $select->where('c1.approvalChainId = ?', $chain->getId());
        
        $records = $db->query($select)->fetchAll();
        $possibleChains = array();
        foreach ($records as $record) {
            $possibleChains[$record['approvalChainId']] = $record['name']; 
        }
        
        return $possibleChains;
    }
}
