<?php

class Requests_ApprovalActionSubmitterApprovalModel extends Requests_ApprovalActionModel
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('q' => 'creqApprovalActionsSubmitterApproval'), 'a.approvalActionId = q.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionSubmitterApproval';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        // Nothing to update
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsSubmitterApproval (approvalActionId) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?) ', $model->_data['approvalActionId']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        foreach ($requests as $request) {
            $request->setSubmitterAttentionRequired();
        }
        Requests_RequestModel::save($requests);

        return new Unl_Model_Collection('Requests_RequestModel');
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        foreach ($requests as $request) {
            $request->setSubmitterAttentionRequired(false);
            $request->setState($requestDecisions[$request->getId()]);
        }
        Requests_RequestModel::save($requests);
        Requests_ApprovalChainModel::advance($requests);
    }

    public function setExtendedData($in)
    {
        // No extended data
    }
    
    public function getResultStatusStrings()
    {
        if (Unl_Util::isArray($this->_data['options'])) {
            $optinos = array();
            foreach ($this->_data['options'] as $option)
            {
                $options[$option] = $option;
            }
        } else {
            $options = array('Resubmit' => 'Resubmit', 'Withdraw' => 'Withdraw');
        }
        return $options;
    }
}

