<?php

class Requests_ApprovalActionVoteModel extends Requests_ApprovalActionQueueModel
{
    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('q' => 'creqApprovalActionsQueue'), 'a.approvalActionId = q.approvalActionId');
        $select->join(array('v' => 'creqApprovalActionsVote'), 'a.approvalActionId = v.approvalActionId');
        $select->join(array('p' => 'creqApprovalActionsQueuePeriods'),
                      'a.approvalActionId = p.approvalAction AND p.endTime IS NULL',
                      array('startTime', 'currentPeriod' => 'approvalActionsQueuePeriodId'));
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $record['dates'] = array();
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }
        

        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqApprovalActionsQueueDates'));
        if (Unl_Util::isArray($id)) {
            $select->where('d.approvalAction IN(?)', $id);
        } else {
            $select->where('d.approvalAction = ?', $id);
        }
        $records = $db->query($select)->fetchAll();
        
        foreach ($records as $record) {
            $objectId = $record['approvalAction'];
            $date = $record['date'];
            $objects[$objectId]->_data['dates'][] = new Zend_Date($date);
        }
        
        $select = new Zend_Db_Select($db);
        $select->from(array('p' => 'creqApprovalActionsQueuePeriods'));
        $select->order('startTime DESC');
        if (Unl_Util::isArray($id)) {
            $select->where('p.approvalAction IN(?)', $id);
        } else {
            $select->where('p.approvalAction = ?', $id);
        }
        $records = $db->query($select)->fetchAll();
        
        foreach ($records as $record) {
            $objectId = $record['approvalAction'];
            $periodId = $record['approvalActionsQueuePeriodId'];
            $objects[$objectId]->_data['periods'][$periodId] = array('start' => new Zend_Date($record['startTime']),
                                                                     'end'   => ($record['endTime'] ? new Zend_Date($record['endTime']) : NULL),
                                                                     'id'    => $record['approvalActionsQueuePeriodId']); 
        }
        
        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }

    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'canChangeVote'       => NULL,
            'quorum'              => NULL,
            'requiredVotes'       => NULL,
            'dates'               => array()
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionVote';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
        
        parent::save($models);
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsVoteUpdate '
             . 'SELECT * FROM creqApprovalActionsVote LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsVoteUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?,'  , $model->_data['quorum'])
                        . $db->quoteInto('?,'  , $model->_data['requiredVotes'])
                        . $db->quoteInto('?)'  , $model->_data['votesToTable']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsVote AS a, '
             . '       creqApprovalActionsVoteUpdate AS b '
             . 'SET a.quorum        = b.quorum, '
             . '    a.requiredVotes = b.requiredVotes '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
        
        $db->query('DROP TABLE creqApprovalActionsVoteUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsVote (approvalActionId, quorum, requiredVotes, votesToTable) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?,'  , $model->_data['quorum'])
                        . $db->quoteInto('?,'  , $model->_data['requiredVotes'])
                        . $db->quoteInto('?)'  , $model->_data['votesToTable']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        return new Unl_Model_Collection('Requests_RequestModel');
    }
    
    public function setExtendedData($data)
    {
        $this->_data['quorum'] = $data['quorum'];
        $this->_data['requiredVotes'] = $data['requiredVotes'];
        
        parent::setExtendedData($data);
    }

    public function finalize($requests)
    {	
    	$requestVotes = Requests_ApproverVoteModel::findVotesForRequests($requests, new Zend_Date(), $this);
        foreach ($requests as $request) {
            $votes = $requestVotes[$request->getId()];
            $result = $this->tallyVotes($votes);
            $request->setState($result);
        }

        Requests_RequestModel::save($requests);
    }

    public function getQuorum()
    {
        return $this->_data['quorum'];
    }
    
    public function getVotesToTable()
    {
        return $this->_data['votesToTable'];
    }
    
    public function getRequiredVotes()
    {
        return $this->_data['requiredVotes'];
    }
    
    public function tallyVotes($votes)
    {
        if (!Unl_Util::isArray($votes) || count($votes) == 0) {
        	return NULL;
        }

        $voteCount = count($votes);
        if ($voteCount < $this->getQuorum()) {
            return NULL;
        }
        
        $voteTally = array();
        foreach ($votes as $vote) {
            $voteTally[$vote->getVote()]++;
        }
        if ($voteTally['Table'] >= $this->getVotesToTable()) {
            return 'Table';
        }

        // TODO: Handle Ties
        $mostVotes = 0;
        $winner = '';
        foreach ($voteTally as $name => $count) {
            if ($count > $mostVotes) {
                $winner = $name;
                $mostVotes = $count;
            }
        }

        if ($this->_data['requiredVotes'] == 'plurality') {
            return $winner;
        } else if ($this->_data['requiredVotes'] == 'majority' && ($mostVotes / $voteCount) > 0.5) {
            return $winner;
        } else if ($this->_data['requiredVotes'] == 'unanimity' && ($mostVotes / $voteCount) == 1.0) {
            return $winner;
        } else {
            return NULL;
        }
    }
}
