<?php

class Requests_ApprovalActionWatchersModel extends Requests_ApprovalActionModel
{
    public static function find($id)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApprovalActions'));
        $select->join(array('d' => 'creqApprovalActionsWatchers'), 'a.approvalActionId = d.approvalActionId');
        if (Unl_Util::isArray($id)) {
            $select->where('a.approvalActionId IN(?)', $id);
        } else {
            $select->where('a.approvalActionId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function fetchNew()
    {
        $data = array(
            'approvalActionId'    => NULL,
            'name'                => '',
            'approvalChain'       => NULL,
            'className'           => '',
            'participatingRoles'  => array(),
            'editingRoles'        => array(),
            'watchingRole'        => NULL,
            'action'              => NULL
        );
        
        $new = new self($data);
        $new->_setClean();
        
        $new->_data['className'] = 'ApprovalActionWatchers';
        
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->_cleanData['approvalActionId'] && $model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalActionsWatchersUpdate '
             . 'SELECT * FROM creqApprovalActionsWatchers LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalActionsWatchersUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?, ',  $model->_data['watchingRole'])
                        . $db->quoteInto('?) ',  $model->_data['action']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalActionsWatchers AS a, '
             . '       creqApprovalActionsWatchersUpdate AS b '
             . 'SET a.watchingRole = b.watchingRole, '
             . '    a.action       = b.action '
             . 'WHERE a.approvalActionId = b.approvalActionId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalActionsWatchers (approvalActionId, watchingRole, action) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalActionId'])
                        . $db->quoteInto('?, ',  $model->_data['watchingRole'])
                        . $db->quoteInto('?)'  , $model->_data['action']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    public function consider($requests)
    {
        if (count($requests) == 0) {
            return $requests;
        }

        $db = Zend_Registry::get('db');
        if ($this->_data['action'] == 'add') {
	        $sql = 'INSERT INTO creqRequestWatchers (request, watchingApprovalRole) VALUES ';
	        $sqlParts = array();
	        foreach ($requests as $request) {
	            $sqlParts[] = $db->quoteInto('(?, ', $request->getId())
	                        . $db->quoteInto('?)',   $this->_data['watchingRole']);
	        }
	        $sql .= implode(', ', $sqlParts);
        } else if ($this->_data['action'] == 'remove') {
        	$sql = 'DELETE FROM creqRequestWatchers WHERE (request, watchingApprovalRole) IN (';
        	$sqlParts = array('(-1, -1)');
        	foreach ($requests as $request) {
                $sqlParts[] = $db->quoteInto('(?, ', $request->getId())
                            . $db->quoteInto('?)',   $this->_data['watchingRole']);
        	}
        	$sql .= implode(', ', $sqlParts) . ')';
        }
        try {
            $db->query($sql);
        } catch (Exception $e) {
            // TODO: this was probably just a duplicate record error, but maybe not
        }

        Requests_ApprovalChainModel::advance($requests);

        return $requests;
    }

    public function userMadeDecisions($requests, $requestDecisions)
    {
        //
    }

    public function setExtendedData($in)
    {
        $this->_data['watchingRole'] = $in['watchingRole'];
        $this->_data['action'] = $in['action'];
    }
    
    public function getWatchingRole()
    {
        return $this->_data['watchingRole'];
    }
    
    public function getAction()
    {
    	return $this->_data['action'];
    }
}