<?php

class Requests_ApprovalBodyModel extends Unl_Model
{
    
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        } else if ($id === NULL) {
            return null;
        }

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'creqApprovalBodies'));
        if (Unl_Util::isArray($id)) {
            $select->where('c.approvalBodyId IN(?)', $id);
        } else {
            $select->where('c.approvalBodyId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects->pop();
        }
    }
    
    static public function findAll()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'creqApprovalBodies'), array('approvalBodyId'));

        $records = $db->query($select)->fetchAll();
        $approvalBodyIds = array();
        foreach ($records as $record) {
            $approvalBodyIds[] = $record['approvalBodyId'];
        }

        return self::find($approvalBodyIds);
    }

    static public function findForApprovalRoles($approvalRoles)
    {
        if (Unl_Util::isArray($approvalRoles)) {
            $inputIsArray = true;
            foreach ($approvalRoles as $approvalRole) {
                $id[$approvalRole->getId()] = $approvalRole->getApprovalBody();
            }
        } else {
            $inputIsArray = false;
            $id[$approvalRoles] = $approvalRoles->getApprovalBody();
        }

        self::find($id);

        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($id as $roleId => $bodyId) {
            $objects[$roleId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $bodyId);
        }

        if ($inputIsArray) {
            return $objects;
        } else {
            return $objects->pop();
        }
    }

    static public function findWithAdminGroups($groups)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('b' => 'creqApprovalBodies'), array('approvalBodyId'));
        $select->where('adminGroup IN (?)', $groups->getId());
        $data = $select->query()->fetchAll();

        $bodyIds = array();
        foreach ($data as $row) {
            $bodyIds[] = $row['approvalBodyId'];
        }

        self::find($bodyIds);

        $groupBodies = array();
        if (Unl_Util::isArray($groups)) {
            foreach ($groups as $group) {
                $groupBodies[$group->getId()] = new Unl_Model_Collection(__CLASS__);
            }
        } else {
            $groupBodies[$groups->getId()] = new Unl_Model_Collection(__CLASS__);
        }

        foreach ($bodyIds as $bodyId) {
            $object = Unl_Model_Registry::getInstance()->get(__CLASS__, $bodyId);
            $groupBodies[$object->getAdminGroup()][$object->getId()] = $object;
        }

        if (Unl_Util::isArray($groups)) {
            return $groupBodies;
        } else {
            return array_pop($groupBodies);
        }
    }

    /**
     * Gets a freshly created model object
     *
     * @return Requests_ApprovalBodyModel
     */
    static function fetchNew()
    {
        $data = array(
            'approvalBodyId' => null,
            'name'           => null,
            'description'    => null,
            'adminGroup'     => null
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalBodiesUpdate '
             . 'SELECT * FROM creqApprovalBodies LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalBodiesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalBodyId'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['description'])
                        . $db->quoteInto('?)'  , $model->_data['adminGroup']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalBodies AS a, '
             . '       creqApprovalBodiesUpdate AS b '
             . 'SET a.name        = b.name, '
             . '    a.description = b.description, '
             . '    a.adminGroup  = b.adminGroup '
             . 'WHERE a.approvalBodyId = b.approvalBodyId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalBodies (name, description, adminGroup) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['description'])
                        . $db->quoteInto('?)'  , $model->_data['adminGroup']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    
        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approvalBodyId'] = $lastId;
            $lastId++;
        }
    }

    public function getId()
    {
        return $this->_data['approvalBodyId'];
    }

    public function getName()
    {
        return $this->_data['name'];
    }

    public function setName($name)
    {
        $this->_data['name'] = $name;
    }

    public function getDescription()
    {
        return $this->_data['description'];
    }

    public function setDescription($description)
    {
        $this->_data['description'] = $description;
    }

    public function getAdminGroup()
    {
        return $this->_data['adminGroup'];
    }

    public function setAdminGroup(Auth_GroupModel $group)
    {
        $this->_data['adminGroup'] = $group->getId();
    }

}
