<?php

class Requests_ApprovalChainModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
		$select = new Zend_Db_Select($db);

		$select->from(array('s' => 'creqApprovalChains'));

		if (Unl_Util::isArray($id)) {
			if (count($id) == 0) {
				return new Unl_Model_Collection(__CLASS__);
			}
			$select->where('approvalChainId IN(?)', $id);
		} else {
			$select->where('approvalChainId = ?', $id);
		}
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($id)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$id];
        }
    }

    static public function findAll()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqApprovalChains'), array('approvalChainId'));
        $data = $select->query()->fetchAll();

        $chainIds = array();
        foreach ($data as $row) {
            $chainIds[] = $row['approvalChainId'];
        }
        return self::find($chainIds);
    }

    static public function findDefaultChainsForRequests($requests)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqDefaultApprovalChains'), array('approvalChain'));
        $select->join(array('r' => 'creqRequests'), 'd.requestType = r.type', array('requestId'));
        $select->where('d.approvalBody = 1');
        $select->where('requestId IN (?)', $requests->getId());
        $data = $select->query()->fetchAll();

        $chainIds = array();
        foreach ($data as $row) {
            $chainIds[$row['requestId']] = $row['approvalChain'];
        }
        self::find($chainIds);

        $chains = new Unl_Model_Collection(__CLASS__);
        foreach ($chainIds as $requestId => $chainId) {
            $chains[$requestId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $chainId);
        }
        return $chains;
    }

    static public function findCurrentChainsForRequests($requests)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('s' => 'creqRequestStacks'), array('chain'));
        $select->join(array('r' => 'creqRequests'), 's.requestStackId = r.stackPointer', array('requestId'));
        $select->where('requestId IN (?)', $requests->getId());
        $data = $select->query()->fetchAll();

        $chainIds = array();
        foreach ($data as $row) {
            $chainIds[$row['requestId']] = $row['chain'];
        }
        self::find($chainIds);

        $chains = new Unl_Model_Collection(__CLASS__);
        foreach ($chainIds as $requestId => $chainId) {
            $chains[$requestId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $chainId);
        }
        return $chains;
    }
    
    static public function findForApprovalBody($approvalBody)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($approvalBody)) {
            if (count($approvalBody) == 0) {
                return array();
            }
        } else if ($approvalBody === NULL || $approvalBody->getId() === NULL) {
        	return new Unl_Model_Collection(__CLASS__);
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'creqApprovalChains'), array('ownerBody', 'approvalChainId'));
        if (Unl_Util::isArray($approvalBody)) {
            $select->where('c.ownerBody IN (?)', $approvalBody->getId());
        } else {
            $select->where('c.ownerBody = ?', $approvalBody->getId());
        }

        $records = $select->query()->fetchAll();

        $approvalChainIds = array();
        $approvalBodyChains = array();
        foreach($records as $record) {
            $approvalChainIds[] = $record['approvalChainId'];
            $approvalBodyChains[$record['ownerBody']][] = $record['approvalChainId'];
        }

        self::find($approvalChainIds);

        $objects = array();
        foreach ((array) $approvalBody->getId() as $approvalBodyId) {
            $objects[$approvalBodyId] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($approvalBodyChains as $approvalBodyId => $approvalChainIds) {
            foreach ($approvalChainIds as $approvalChainId) {
                $objects[$approvalBodyId][$approvalChainId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $approvalChainId);;
            }
        }

        if (Unl_Util::isArray($approvalBody)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }

    static public function fetchNew()
    {
        $data = array(
            'approvalChainId' => null,
            'ownerBody'       => null,
            'name'            => '',
            'description'     => '',
            'module'          => 'requests',
            'enterXPosition'  => 0,
            'enterYPosition'  => 0,
            'exitXPosition'   => 0,
            'exitYPosition'   => 0,
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
    	$modelsToUpdate = new Unl_Model_Collection(__CLASS__);

    	if (!Unl_Util::isArray($models)) {
    	    $model = $models;
    	    $models = new Unl_Model_Collection(__CLASS__);
    	    $models[$model->getId()] = $model;
    	}

    	foreach ($models as $model) {
    		if ($model->getId()) {
    			if ($model->_cleanData != $model->_data) {
    			    $modelsToUpdate[] = $model;
    			}
    		} else {
    			$modelsToInsert[] = $model;
    		}
    	}

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
    	$db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalChainsUpdate '
             . 'SELECT * FROM creqApprovalChains LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalChainsUpdate VALUES ';
        $sqlParts = array();
    	foreach ($models as $model) {
    		$sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalChainId'])
                        . $db->quoteInto('?, ' , $model->_data['ownerBody'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['description'])
                        . $db->quoteInto('?, ' , $model->_data['module'])
                        . $db->quoteInto('?, ' , $model->_data['enterXPosition'])
                        . $db->quoteInto('?, ' , $model->_data['enterYPosition'])
                        . $db->quoteInto('?, ' , $model->_data['exitXPosition'])
                        . $db->quoteInto('?)'  , $model->_data['exitYPosition']);
    	}
    	$sql .= implode(', ', $sqlParts);
    	$db->query($sql);

    	$sql = 'UPDATE creqApprovalChains AS a, '
    	     . '       creqApprovalChainsUpdate AS b '
    	     . 'SET a.ownerBody      = b.ownerBody, '
             . '    a.name           = b.name, '
             . '    a.description    = b.description, '
             . '    a.module         = b.module, '
             . '    a.enterXPosition = b.enterXPosition, '
             . '    a.enterYPosition = b.enterYPosition, '
             . '    a.exitXPosition  = b.exitXPosition, '
             . '    a.exitYPosition  = b.exitYPosition '
             . 'WHERE a.approvalChainId = b.approvalChainId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalChains (ownerBody, name, description, module, enterXPosition, enterYPosition, exitXPosition, exitYPosition) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['ownerBody'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?, ' , $model->_data['description'])
                        . $db->quoteInto('?, ' , $model->_data['module'])
                        . $db->quoteInto('?, ' , $model->_data['enterXPosition'])
                        . $db->quoteInto('?, ' , $model->_data['enterYPosition'])
                        . $db->quoteInto('?, ' , $model->_data['exitXPosition'])
                        . $db->quoteInto('?)'  , $model->_data['exitYPosition']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approvalChainId'] = $lastId;
            $lastId++;
        }
    }


    /**
     * Checks the given requests to start/advance them in their chain, if applicable.
     *
     * @param unknown_type $requests
     */
    static public function consider($requests)
    {
        if (!Unl_Util::isArray($requests)) {
            $request = $requests;
            $requests = new Unl_Model_Collection('Requests_RequestModel');
            $requestId = $request->getId();
            $requests[$requestId] = $request;
        }
        if (count($requests) == 0) {
            return;
        }
    
        // If any advanced requests are completed, drop them from our list
        $incompleteRequests = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($requests as $requestId => $request) {
            if ($request->isComplete()) {
                continue;
            }
            $incompleteRequests[$requestId] = $request;
        }
        $requests = $incompleteRequests;

        // If we don't have any requests left to process (after dropping completed requests), stop.
        if (count($requests) == 0) {
            return;
        }

        // Find requests which haven't yet started (stack pointer = null and complete = no)
        $requestsToInit = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($requests as $request)
        {
            $stackPointer = $request->getStackPointer();
            if (!$stackPointer) {
                $requestsToInit[] = $request;
            }
        }
        self::_initRequests($requestsToInit);

        $stacks = Requests_StackModel::findForRequests($requests);

        $requestsToAdvance = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($requests as $request) {
            if ($stacks[$request->getId()]->getAction()) {
                continue;
            }
            $requestsToAdvance[] = $request;
        }
        self::advance($requestsToAdvance);

        // If any advanced requests were completed, drop them from our list
        $incompleteRequests = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($requests as $requestId => $request) {
            if ($request->isComplete()) {
                continue;
            }
            $incompleteRequests[$requestId] = $request;
        }
        $requests = $incompleteRequests;

        // If we don't have any requests left to process (after dropping completed requests), stop.
        if (count($requests) == 0) {
            return;
        }


        $requestActions = Requests_ApprovalActionModel::findByRequest($requests);
        $actions = new Unl_Model_Collection('Requests_ApprovalActionModel');
        foreach ($requestActions as $requestAction) {
            $actions[$requestAction->getId()] = $requestAction;
        }

        $actionRequests = array();
        foreach ($requests as $request) {
            $actionId = $requestActions[$request->getId()]->getId();
            if (!$actionRequests[$actionId] instanceof Unl_Model_Collection) {
                $actionRequests[$actionId] = new Unl_Model_Collection('Requests_RequestModel');
            }
            $actionRequests[$actionId][$request->getId()] = $request;
        }

        $requestsToReconsider = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($actionRequests as $actionId => $myRequests)
        {
            $requestsToReconsider->merge($actions[$actionId]->consider($myRequests));
        }
        self::consider($requestsToReconsider);
    }

    static protected function _initRequests($requests)
    {
        if (count($requests) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $defaultChains = self::findDefaultChainsForRequests($requests);
        $stacks = new Unl_Model_Collection('Requests_StackModel');
        foreach ($requests as $request) {
            $stack = Requests_StackModel::fetchNew();
            $stack->setChain($defaultChains[$request->getId()]);
            $stacks[$request->getId()] = $stack;
        }
        Requests_StackModel::save($stacks);
        foreach ($requests as $request) {
            $request->setStackPointer($stacks[$request->getId()]);
        }
        Requests_RequestModel::save($requests);
    }

    static public function advance($requests)
    {
        if (!Unl_Util::isArray($requests)) {
            $request = $requests;
            $requests = new Unl_Model_Collection('Requests_RequestModel');
            $requestId = $request->getId();
            $requests[$requestId] = $request;
        }

        if (count($requests) == 0) {
            return;
        }

        $nextActions = Requests_ApprovalActionModel::findNextForRequest($requests);
        $stacks = Requests_StackModel::findForRequests($requests);
        $previousStacks = Requests_StackModel::findPreviousForStacks($stacks);
        Requests_ApprovalHistoryModel::recordHistoryForRequests($requests);

        $requestsToReadvance = new Unl_Model_Collection('Requests_RequestModel');
        foreach ($requests as $request)
        {
            $nextAction = $nextActions[$request->getId()];
            if ($nextAction) {
                $stacks[$request->getId()]->setAction($nextAction);
            } else {
                $currentStack = $stacks[$request->getId()];
                $previousStack = $previousStacks[$currentStack->getId()];
                if ($previousStack) {
                    $request->setStackPointer($previousStack);
                    $requestsToReadvance[] = $request;
                } else {
                    /**
                     * TODO: Be sure that completed requests are resolved well
                     * in the consider function.
                     */
                    $request->clearStackPointer();
                    $request->setComplete();
                }
            }
        }
        Requests_StackModel::save($stacks);
        Requests_RequestModel::save($requests);
        self::advance($requestsToReadvance);
    }

    public  function getId()
    {
        return $this->_data['approvalChainId'];
    }

    public function getName()
    {
        return $this->_data['name'];
    }

    public function setName($name)
    {
        $this->_data['name'] = $name;
    }

    public function getModule()
    {
        return $this->_data['module'];
    }

    public function setModule($module)
    {
        $this->_data['module'] = $module;
    }

    public function getOwnerBody()
    {
        return $this->_data['ownerBody'];
    }

    public function setOwnerBodyId($ownerBodyId)
    {
        $this->_data['ownerBody'] = $ownerBodyId;
    }

    public function getEnterPosition()
    {
        return array(
            'x' => $this->_data['enterXPosition'],
            'y' => $this->_data['enterYPosition'],
        );
    }

    public function setEnterPosition($x, $y)
    {
        $this->_data['enterXPosition'] = $x;
        $this->_data['enterYPosition'] = $y;
    }

    public function getExitPosition()
    {
        return array(
            'x' => $this->_data['exitXPosition'],
            'y' => $this->_data['exitYPosition'],
        );
    }

    public function setExitPosition($x, $y)
    {
        $this->_data['exitXPosition'] = $x;
        $this->_data['exitYPosition'] = $y;
    }
}