<?php

class Requests_ApprovalHistoryModel extends Unl_Model
{
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('c' => 'creqApprovalHistories'));
        if (Unl_Util::isArray($id)) {
            $select->where('c.approvalHistoryId IN(?)', $id);
        } else {
            $select->where('c.approvalHistoryId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects->pop();
        }
    }
    
    static public function findByRequest($request)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($request)) {
            if (count($request) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('h' => 'creqApprovalHistories', array('request', 'approvalHistoryId')));
        if (Unl_Util::isArray($request)) {
            $select->where('h.request IN (?)', $request->getId());
        } else {
            $select->where('h.request = ?', $request->getId());
        }

        $records = $select->query()->fetchAll();

        $historyIds = array();
        $requestHistories = array();
        foreach($records as $record) {
            $historyIds[] = $record['approvalHistoryId'];
            $requestHistories[$record['request']][] = $record['approvalHistoryId'];
        }

        self::find($historyIds);

        $objects = array();
        foreach ((array) $request->getId() as $requestId) {
            $objects[$requestId] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($requestHistories as $requestId => $historyIds) {
            foreach ($historyIds as $historyId) {
                $object = Unl_Model_Registry::getInstance()->get(__CLASS__, $historyId);
                $objects[$requestId][$historyId] = $object;
            }
        }

        if (Unl_Util::isArray($request)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }
    
    static public function findRequestsWithinTimeRangeAndApprovalAction(Zend_Date $startTime, 
    		                                                            Zend_Date $endTime,
                                                                        Requests_ApprovalActionModel $action = null,
                                                                        $decision = '')
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('h' => 'creqApprovalHistories'), array('request'));
        $select->where('h.time >= ?', $startTime->getTimestamp());
        $select->where('h.time <= ?', $endTime->getTimestamp());
        if ($action) {
            $select->where('approvalAction = ?', $action->getId());
        }
        if ($decision) {
            $select->where('decision = ?', $decision);
        }

        $records = $select->query()->fetchAll();

        $requestIds = array();
        foreach($records as $record) {
            $requestIds[] = $record['request'];
        }

        return Requests_RequestModel::find($requestIds);
    }

    /**
     * Gets a freshly created model object
     *
     * @return Requests_ApprovalHistoryModel
     */
    static function fetchNew()
    {
        $data = array(
            'approvalHistoryId'  => null,
            'request'            => null,
            'approvalBody'       => null,
            'approvalBodyName'   => null,
            'approvalAction'     => null,
            'approvalActionName' => null,
            'user'               => null,
            'userName'           => null,
            'decision'           => null,
            'time'               => null
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalHistoriesUpdate '
             . 'SELECT * FROM creqApprovalHistories LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalHistoriesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalHistoryId'])
                        . $db->quoteInto('?, ' , $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBodyName'])
                        . $db->quoteInto('?, ' , $model->_data['approvalAction'])
                        . $db->quoteInto('?, ' , $model->_data['approvalActionName'])
                        . $db->quoteInto('?, ' , $model->_data['user'])
                        . $db->quoteInto('?, ' , $model->_data['userName'])
                        . $db->quoteInto('?, ' , $model->_data['decision'])
                        . $db->quoteInto('?)'  , $model->_data['time']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalHistories AS a, '
             . '       creqApprovalHistoriesUpdate AS b '
             . 'SET a.request            = b.request, '
             . '    a.approvalBody       = b.approvalBody, '
             . '    a.approvalBodyName   = b.approvalBodyName, '
             . '    a.approvalAction     = b.approvalAction, '
             . '    a.approvalActionName = b.approvalActionName, '
             . '    a.user               = b.user, '
             . '    a.userName           = b.userName, '
             . '    a.decision           = b.decision, '
             . '    a.time               = b.time '
             . 'WHERE a.approvalHistoryId = b.approvalHistoryId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalHistories (request, approvalBody, approvalBodyName, approvalAction, approvalActionName, user, userName, decision, time) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBodyName'])
                        . $db->quoteInto('?, ' , $model->_data['approvalAction'])
                        . $db->quoteInto('?, ' , $model->_data['approvalActionName'])
                        . $db->quoteInto('?, ' , $model->_data['user'])
                        . $db->quoteInto('?, ' , $model->_data['userName'])
                        . $db->quoteInto('?, ' , $model->_data['decision'])
                        . $db->quoteInto('?)'  , $model->_data['time']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    }

    static public function recordHistoryForRequests($requests)
    {
        $userId = NULL;
        $userName = '';
        if (Zend_Auth::getInstance()->hasIdentity()) {
            $user = Auth_UserModel::findCurrentUser();
            $userId = $user->getId();
            $userName = $user->getFirstname() . ' ' . $user->getLastname();
        }
        
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('r' => 'creqRequests'), array('request' => 'requestId', 'decision' => 'state'));
        $select->join(array('s' => 'creqRequestStacks'), 'r.stackPointer = s.requestStackId', array());
        $select->join(array('a' => 'creqApprovalActions'), 's.action = a.approvalActionId', array('approvalAction' => 'approvalActionId', 'approvalActionName' => 'name'));
        $select->join(array('c' => 'creqApprovalChains'), 'a.approvalChain = c.approvalChainId', array());
        $select->join(array('b' => 'creqApprovalBodies'), 'c.ownerBody = b.approvalBodyId', array('approvalBody' => 'approvalBodyId', 'approvalBodyName' => 'name'));
        $select->where('r.requestId IN (?)', $requests->getId());

        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $record['time'] = time();
            $record['user'] = $userId;
            $record['userName'] = $userName;
            $object = new self($record);
            $objects[$record['request']] = $object;
        }
        self::save($objects);
    }

    public function getId()
    {
        return $this->_data['approvalHistoryId'];
    }

    public function getRequest()
    {
        return $this->_data['request'];
    }

    public function setRequest(Requests_RequestModel $request)
    {
        $this->_data['request'] = $request->getId();
    }

    public function getApprovalBody()
    {
        return $this->_data['approvalBody'];
    }

    public function getApprovalBodyName()
    {
        return $this->_data['approvalBodyName'];
    }

    public function setApprovalBody()
    {
        // TODO
    }

    public function getApprovalAction()
    {
        return $this->_data['approvalAction'];
    }

    public function getApprovalActionName()
    {
        return $this->_data['approvalActionName'];
    }

    public function setApprovalAction(Requests_ApprovalActionModel $action)
    {
        $this->_data['approvalAction'] = $action->getId();
        $this->_data['approvalActionName'] = $action->getName();
    }

    public function getUser()
    {
        return $this->_data['user'];
    }

    public function getUserName()
    {
        return $this->_data['userName'];
    }

    public function setUser(Auth_UserModel $user)
    {
        $this->_data['user'] = $user->getId();
        $this->_data['userName'] = $user->getFirstName . ' ' . $user->getLastName();
    }

    public function getDecision()
    {
        return $this->_data['decision'];
    }

    public function setDecision($decision)
    {
        $this->_data['decision'] = $decision;
    }

    /**
     * Returns the time the comment was posted.
     *
     * @return Zend_Date
     */
    public function getTime()
    {
        $time = new Zend_Date($this->_data['time']);
        return $time;
    }

    public function getUnixtime()
    {
        return $this->_data['time'];
    }
    
    /**
     * Sets the date the comment was posted
     *
     * @param Zend_Date $time
     */
    public function setTime(Zend_Date $time)
    {
        $this->_data['time'] = $time->getTimestamp();
    }
}
