<?php

class Requests_ApprovalLinkModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
		$select = new Zend_Db_Select($db);

		$select->from(array('l' => 'creqApprovalLinks'));

		if (Unl_Util::isArray($id)) {
			if (count($id) == 0) {
				return new Unl_Model_Collection(__CLASS__);
			}
			$select->where('approvalLinkId IN(?)', $id);
		} else {
			$select->where('approvalLinkId = ?', $id);
		}
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($id)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$id];
        }
    }

    static public function findForApprovalChains($chains)
    {
        if (!$chains->getId()) {
            if (Unl_Util::isArray($chains)) {
                return array();
            }
            return NULL;
        }

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('l' => 'creqApprovalLinks'), array('approvalLinkId'));
        $select->where('chain IN (?)', $chains->getId());
        $data = $select->query()->fetchAll();

        $linkIds = array();
        foreach ($data as $row) {
            $linkIds[] = $row['approvalLinkId'];
        }

        self::find($linkIds);

        $chainLinks = array();
        if (Unl_Util::isArray($chains)) {
            foreach ($chains as $chain) {
                $chainLinks[$chain->getId()] = new Unl_Model_Collection(__CLASS__);
            }
        } else {
            $chainLinks[$chains->getId()] = new Unl_Model_Collection(__CLASS__);
        }

        foreach ($linkIds as $linkId) {
            $object = Unl_Model_Registry::getInstance()->get(__CLASS__, $linkId);
            $chainLinks[$object->getChain()][$object->getId()] = $object;
        }

        if (Unl_Util::isArray($chains)) {
            return $chainLinks;
        } else {
            return array_pop($chainLinks);
        }
    }

    /**
     * Returns a new, empty model.
     *
     * @return Requests_ApprovalLinkModel
     */
    static public function fetchNew()
    {
        $data = array(
            'approvalLinkId' => NULL,
            'chain'          => NULL,
            'currentAction'  => NULL,
            'currentState'   => NULL,
            'nextAction'     => NULL,
            'name'           => '',
            'description'    => ''
        );

        $new = new self($data);
        $new->_setClean();

        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
    	$modelsToUpdate = new Unl_Model_Collection(__CLASS__);

    	if (!Unl_Util::isArray($models)) {
    	    $model = $models;
    	    $models = new Unl_Model_Collection(__CLASS__);
    	    $models[$model->getId()] = $model;
    	}

    	foreach ($models as $model) {
    		if ($model->getId()) {
    			if ($model->_cleanData != $model->_data) {
    			    $modelsToUpdate[] = $model;
    			}
    		} else {
    			$modelsToInsert[] = $model;
    		}
    	}

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
    	$db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalLinksUpdate '
             . 'SELECT * FROM creqApprovalLinks LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalLinksUpdate VALUES ';
        $sqlParts = array();
    	foreach ($models as $model) {
    		$sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalLinkId'])
                        . $db->quoteInto('?, ' , $model->_data['chain'])
                        . $db->quoteInto('?, ' , $model->_data['currentAction'])
                        . $db->quoteInto('?, ' , $model->_data['currentState'])
                        . $db->quoteInto('?, ' , $model->_data['nextAction'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['description']);
    	}
    	$sql .= implode(', ', $sqlParts);
    	$db->query($sql);

    	$sql = 'UPDATE creqApprovalLinks AS a, '
    	     . '       creqApprovalLinksUpdate AS b '
    	     . 'SET a.chain         = b.chain, '
             . '    a.currentAction = b.currentAction, '
             . '    a.currentState  = b.currentState, '
             . '    a.nextAction    = b.nextAction, '
             . '    a.name          = b.name, '
             . '    a.description   = b.description '
             . 'WHERE a.approvalLinkId = b.approvalLinkId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

    	$sql = 'INSERT INTO creqApprovalLinks (chain, currentAction, currentState, nextAction, name, description) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['chain'])
                        . $db->quoteInto('?, ' , $model->_data['currentAction'])
                        . $db->quoteInto('?, ' , $model->_data['currentState'])
                        . $db->quoteInto('?, ' , $model->_data['nextAction'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['description']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approvalLinkId'] = $lastId;
            $lastId++;
        }
    }

    static public function delete($models)
    {
        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        if (count($models) == 0) {
            return;
        }

        $db = Zend_Registry::get('db');
        $sql = $db->quoteInto('DELETE FROM creqApprovalLinks WHERE approvalLinkId IN (?)', $models->getId());
        $db->query($sql);
    }

    public function getId()
    {
        return $this->_data['approvalLinkId'];
    }

    public function getChain()
    {
        return $this->_data['chain'];
    }

    public function setChain(Requests_ApprovalChainModel $chain)
    {
        $this->_data['chain'] = $chain->getId();
    }

    public function setChainId($chainId)
    {
        $this->_data['chain'] = $chainId;
    }

    public function getCurrentAction()
    {
        return $this->_data['currentAction'];
    }

    public function setCurrentActionId($actionId)
    {
        $this->_data['currentAction'] = $actionId;
    }

    public function getCurrentState()
    {
        return $this->_data['currentState'];
    }

    public function setCurrentState($state)
    {
        $this->_data['currentState'] = $state;
    }


    public function getNextAction()
    {
        return $this->_data['nextAction'];
    }

    public function setNextActionId($actionId)
    {
        $this->_data['nextAction'] = $actionId;
    }
}