<?php

class Requests_ApprovalRoleModel extends Unl_Model
{
    
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        } else if ($id === NULL) {
            return null;
        }
        
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('r' => 'creqApprovalBodyRoles'));

        if (Unl_Util::isArray($id)) {
            if (count($id) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
            $select->where('approvalBodyRoleId IN(?)', $id);
        } else {
            $select->where('approvalBodyRoleId = ?', $id);
        }
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($id)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$id];
        }
    }
    
    static public function findAll()
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        $select->from(array('r' => 'creqApprovalBodyRoles'));
        $records = $select->query()->fetchAll();
        $roleIds = array();
        foreach ($records as $record) {
            $roleIds[] = $record['approvalBodyRoleId'];
        }
        
        return self::find($roleIds);
    }
    
    static public function findByUser($user)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);

        // get userId as a string or array from the user object(s)
        $objects = array();
        if (Unl_Util::isArray($user)) {
            $userId = array();
            foreach ($user as $aUser) {
                $userId[] = $aUser->getId();
                $objects[$aUser->getId()] = new Unl_Model_Collection(__CLASS__);
            }
        } else if ($user instanceof Auth_UserModel) {
            $userId = $user->getId();
            $objects[$user->getId()] = new Unl_Model_Collection(__CLASS__);
        } else {
            throw new Exception('$user is not a Unl_Model_Collection or a Auth_UserModel');
        }


        // find the group(s) that the user(s) is a "primary" member of
        $select = new Zend_Db_Select($db);
        $select->from(array('u' => 'creqUsers'), array('userId'));
        $select->join(array('r' => 'creqApprovalBodyRoles'), 'u.primaryGroup = r.group');
        if (Unl_Util::isArray($userId)) {
            $select->where('u.userId IN (?)', $userId);
        } else {
            $select->where('u.userId = ?', $userId);
        }

        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $recordUserId = $record['userId'];
            unset($record['userId']);

            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$recordUserId][$objectId] = $object;
        }

        // find any groups that the user(s) is a "non-primary" member of
        $select = new Zend_Db_Select($db);
        $select->from(array('u' => 'creqUsers'), array('userId'));
        $select->join(array('m' => 'creqGroupImpliedMemberships'), 'u.primaryGroup = m.childGroup', array());
        $select->join(array('r' => 'creqApprovalBodyRoles'), 'm.parentGroup  = r.group');
        if (Unl_Util::isArray($userId)) {
            $select->where('u.userId IN (?)', $userId);
        } else {
            $select->where('u.userId = ?', $userId);
        }

        $records = $select->query()->fetchAll();
        foreach ($records as $record) {
            $recordUserId = $record['userId'];
            unset($record['userId']);
            if (!$objects[$recordUserId]) {
                $objects[$recordUserId] = new Unl_Model_Collection('Requests_ApprovalRoleModel');
            }

            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$recordUserId][$objectId] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($userId)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$userId];
        }
    }
    
    static public function findByParentBody($parentBody)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($parentBody)) {
            if (count($parentBody) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
        }
        if (!$parentBody) {
            return NULL;
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('r' => 'creqApprovalBodyRoles', array('approvalBodyRoleId')));
        $select->join(array('b' => 'creqApprovalBodies'), 'r.approvalBody = b.approvalBodyId', array('approvalBodyId'));
        if (Unl_Util::isArray($parentBody)) {
            $select->where('b.approvalBodyId IN (?)', $parentBody->getId());
        } else {
            $select->where('b.approvalBodyId = ?', $parentBody->getId());
        }

        $records = $select->query()->fetchAll();

        $approvalRoleIds = array();
        $approvalBodyRoles = array();
        foreach($records as $record) {
            $approvalRoleIds[] = $record['approvalBodyRoleId'];
            $approvalBodyRoles[$record['approvalBodyId']][] = $record['approvalBodyRoleId'];
        }

        self::find($approvalRoleIds);

        $objects = array();
        foreach ((array) $parentBody->getId() as $parentBodyId) {
            $objects[$parentBodyId] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($approvalBodyRoles as $approvalBodyId => $approvalRoleIds) {
            foreach ($approvalRoleIds as $approvalRoleId) {
                $objects[$approvalBodyId][$approvalRoleId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $approvalRoleId);
            }
        }

        if (Unl_Util::isArray($parentBody)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }

    /**
     * Gets a freshly created model object
     *
     * @return Requests_ApprovalRoleModel
     */
    static function fetchNew()
    {
        $data = array(
            'approvalBodyRoleId' => null,
            'approvalBody'       => null,
            'group'              => null,
            'name'               => null,
            'notice'             => ''
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApprovalBodyRolesUpdate '
             . 'SELECT * FROM creqApprovalBodyRoles LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApprovalBodyRolesUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalBodyRoleId'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['group'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['notice']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqApprovalBodyRoles AS a, '
             . '       creqApprovalBodyRolesUpdate AS b '
             . 'SET a.approvalBody = b.approvalbody, '
             . '    a.`group`      = b.`group`, '
             . '    a.name         = b.name, '
             . '    a.notice       = b.notice '
             . 'WHERE a.approvalBodyRoleId = b.approvalBodyRoleId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqApprovalBodyRoles (approvalBody, `group`, name, notice) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['group'])
                        . $db->quoteInto('?, ' , $model->_data['name'])
                        . $db->quoteInto('?)'  , $model->_data['notice']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    
        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approvalBodyRoleId'] = $lastId;
            $lastId++;
        }
    }

    public function getId()
    {
        return $this->_data['approvalBodyRoleId'];
    }

    public function getName()
    {
        return $this->_data['name'];
    }
    
    public function setName($name)
    {
        $this->_data['name'] = $name;
    }

    public function getNotice()
    {
        return $this->_data['notice'];
    }
    
    public function setNotice($notice)
    {
        $this->_data['notice'] = $notice;
    }

    public function getApprovalBody()
    {
        return $this->_data['approvalBody'];
    }
    
    public function setApprovalBody(Requests_ApprovalBodyModel $approvalBody)
    {
        $this->_data['approvalBody'] = $approvalBody->getId();
    }
    
    public function getGroup()
    {
        return $this->_data['group'];
    }
    
    public function setGroup(Auth_GroupModel $group)
    {
        $this->_data['group'] = $group->getId();
    }

}
