<?php

class Requests_ApproverVoteModel extends Unl_Model
{
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }


        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApproverVotes'));
        if (Unl_Util::isArray($id)) {
            $select->where('a.approverVoteId IN(?)', $id);
        } else {
            $select->where('a.approverVoteId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        foreach ($objects as $object) {
            $object->_setClean();
        }
        
        if (Unl_Util::isArray($objects)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }

    static public function findVotesForRequests($requests, Zend_Date $timeInPeriod, Requests_ApprovalActionModel $approvalAction)
    {
        $db = Zend_Registry::get('db');
        
        if (!Unl_Util::isArray($requests)) {
            if (!$requests instanceof Requests_RequestModel) {
                throw new Zend_Exception('$requests is not a valid Request model or collection.');
            }
            $requests = array($requests);
        }
        if (count($requests) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApproverVotes'), array('approverVoteId', 'request'));
        $select->join(array('p' => 'creqApprovalActionsQueuePeriods'),
                      'a.approvalAction = p.approvalAction AND p.startTime <= a.time AND (a.time <= p.endTime OR p.endTime IS NULL)',
                      array());
        $select->where('a.request IN(?)', $requests->getId());
        $select->where('a.approvalAction = ?', $approvalAction->getId());
        $select->where('p.startTime < ?', $timeInPeriod->getTimestamp());
        $select->where('p.endTime >= ? OR p.endTime IS NULL', $timeInPeriod->getTimestamp());
        $select->order('time');

        $records = $db->query($select)->fetchAll();
        $requestVoteIds = array();
        foreach ($records as $record)
        {
            $requestVoteIds[] = $record['approverVoteId'];
        }

        self::find($requestVoteIds);

        $requestVotes = array();
        foreach ($requests as $request) { 
            $requestVotes[$request->getId()] = new Unl_Model_Collection(__CLASS__);
        }
        foreach ($records as $record) {
            $requestVotes[$record['request']][$record['approverVoteId']] = Unl_Model_Registry::getInstance()->get(__CLASS__, $record['approverVoteId']);
        }
        return $requestVotes;
    }
    
    static public function findUsersVotesForRequests(Auth_UserModel $user, $requests, $queuePeriodId = null)
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('a' => 'creqApproverVotes'), array('approverVoteId', 'request'));
        if (Unl_Util::isArray($requests)) {
        	if (count($requests) == 0) {
        		return new Unl_Model_Collection(__CLASS__);
        	}
            $select->where('a.request IN(?)', $requests->getId());
        } else {
            $select->where('a.request = ?', $requests->getId());
        }
        $select->where('user = ?', $user->getId());
        $select->order('time');
       
        if ($queuePeriodId) {
            $select->join(array('p' => 'creqApprovalActionsQueuePeriods'),
                          'a.approvalAction = p.approvalAction AND a.time >= p.startTime AND (a.time <= p.endTime OR p.endTime IS NULL)',
                          array());
            $select->where('p.approvalActionsQueuePeriodId = ?', $queuePeriodId);
        }

        $records = $db->query($select)->fetchAll();
        $requestVoteIds = array();
        foreach ($records as $record)
        {
            $requestVoteIds[$record['request']] = $record['approverVoteId'];
        }

        self::find($requestVoteIds);

        $requestVotes = new Unl_Model_Collection(__CLASS__);
        foreach ($requestVoteIds as $requestId => $voteId) {
            $requestVotes[$requestId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $voteId);
        }
        return $requestVotes;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
    	$modelsToUpdate = new Unl_Model_Collection(__CLASS__);

    	if (!Unl_Util::isArray($models)) {
    	    $model = $models;
    	    $models = new Unl_Model_Collection(__CLASS__);
    	    $models[$model->getId()] = $model;
    	}

    	foreach ($models as $model) {
    		if ($model->getId()) {
    			if ($model->_cleanData != $model->_data) {
    			    $modelsToUpdate[] = $model;
    			}
    		} else {
    			$modelsToInsert[] = $model;
    		}
    	}

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
    	$db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqApproverVotesUpdate '
             . 'SELECT * FROM creqApproverVotes LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqApproverVotesUpdate VALUES ';
        $sqlParts = array();
    	foreach ($models as $model) {
    		$sqlParts[] = $db->quoteInto('(?, ', $model->_data['approverVoteId'])
                        . $db->quoteInto('?, ' , $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['approvalAction'])
                        . $db->quoteInto('?, ' , $model->_data['user'])
                        . $db->quoteInto('?, ' , $model->_data['vote'])
                        . $db->quoteInto('?)'  , $model->_data['time']);
    	}
    	$sql .= implode(', ', $sqlParts);
    	$db->query($sql);

    	$sql = 'UPDATE creqApproverVotes AS a, '
    	     . '       creqApproverVotesUpdate AS b '
    	     . 'SET a.request        = b.request, '
             . '    a.approvalAction = b.approvalAction, '
             . '    a.user           = b.user, '
             . '    a.vote           = b.vote, '
             . '    a.time           = b.time '
             . 'WHERE a.approverVoteId = b.approverVoteId ';
        $db->query($sql);
        $db->query('DROP TABLE creqApproverVotesUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

    	$sql = 'INSERT INTO creqApproverVotes (request, approvalAction, user, vote, time) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['request'])
                        . $db->quoteInto('?, ' , $model->_data['approvalAction'])
                        . $db->quoteInto('?, ' , $model->_data['user'])
                        . $db->quoteInto('?, ' , $model->_data['vote'])
                        . $db->quoteInto('?)'  , $model->_data['time']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['approverVoteId'] = $lastId;
            $lastId++;
        }
    }

    static public function fetchNew()
    {
        $data = array(
            'approvalVoteId' => NULL,
            'request'        => NULL,
            'approvalAction' => NULL,
            'user'           => NULL,
            'vote'           => NULL,
            'time'           => NULL
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    public function getId()
    {
        return $this->_data['approverVoteId'];
    }

    public function getRequest()
    {
        return $this->_data['request'];
    }

    public function setRequest(Requests_RequestModel $request)
    {
        $this->_data['request'] = $request->getId();
    }

    public function getApprovalAction()
    {
        return $this->_data['approvalAction'];
    }

    public function setApprovalAction(Requests_ApprovalActionModel $approvalAction)
    {
        $this->_data['approvalAction'] = $approvalAction->getId();
    }

    public function getUser()
    {
        return $this->_data['user'];
    }

    public function setUser(Auth_UserModel $user)
    {
        $this->_data['user'] = $user->getId();
    }

    public function getVote()
    {
        return $this->_data['vote'];
    }

    public function setVote($vote)
    {
        $this->_data['vote'] = $vote;
    }

    public function getTime()
    {
        return new Zend_Date($this->_data['time']);
    }
    
    public function getUnixtime()
    {
    	return $this->_data['time'];
    }

    public function setTime(Zend_Date $time)
    {
        $this->_data['time'] = $time->getTimestamp();
    }
}