<?php

class Requests_DefaultApprovalChainsModel extends Unl_Model
{
    static public function find($id)
    {
        if (Unl_Util::isArray($id) && count($id) == 0) {
            return new Unl_Model_Collection(__CLASS__);
        }

        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqDefaultApprovalChains'));
        if (Unl_Util::isArray($id)) {
            $select->where('d.defaultApprovalChainId IN(?)', $id);
        } else {
            $select->where('d.defaultApprovalChainId = ?', $id);
        }

        $records = $db->query($select)->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objectId = $object->getId();
            $objects[$objectId] = $object;
        }

        if (Unl_Util::isArray($id)) {
            return $objects;
        } else {
            return $objects->pop();
        }
    }
    
    static public function findAll()
    {
        $db = Zend_Registry::get('db');

        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqDefaultApprovalChains'), array('defaultApprovalChainId'));

        $records = $db->query($select)->fetchAll();
        $ids = array();
        foreach ($records as $record) {
            $ids[] = $record['defaultApprovalChainId'];
        }

        return self::find($ids);
    }

    static public function findForApprovalBody($approvalBody)
    {
        $db = Zend_Registry::get('db');

        if (Unl_Util::isArray($approvalBody)) {
            if (count($approvalBody) == 0) {
                return new Unl_Model_Collection(__CLASS__);
            }
        }
        if (!$approvalBody || !$approvalBody->getId()) {
            return NULL;
        }

        $select = new Zend_Db_Select($db);
        $select->from(array('d' => 'creqDefaultApprovalChains', array('defaultApprovalChainId')));
        $select->join(array('b' => 'creqApprovalBodies'), 'd.approvalBody = b.approvalBodyId', array('approvalBodyId'));
        if (Unl_Util::isArray($approvalBody)) {
            $select->where('b.approvalBodyId IN (?)', $approvalBody->getId());
        } else {
            $select->where('b.approvalBodyId = ?', $approvalBody->getId());
        }

        $records = $select->query()->fetchAll();

        $defaultApprovalChainIds = array();
        $approvalBodyDefaults = array();
        foreach($records as $record) {
            $defaultApprovalChainIds[] = $record['defaultApprovalChainId'];
            $approvalBodyDefaults[$record['approvalBodyId']][] = $record['defaultApprovalChainId'];
        }

        self::find($defaultApprovalChainIds);

        $objects = array();
        foreach ((array) $approvalBody->getId() as $approvalBodyId) {
            $objects[$approvalBodyId] = new Unl_Model_Collection(__CLASS__);
        }
        
        foreach ($approvalBodyDefaults as $approvalBodyId => $defaultApprovalChainIds) {
            foreach ($defaultApprovalChainIds as $defaultApprovalChainId) {
            	$object = Unl_Model_Registry::getInstance()->get(__CLASS__, $defaultApprovalChainId);
            	$requestType = $object->getRequestType();
                $objects[$approvalBodyId][$requestType] = $object;
            }
        }

        if (Unl_Util::isArray($approvalBody)) {
            return $objects;
        } else {
            return array_pop($objects);
        }
    }

    /**
     * Gets a freshly created model object
     *
     * @return Requests_DefaultApprovalChainModel
     */
    static function fetchNew()
    {
        $data = array(
            'defaultApprovalChainId' => null,
            'approvalBody'           => null,
            'requestType'            => null,
            'approvalChain'          => null
        );

        $new = new self($data);
        $new->_setClean();
        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
        $modelsToUpdate = new Unl_Model_Collection(__CLASS__);

        if (!Unl_Util::isArray($models)) {
            $model = $models;
            $models = new Unl_Model_Collection(__CLASS__);
            $models[$model->getId()] = $model;
        }

        foreach ($models as $model) {
            if ($model->getId()) {
                if ($model->_cleanData != $model->_data) {
                    $modelsToUpdate[] = $model;
                }
            } else {
                $modelsToInsert[] = $model;
            }
        }

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqDefaultApprovalChainsUpdate '
             . 'SELECT * FROM creqDefaultApprovalChains LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqDefaultApprovalChainsUpdate VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['defaultApprovalChainId'])
                        . $db->quoteInto('?, ' , $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['requestType'])
                        . $db->quoteInto('?)'  , $model->_data['approvalChain']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $sql = 'UPDATE creqDefaultApprovalChains AS a, '
             . '       creqDefaultApprovalChainsUpdate AS b '
             . 'SET a.approvalBody  = b.approvalBody, '
             . '    a.requestType   = b.requestType, '
             . '    a.approvalChain = b.approvalChain '
             . 'WHERE a.defaultApprovalChainId = b.defaultApprovalChainId ';
        $db->query($sql);
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

        $sql = 'INSERT INTO creqDefaultApprovalChains (approvalBody, requestType, approvalChain) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['approvalBody'])
                        . $db->quoteInto('?, ' , $model->_data['requestType'])
                        . $db->quoteInto('?)'  , $model->_data['approvalChain']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);
    
        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['defaultApprovalChainId'] = $lastId;
            $lastId++;
        }
    }

    public function getId()
    {
        return $this->_data['defaultApprovalChainId'];
    }

    public function getApprovalBody()
    {
        return $this->_data['approvalBody'];
    }

    public function setApprovalBody(Requests_ApprovalBodyModel $approvalBody)
    {
        $this->_data['approvalBody'] = $approvalBody->getId();
    }

    public function getRequestType()
    {
        return $this->_data['requestType'];
    }

    public function setRequestType($requestType)
    {
        $this->_data['requestType'] = $requestType;
    }

    public function getApprovalChain()
    {
        return $this->_data['approvalChain'];
    }

    public function setApprovalChain(Requests_ApprovalChainModel $approvalChain)
    {
        $this->_data['approvalChain'] = $approvalChain->getId();
    }

}