<?php

class Requests_StackModel extends Unl_Model
{
    static public function find($id)
    {
        $db = Zend_Registry::get('db');
		$select = new Zend_Db_Select($db);

		$select->from(array('s' => 'creqRequestStacks'));

		if (Unl_Util::isArray($id)) {
			if (count($id) == 0) {
				return new Unl_Model_Collection(__CLASS__);
			}
			$select->where('requestStackId IN(?)', $id);
		} else {
			$select->where('requestStackId = ?', $id);
		}
        $records = $select->query()->fetchAll();
        $objects = new Unl_Model_Collection(__CLASS__);
        foreach ($records as $record) {
            $object = Unl_Model_Registry::getInstance()->getOrAdd(new self($record));
            $objects[$object->getId()] = $object;
        }

        // if we were passed an array of users, return an array for each user
        if (Unl_Util::isArray($id)) {
            return $objects;
        // otherwise, return an array of groups for the single user we were passed
        } else {
            return $objects[$id];
        }
    }

    static public function findForRequests($requests)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('r' => 'creqRequests'), array('requestId', 'stackPointer'));
        if (Unl_Util::isArray($requests)) {
            $select->where('requestId IN (?)', $requests->getId());
        } else {
            $select->where('requestId = ?', $requests->getId());
        }
        $data = $select->query()->fetchAll();

        $stackIds = array();
        foreach ($data as $row) {
            $stackIds[$row['requestId']] = $row['stackPointer'];
        }
        self::find($stackIds);

        $stacks = new Unl_Model_Collection(__CLASS__);
        foreach ($stackIds as $requestId => $stackId) {
            if (!$stackId) {
                continue;
            }
            $stacks[$requestId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $stackId);
        }
        return $stacks;
    }

    static public function findPreviousForStacks($stacks)
    {
        $db = Zend_Registry::get('db');
        $select = new Zend_Db_Select($db);
        $select->from(array('r' => 'creqRequestStacks'), array('requestStackId', 'previous'));
        if (Unl_Util::isArray($stacks)) {
            $select->where('r.requestStackId IN (?)', $stacks->getId());
        } else {
            $select->where('r.requestStackId = ?', $stacks->getId());
        }
        $data = $select->query()->fetchAll();

        $stackIds = array();
        foreach ($data as $row) {
            $stackIds[$row['requestStackId']] = $row['previous'];
        }
        self::find($stackIds);

        $stacks = new Unl_Model_Collection(__CLASS__);
        foreach ($stackIds as $stackId => $previousId) {
            $previousStack = Unl_Model_Registry::getInstance()->get(__CLASS__, $previousId);
            if (!$previousStack) {
                continue;
            }
            $stacks[$stackId] = Unl_Model_Registry::getInstance()->get(__CLASS__, $previousId);
        }
        return $stacks;
    }

    static public function fetchNew()
    {
        $data = array(
            'requestStackId' => NULL,
            'chain'          => NULL,
            'action'         => NULL,
            'previous'       => NULL
        );

        $new = new self($data);
        $new->_setClean();

        return $new;
    }

    static public function save($models)
    {
        $modelsToInsert = new Unl_Model_Collection(__CLASS__);
    	$modelsToUpdate = new Unl_Model_Collection(__CLASS__);

    	if (!Unl_Util::isArray($models)) {
    	    $model = $models;
    	    $models = new Unl_Model_Collection(__CLASS__);
    	    $models[$model->getId()] = $model;
    	}

    	foreach ($models as $model) {
    		if ($model->getId()) {
    			if ($model->_cleanData != $model->_data) {
    			    $modelsToUpdate[] = $model;
    			}
    		} else {
    			$modelsToInsert[] = $model;
    		}
    	}

        if (count($modelsToInsert) > 0) {
            self::_insert($modelsToInsert);
        }

        if (count($modelsToUpdate) > 0) {
            self::_update($modelsToUpdate);
        }

        foreach ($models as $model)
        {
            $model->_setClean();
        }
    }

    static protected function _update(Unl_Model_Collection $models)
    {
    	$db = Zend_Registry::get('db');

        $sql = 'CREATE TEMPORARY TABLE creqRequestStacksUpdate '
             . 'SELECT * FROM creqRequestStacks LIMIT 0';
        $db->query($sql);

        $sql = 'INSERT INTO creqRequestStacksUpdate VALUES ';
        $sqlParts = array();
    	foreach ($models as $model) {
    		$sqlParts[] = $db->quoteInto('(?, ', $model->_data['requestStackId'])
                        . $db->quoteInto('?, ' , $model->_data['chain'])
                        . $db->quoteInto('?, ' , $model->_data['action'])
                        . $db->quoteInto('?)'  , $model->_data['previous']);
    	}
    	$sql .= implode(', ', $sqlParts);
    	$db->query($sql);

    	$sql = 'UPDATE creqRequestStacks AS a, '
    	     . '       creqRequestStacksUpdate AS b '
    	     . 'SET a.chain    = b.chain, '
             . '    a.action   = b.action, '
             . '    a.previous = b.previous '
             . 'WHERE a.requestStackId = b.requestStackId ';
        $db->query($sql);
        $db->query('DROP TABLE creqRequestStacksUpdate');
    }

    static protected function _insert(Unl_Model_Collection $models)
    {
        $db = Zend_Registry::get('db');

    	$sql = 'INSERT INTO creqRequestStacks (chain, action, previous) VALUES ';
        $sqlParts = array();
        foreach ($models as $model) {
            $sqlParts[] = $db->quoteInto('(?, ', $model->_data['chain'])
                        . $db->quoteInto('?, ' , $model->_data['action'])
                        . $db->quoteInto('?)'  , $model->_data['previous']);
        }
        $sql .= implode(', ', $sqlParts);
        $db->query($sql);

        $lastId = $db->lastInsertId();
        foreach ($models as $model) {
            $model->_data['requestStackId'] = $lastId;
            $lastId++;
        }
    }

    public function getId()
    {
        return $this->_data['requestStackId'];
    }

    public function getChain()
    {
        return $this->_data['chain'];
    }

    public function setChain(Requests_ApprovalChainModel $chain)
    {
        $this->setChainById($chain->getId());
    }

    public function setChainById($chainId)
    {
        $this->_data['chain'] = $chainId;
    }

    public function getAction()
    {
        return $this->_data['action'];
    }

    public function setAction(Requests_ApprovalActionModel $action)
    {
        $this->_data['action'] = $action->getId();
    }

    public function getPrevious()
    {
        return $this->_data['previous'];
    }

    public function setPrevious(Requests_StackModel $previous)
    {
        $this->setPreviousById($previous->getId());
    }

    public function setPreviousById($previousId)
    {
        $this->_data['previous'] = $previousId;
    }

    public function __clone()
    {
        unset($this->_data['requestStackId']);
        if ($this->_data['previous']) {
            $newPrevious = clone self::find($this->_data['previous']);
            $this->_data['previous'] = $newPrevious->getId();
        }
        self::save($this);
    }
}