<?php

class Ucc_ReportsController extends Creq_Controller_Action
{
	protected $_internalDecisions = array('UCC Queue');
	
	public function indexAction()
	{
		$this->_redirect('/ucc/reports/monthly');
	}
	
	public function monthlyAction()
	{
        $year = $this->_getParam('year');
        $month = $this->_getParam('month');
	    
        if (!$year && !$month) {
            $startDate = new Zend_Date();
            $startDate->setYear(2008);
            $startDate->setDay(1);
            $startDate->setMonth(9);
            $startDate->setHour(0);
            $startDate->setMinute(0);
            $startDate->setSecond(0);
            
            $endDate = new Zend_Date();
            
            $this->view->startDate = $startDate;
            $this->view->endDate = $endDate;
            
            $this->render('select-date');
            return;
        }
        
        $startDate = new Zend_Date();
        $startDate->setDay(1);
        $startDate->setYear($year);
        $startDate->setMonth($month);
        $startDate->setHour(0);
        $startDate->setMinute(0);
        $startDate->setSecond(0);
        
        $endDate = clone $startDate;
        $endDate->addMonth(1);
        
        if ($this->_getParam('type') == 'summary') {
            return $this->_monthlySummaryReport($startDate, $endDate);
        } else if ($this->_getParam('type') == 'extended') {
            return $this->_monthlyExtendedReport($startDate, $endDate);
        }
	}
	
	public function annualAction()
	{
		$year = $this->_getParam('year');
		
		$startDate = new Zend_Date();
		$startDate->setYear($year);
		$startDate->setDay(1);
        $startDate->setMonth(1);
		$startDate->setHour(0);
		$startDate->setMinute(0);
		$startDate->setSecond(0);
		
		$endDate = clone $startDate;
		$endDate->addMonth(1);
		
		$data = new Unl_Model_Collection('Unl_Model_Array');
		for ($i = 0; $i < 12; $i++) {
			$data->merge($this->_getSummaryData($startDate, $endDate, 31),  TRUE);
			$data->merge($this->_getSummaryData($startDate, $endDate, 313), TRUE);
			$data->merge($this->_getSummaryData($startDate, $endDate, 325), TRUE);
			
			$startDate->addMonth(1);
			$endDate->addMonth(1);
		}
		
		echo '"Date","College","Course Code","Type","Result"'. PHP_EOL;
		
		foreach ($data as $row) {
			echo '"' . date('m-d-y', $row->getTime()) . '",'
			   . '"' . $row->getCollege() . '",'
			   . '"' . $row->getCourseCode() . '",'
			   . '"' . $row->getType() . '",'
			   . '"' . ($row->getVoteResult() == 'Approve' ? 'Approved' : 'Not Approved') . '"'
			   . PHP_EOL;
		}
		
		header('Content-Disposition:  attachment; filename="UCC ' . $year . ' Report.csv"');
		
		exit;
	}
	
	protected function _monthlySummaryReport($startDate, $endDate)
	{
        $this->view->data = $this->_getSummaryData($startDate, $endDate, 31);
        $this->view->aceData = $this->_getSummaryData($startDate, $endDate, 313);
        $this->view->aceData->merge($this->_getSummaryData($startDate, $endDate, 325), TRUE);
        $this->view->gradData = $this->_getSummaryData($startDate, $endDate, 362);
	    
        
        $session = new Zend_Session_Namespace(__CLASS__);
        $sortBy = $session->sortBy;
        if (!Unl_Util::isArray($sortBy)) {
            $sortBy = array();
        }
        if ($this->_getParam('sort')) {
            if (($key = array_search($this->_getParam('sort'), $sortBy)) !== FALSE) {
                unset($sortBy[$key]);
            }
            $sortBy[] = $this->_getParam('sort');
        }
        $session->sortBy = $sortBy;
        
        foreach ($sortBy as $sortKey) {
            $this->view->data->orderBy('get' . $sortKey);
            $this->view->aceData->orderBy('get' . $sortKey);
        }
        
        $this->render('monthly-summary');
	}
	
	protected function _getSummaryData($startDate, $endDate, $approvalActionId)
	{
	    $approvalAction = Requests_ApprovalActionModel::find($approvalActionId);
        
        $requests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction($startDate, $endDate, $approvalAction);

        $histories = Requests_ApprovalHistoryModel::findByRequest($requests);
        
        $currentCourses = Courses_CourseModel::findLatestOfRequest($requests);
        $originalCourses = Courses_CourseModel::findParentOfRequest($requests);
        $requestVotes = Requests_ApproverVoteModel::findVotesForRequests($requests, $startDate, $approvalAction);
        
        $data = new Unl_Model_Collection('Unl_Model_Array');
        foreach ($requests as $request) {
            $currentCourse = $currentCourses[$request->getId()];
            $originalCourse = $originalCourses[$request->getId()];
            $history = $histories[$request->getId()];
            $votes = $requestVotes[$request->getId()];
            unset($voteResult);

            $automatedAction = FALSE;
            foreach ($history as $aHistory) {
                if ($aHistory->getApprovalAction() != $approvalActionId) {
                    continue;
                }
                if ($startDate->getTimestamp() > $aHistory->getTime()->getTimestamp() ||
                    $endDate->getTimestamp() < $aHistory->getTime()->getTimestamp()) {
                    continue;
                }
                if (in_array($aHistory->getDecision(), $this->_internalDecisions)) {
                    continue;
                }
                $time = $aHistory->getTime();
                $voteResult = $aHistory->getDecision();
                if (!$aHistory->getUser()) {
                    $automatedAction = TRUE;
                }
            }
            $loggedVoteResult = $voteResult;
        
            // If we couldn't find a time in the history, this $approvalAction hasn't happened yet, so continue on.
            if (!$time) {
                continue;
            }
        
            // If there's no voteResult from the Facilitator, then the course was tabled and we don't
            // want it to be displayed again.  Continue on.
            if (!$voteResult && $approvalActionId == 325) {
                continue;
            }
            
            if (!$originalCourse) {
                $originalCourse = $currentCourse;
            }
            
            $difference = $currentCourse->getDifferenceSummary($originalCourse, $request);

            if ($approvalAction instanceof Requests_ApprovalActionVoteModel) {
                $voteResult = $approvalAction->tallyVotes($votes);
            }

            // If quorum wasn't met, how should this course be labeled?
            if (!$voteResult) {
                if ($time->getTimestamp() <= 1283317200) {
                    // Before 2010-09-01, it was marked Resubmit
                    $voteResult = 'Resubmit';
                } else if ($approvalActionId == 31 && $time->getTimestamp() > 1359698400) {
                    // After 2013-02-01, UCC votes will be marked with whatever was recorded in the history to allow for queue advances
                    $voteResult = $loggedVoteResult;
                } else if (!$automatedAction) {
                    // If the action wasn't automatically advanced (eg: Queue manager was used), just use the logged decision.
                    $voteResult = $loggedVoteResult;
                } else {
                    // Otherwise, its marked as Tabled.
                    $voteResult = 'Tabled';
                }
            }

            // Add an exception for the screwed up Februrary 2013 vote.
            $feb2013 = new Zend_Date(array('year' => 2013, 'month' => 2, 'day' => 10));
            if ($approvalActionId == 31 && $feb2013->isLater($startDate) && $feb2013->isEarlier($endDate)) {
                $voteResult = 'Approve';
            }
            
            $data[] = new Unl_Model_Array(array(
                'time' => $time->getTimestamp(),
                'college' => $originalCourse->getCollege(),
                'courseCode' => $originalCourse->getCourseCode(),
                'title' => $originalCourse->getTitle(),
                'difference' => $difference,
                'IS' => $originalCourse->isIntegratedStudies(),
                'esAreas' => $originalCourse->getEssentialStudiesAreas(),
                'crosslist' => $originalCourse->getCrosslistingsText(),
                'course' => $originalCourse->getCourseId(),
                'voteResult' => $voteResult,
                'type' => $request->getTypeDescription()
            ));
        }
        
        return $data;
	}
	
	protected function _monthlyExtendedReport($startDate, $endDate)
	{
	    $approvalAction = Requests_ApprovalActionModel::find(31);
        $requests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction($startDate, $endDate, $approvalAction);
        $histories = Requests_ApprovalHistoryModel::findByRequest($requests);
        
        $this->view->requests = $requests;
        $this->view->currentCourses = Courses_CourseModel::findLatestOfRequest($requests);
        $this->view->originalCourses = Courses_CourseModel::findParentOfRequest($requests);
        
        $requestVotes = Requests_ApproverVoteModel::findVotesForRequests($requests, $startDate, $approvalAction);
        $requestVoteTally = array();
        $requestVoteResults = array();
        foreach ($requestVotes as $requestId => $votes) {
            foreach ($votes as $vote) {
                if ($vote->getTime()->getTimestamp() > $endDate->getTimestamp()) {
                    continue;
                }
                if ($vote->getTime()->getTimestamp() < $startDate->getTimestamp()) {
                    continue;
                }
                $requestVoteTally[$requestId][$vote->getVote()]++;
            }
            
            $requestVoteResults[$requestId] = $approvalAction->tallyVotes($votes);
        }
        $this->view->requestVoteTally = $requestVoteTally;
        $this->view->requestVoteResults = $requestVoteResults;
        
        
        
        
        
        $approvalAction = Requests_ApprovalActionModel::find(313);
        $requests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction($startDate, $endDate, $approvalAction);
        
        $requestVotes = Requests_ApproverVoteModel::findVotesForRequests($requests, $startDate, $approvalAction);
        $requestVoteTally = array();
        $requestVoteResults = array();
        foreach ($requestVotes as $requestId => $votes) {
            foreach ($votes as $vote) {
                if ($vote->getTime()->getTimestamp() > $endDate->getTimestamp()) {
                    continue;
                }
                if ($vote->getTime()->getTimestamp() < $startDate->getTimestamp()) {
                    continue;
                }
                $requestVoteTally[$requestId][$vote->getVote()]++;
            }
            
            $requestVoteResults[$requestId] = $approvalAction->tallyVotes($votes);
        }
        
        $facilitatorApprovalActionId = 325;
        $facilitatorApprovalAction = Requests_ApprovalActionModel::find($facilitatorApprovalActionId);
        $facilitatorRequests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction($startDate, $endDate, $facilitatorApprovalAction);
        $histories = Requests_ApprovalHistoryModel::findByRequest($facilitatorRequests);

        foreach ($histories as $requestId => $history) {
            foreach ($history as $aHistory) {
                if ($aHistory->getApprovalAction() != $facilitatorApprovalActionId) {
                    continue;
                }
                if ($startDate->getTimestamp() > $aHistory->getTime()->getTimestamp() ||
                    $endDate->getTimestamp() < $aHistory->getTime()->getTimestamp()) {
                    continue;
                }
                if (in_array($aHistory->getDecision(), $this->_internalDecisions)) {
                	continue;
                }
                $requestVoteResults[$requestId] = $aHistory->getDecision() . ' (via Facilitator)';
            }
        }
        
        $requests->merge($facilitatorRequests, TRUE);
        
        $this->view->aceRequests = $requests;
        $this->view->aceCurrentCourses = Courses_CourseModel::findLatestOfRequest($requests);
        $this->view->aceOriginalCourses = Courses_CourseModel::findParentOfRequest($requests);
        
        
        $this->view->aceRequestVoteTally = $requestVoteTally;
        $this->view->aceRequestVoteResults = $requestVoteResults;
        
        
        $gradApprovalAction = Requests_ApprovalActionModel::find(362);
        $gradRequests = Requests_ApprovalHistoryModel::findRequestsWithinTimeRangeAndApprovalAction($startDate, $endDate, $gradApprovalAction);
        
        $this->view->gradRequests = $gradRequests;
        $this->view->gradCurrentCourses = Courses_CourseModel::findLatestOfRequest($gradRequests);
        $this->view->gradOriginalCourses = Courses_CourseModel::findParentOfRequest($gradRequests);
        
        
        
        $this->render('monthly-extended');
	}
	
}
