<?php

class Creq_View_Helper_BulletinEntryDiff
{
    static protected $_diffRenderer;

    public function bulletinEntryDiff(Courses_CourseModel $currentCourse, Courses_CourseModel $proposedCourse, $request, $parts = 'both')
    {
        
        // ACE
    
        $currentAce = '';
        if (count($currentCourse->getAceOutcomes()) > 0) {
            $outcomeList = array();
            foreach ($currentCourse->getAceOutcomes() as $outcome) {
                $outcomeList[] = substr($outcome['slo'], 3);
            }
            $currentAce = '(ACE ' . implode(', ', $outcomeList) . ')';
        }
    
        $proposedAce = '';
        if (count($proposedCourse->getAceOutcomes()) > 0) {
            $outcomeList = array();
            foreach ($proposedCourse->getAceOutcomes() as $outcome) {
                $outcomeList[] = substr($outcome['slo'], 3);
            }
            $proposedAce = '(ACE ' . implode(', ', $outcomeList) . ')';
        }
        
        $diffAce = $this->_getDiffText($currentAce, $proposedAce);
        
        // ES and IS
        $currentES = ($currentCourse->isEssentialStudies() ? '[ES]' : '');
        $proposedES = $currentES;
        $diffES = $this->_getDiffText($currentES, $proposedES);

        $currentIS = ($currentCourse->isIntegratedStudies() ? '[IS]' : '');
        if (in_array($request->getType(), array('AddISToCourse'))) {
            $proposedIS = '[IS]';
        } else {
            $proposedIS = $currentIS;
        }
        $diffIS = $this->_getDiffText($currentIS, $proposedIS);


        // course number with same-subject crosslistings
        $currentCourseId = $currentCourse->getCourseCode();
        $proposedCourseId = $proposedCourse->getCourseCode();
        $diffCourseId = $this->_getDiffText($currentCourseId, $proposedCourseId);
        
        $diffTitle = $this->_getDiffText($currentCourse->getTitle(), $proposedCourse->getTitle());

        // crosslistings in parenthesis
        $currentCrosslistings = $currentCourse->getCrosslistingsText();
        $proposedCrosslistings = $proposedCourse->getCrosslistingsText();
        if ($currentCrosslistings) {
        	$currentCrosslistings = '(' . $currentCrosslistings . ')';
        }
        if ($proposedCrosslistings) {
        	$proposedCrosslistings = '(' . $proposedCrosslistings . ')';
        }

        $diffCrosslistings = $this->_getDiffText($currentCrosslistings, $proposedCrosslistings);

        // credits
        $currentCredits = $currentCourse->getCreditsText();
        $proposedCredits = $proposedCourse->getCreditsText();
        
        $diffCredits = $this->_getDiffText($currentCredits, $proposedCredits);





        // campuses
        $campuses = $currentCourse->getCampuses();
        if (count($campuses) > 1 || !in_array($campuses[0], array('UNL', ''))) {
            $currentCampuses = '(' . implode(', ', $campuses) . ')';
        }
        $campuses = $proposedCourse->getCampuses();
        if (count($campuses) > 1 || !in_array($campuses[0], array('UNL', ''))) {
            $proposedCampuses = '(' . implode(', ', $campuses) . ')';
        }
        $diffCampuses = $this->_getDiffText($currentCampuses, $proposedCampuses);




        // activities
        $currentActivities = $currentCourse->getActivityText();
        $proposedActivities = $proposedCourse->getActivityText();

        $diffActivities = $this->_getDiffText($currentActivities, $proposedActivities);


        if ($currentPrereq = $currentCourse->getPrerequisite()) {
        	$currentPrereq = 'Prereq: ' . $currentPrereq;
        }
        if ($proposedPrereq = $proposedCourse->getPrerequisite()) {
        	$proposedPrereq = 'Prereq: ' . $proposedPrereq;
        }
        $diffPrereq = $this->_getDiffText($currentPrereq, $proposedPrereq);
        
        $diffNotes = $this->_getDiffText($currentCourse->getNotes(), $proposedCourse->getNotes());
        $diffDescription = $this->_getDiffText($currentCourse->getDescription(), $proposedCourse->getDescription());


        ob_start();
        ?>
        <div class="bulletinEntry">
            <div>
                <?php echo $diffAce; ?>
                <?php echo $diffES; ?>
                <?php echo $diffIS; ?>
                <b>
                <?php
                // course code with same-subject crosslistings
                echo $diffCourseId;

                ?>.
                <?php echo $diffTitle; //title ?>
                </b>
                <?php
                // crosslistings in parenthesis
                echo $diffCrosslistings;
                ?>

                <?php
                // credits
                echo $diffCredits;
                ?>

                <?php
                // campuses
                echo $diffCampuses;
                ?>

                <?php
                // activities
                echo $diffActivities;

                ?>
                <?php if($diffPrereq != '') { ?>
                <?php echo $diffPrereq; ?>
                <?php } ?>
                <em><?php echo $diffNotes; ?></em>
            </div>

            <?php echo $diffDescription; ?>


        </div>
        <?php

        $tidyConfig = array('show-body-only' => true);
        $html = ob_get_clean();
    
        if ($parts == 'current') {
            for ($start = strpos($html, '<ins>'); $start != FALSE; $start = strpos($html, '<ins>', $start)) {
                $end = strpos($html, '</ins>', $start);
                if ($end > $start) {
                    $html = substr($html, 0, $start) . substr($html, $end + 6);
                }
            }
        }
        if ($parts == 'proposed') {
            for ($start = strpos($html, '<del>'); $start != FALSE; $start = strpos($html, '<del>', $start)) {
                $end = strpos($html, '</del>', $start);
                if ($end > $start) {
                    $html = substr($html, 0, $start) . substr($html, $end + 6);
                }
            }
        }
        
        $html = tidy_repair_string($html, $tidyConfig, 'utf8');
        return $html;
    }

    protected function _getDiffText($current, $proposed)
    {
        $delim = ' ';

        Zend_Loader_Autoloader::getInstance()->registerNamespace('Horde_');
        if (!self::$_diffRenderer) {
            self::$_diffRenderer = new Horde_Text_Diff_Renderer_Inline();
        }

        $currentArray = array();
        for($tok = strtok($current, $delim); $tok !== false; $tok = strtok($delim)) {
            $currentArray[] = $tok;
        }

        $proposedArray = array();
        for($tok = strtok($proposed, $delim); $tok !== false; $tok = strtok($delim)) {
            $proposedArray[] = $tok;
        }

        $diff = new Horde_Text_Diff('auto', array($currentArray, $proposedArray));
        $diffText = ($diff->isEmpty() ? $current : self::$_diffRenderer->render($diff));

        return $diffText;
    }
}