<?php
class Creq_View_Helper_CollectionTable extends Zend_View_Helper_Abstract
{
    static protected $_uniqueIndex = 0;
    
    public function collectionTable(Unl_Model_Collection $collection,
                                    array $columns,
                                    array $searchUrlParameters = null,
                                    array $formatOptions = null,
                                    $rowClassColumn = null)
    {
        $this->view->headScript()->appendFile($this->view->baseUrl('/javascript/jquery.tablesorter.js'));
        self::$_uniqueIndex++;
        
        if (!is_array($searchUrlParameters)) {
            $searchUrlParameters = array();
        }
        if (!is_array($formatOptions)) {
            $formatOptions = array();
        }
        
        $output = '';
        
        $output .= '<table id="collectionTable' . self::$_uniqueIndex . '" class="collectionTable">' . "\n";
        $output .= "<thead>\n";
        $output .= "<tr>\n";
        foreach ($columns as $field => $title) {
            $output .= "<th>\n";
            if ($field[0] != '#') {
                /*
                $output .= '<a href="'
                        .  $this->view->url(array_merge($searchUrlParameters, array('sort' => $field)))
                        .  '">' . $title . '</a>' . "\n";
                */
                $output .= $title;
            }
            $output .= "</th>\n";
        }
        $output .= "</tr>\n";
        $output .= "</thead>\n";
        $output .= "<tbody>\n";
        foreach ($collection as $row) {
            $rowClasses = array();
            if ($rowCount++ % 2) {
                $rowClasses[] = 'even';
            }
            if ($rowClassColumn) {
                if ($row->{'get' . $rowClassColumn}()) {
                    $rowClasses[] = $row->{'get' . $rowClassColumn}();
                }
            }
            if ($rowClasses) {
                $output .= '<tr class="' . implode(' ', $rowClasses) . '">';
            } else {
                $output .= '<tr>';
            }
            foreach ($columns as $field => $title) {
                if ($field[0] == '#') {
                    $data = $title;
                } else {
                    $data = $row->{'get' . $field}();
                }
                $output .= "<td>";
                $output .= $this->_applyFormatting($data, $formatOptions[$field], $row);
                $output .= "</td>\n";
            }
            $output .= "</tr>\n";
        }
        $output .= "</tbody>\n";
        $output .= "</table>\n";
        
        $output .= '<script type="text/javascript">' . PHP_EOL
                .  '$(document).ready(function() {' . PHP_EOL
                .  '    WDN.jQuery("#collectionTable' . self::$_uniqueIndex . '").tablesorter({widgets: ["zebra"]});' . PHP_EOL
                .  '});' . PHP_EOL
                .  '</script>' . PHP_EOL;
        
        return $output;
    }
    
    protected function _getRowUrlFromSpec(Unl_Model $row, array $rowUrlSpec)
    {
        if (count($rowUrlSpec) < 1) {
             return '';
        }
        $params = array();
        $params[] = array_shift($rowUrlSpec);
        foreach ($rowUrlSpec as $field) {
             $params[] = $row->{'get' . $field}();
        }
        return call_user_func_array('sprintf', $params);
    }
    
    protected function _applyFormatting($data, $options, $row)
    {
        if (!is_array($options)) {
        
            if (is_array($data)) {
                $data = implode(', ', $data);
            }
            
            return $data;
        }
        
        if ($options['type'] == 'Zend_Date' && $data instanceof Zend_Date) {
            $data = $data->toString($options['format']);
        }
        
        if ($options['type'] == 'link') {
            $url = $this->_getRowUrlFromSpec($row, $options['urlSpec']);
            $data = '<a href="' . $url . '">' . $data . '</a>';
        }
        
        return $data;
    }
}